dojo.provide("turbo.parse.lexer");

dojo.declare("turbo.lexer.Base", null, 
	function(inText) {
		(inText)&&(this.start(inText));
	},{
	p0: 0,
	p: 0,
	ib: function() {
		return Boolean(this.m);
	},
	search: function(inRegEx) {
		var r = (inRegEx.global ? inRegEx : new RegExp(inRegEx.source, "g"));
		r.lastIndex = this.p;
		this.m = r.exec(this.s);
		// find a delimiter
		//   <token><delimeter>
		// p0<----->p
		this.p = (this.m ? this.m.index: -1);
		// d is the first character of <delimeter>, if return value is true
		return (this.ib())&&(this.d = this.s.charAt(this.p));
	},
	lookahead: function(inChars) {
		return this.s.charAt(this.p + inChars);
	},
	getToken: function() {
		return this.s.slice(this.p0, this.p);
	},
	tokenize: function(inD) {
		this.p += (inD)||(0);
	},
	pushToken: function(inKind, inD, inAllowEmpty) {
		this.tokenize(inD);
		var token = this.getToken();
		this.p0 = this.p;
		if ((!token)&&(!inAllowEmpty))
			return {};
		var mToken = { kind: inKind, token: token };
		this.r.push(mToken);
		return mToken;
	},
	tossToken: function(inD) {
		this.tokenize(inD);
		this.p0 = this.p;
	},
	start: function(inS) {
		this.s = inS;
		this.l = this.s.length;
		this.r = [ ];
		this.d = '';
		this.p = 0;
		this.analyze();
		this.finish();
	},
	finish: function() {
		this.t += this.s;
		this.pushToken("gah");
	}
});

dojo.declare("turbo.lexer.Code", turbo.lexer.Base, {
	symbols: "(){}[];=,:<>+-",
	operators: [ "++", "--", "+=", "-=", "==", "!=", "&&", "||" ],
	keywords: [ "function", "new", "return", "if", "while", "do", "break", "continue", "switch", "case" ],
	//
	doWhitespace: function() {
		this.tokenize(1);
		this.search(/\S/g);
		this.pushToken('ws');	
	},
	doEscape: function() {
		this.tokenize(2);
	},
	doLiteral: function() {
		this.tossToken(1);	
		var delim = this.d;
		var rx = new RegExp("\\" + delim + "|\\\\", "g");
		while (this.search(rx))	{
			switch (this.d) {
				case '\\':
					this.doEscape();
					break;
				default:
					this.pushToken('literal', 0, true).delimiter = delim;
					this.tossToken(1);
					return;
			}
		}
	},
	doSymbol: function() {
		this.pushToken('symbol', this.m[0].length);	
	},
	doKeyword: function() {
		this.pushToken('keyword', this.m[0].length);	
	},
	doLineComment: function() {
		this.tokenize(2);
		//if (this.search(/\r\n|[\n\r]/g))
		//	this.tokenize(1);
		if (this.search(/[\r\n]/g))
			this.tokenize(0);
		this.pushToken("comment");	
	},
	doCComment: function() {
		this.tokenize(2);
		var n = 1;
		while ((n)&&(this.search(/\/\*|\*\//g)))	{
			n += (this.d == '/' ? 1 : (this.d == '*' ? -1 : 0));
			this.tokenize(2);
		}
		this.pushToken("comment");	
	},
	process: function(inMatchers) {
		for (var i=0, f; (f=inMatchers[i]); i++)
			if (this.m[i+1]) {
				this[f].apply(this);
				return;
			}
		this.doSymbol();
	},
	buildPattern: function() {
		this.matchers = [ "doKeyword", "doLineComment", "doCComment", "doSymbol", "doLiteral", "doWhitespace" ];
		//
		var rkeys = '\\b(?:' + this.keywords.join('|') + ')\\b';
		var rsymbols = '[\\' + this.symbols.split('').join('\\') + ']';
		var rops = [];
		for (var i=0, o; (o=this.operators[i]); i++)
			rops.push('\\' + o.split('').join('\\'));
		rops = rops.join('|');
		rsymbols += '|' + rops;
		//
		var matches = [ rkeys, '\\/\\/', '\\/\\*', rsymbols, "'\"", '\\s' ];
		this.pattern = '(' + matches.join(')|(') + ')';
	},
	analyze: function() {
		this.buildPattern();
		var regex = new RegExp(this.pattern, "gi");
		while (this.search(regex)) {
			this.pushToken("identifier");
			this.process(this.matchers);
			this.pushToken("identifier");
		}
	}
});

dojo.declare("turbo.lexer.Js", turbo.lexer.Code, {});
dojo.provide("turbo.parse.parser");
dojo.require("turbo.parse.lexer");

dojo.declare("turbo.parser.Base", null, 
 	function(inTokens) {
		(inTokens)&&(this.parse(inTokens));
	},{
	i: 0,
	a: [],
	html: [],
	lastToken: {},
	next: function() {
		return this.tokens[this.i++];
	},
	setTokens: function(inTokens) {
		this.i = 0;
		this.tokens = inTokens;
	},
	pushToken: function(inT) {
		this.a.push(inT);
	},
	parse: function(inLexer) {
		this.setTokens(inLexer.r);
		return this.processTokens();
	}
});

dojo.declare("turbo.parser.Code", turbo.parser.Base, {
	pushNode: function(inCode, inKind, inToken, inDelim) {
		var token = this.a.join('') + (inDelim||'') + (inToken||'') + (inDelim||'');
		(arguments.length > 2)&&(this.a.push(inToken));
		var node = { kind: inKind, tokens: this.a, token: token};
		this.a = [ ];
		inCode.push(node);
		return node
	},
	identifier: function(inCode) {
		if (this.a.length)
			this.pushNode(inCode, 'identifier');
		return inCode;	
	},
	processArray: function(inCode, inToken) {
		this.identifier(inCode);
		this.pushNode(inCode, 'array').children = this.processTokens();
	},
	processBlock: function(inCode, inToken) {
		this.identifier(inCode);
		this.pushNode(inCode, 'block').children = this.processTokens();
	},
	processArguments: function(inCode, inToken) {
		this.identifier(inCode);
		var kind = 'association';
		if ((this.lastToken.kind == "identifier")||(this.lastToken.token=="function"))
			kind = 'argument-list';
		this.pushNode(inCode, kind).children = this.processTokens();
	},
	processTokens: function(inKind) {
		var mt, t, code = [];
		var self = this;
		while (mt = this.next()) {
			t = mt.token;
			
			if (mt.kind == "ws")
				continue;
			else if (mt.kind == "literal")
				this.pushNode(code, mt.kind, t, mt.delimiter);
			else if ((mt.kind == "comment")||(mt.kind=="keyword")) {
				this.identifier(code);
				this.pushNode(code, mt.kind, t);
			}

			else if ((t == ";")||(t == ','))
				this.identifier(code);
				
			else if (t == '[')
				this.processArray(code, t);
			else if (t == ']')
				return this.identifier(code);
			
			else if (t == '{')
				this.processBlock(code, t);
			else if (t == '}') 
				return this.identifier(code);
			
			else if (t == '(') 
				this.processArguments(code, t);
			else if (t == ')')				
				return this.identifier(code);
			
		 	else this.pushToken(t);
			this.lastToken = mt;
		}
		return code;	
	}
});

dojo.declare("turbo.parser.Text", turbo.parser.Base, {
	pushLine: function(inT) {
		(arguments.length)&&(this.a.push(inT));
		this.html.push('<span>', this.a.join("&middot;"), "</span><br />");
		this.a = [ ];
	},
	processParams: function(inToken) {
		if (this.lastToken.kind != "symbol")
			this.pushToken("[arguments|params]")
		else
			this.pushToken("[ternary op]")
		this.pushToken(inToken);	
		this.processTokens();	
	},
	processTokens: function() {
		var mt, t;
		while (mt = this.next()) {
			t = mt.token;
			if (mt.kind == "ws")
				continue;
			else if (t == ";")
				this.pushLine(t);
			else if (t == '{')
				this.pushLine(t + "<blockquote>");
			else if (t == '}')
				this.pushLine("</blockquote>" + t);
			else if (t == '(')
				this.processParams(t);
			else if (t == ')') {
				this.pushToken(t);	
				return;
			} else this.pushToken(t);
			this.lastToken = mt;
		}
		return this.html.join("");	
	}
});

dojo.declare("turbo.parser.Js", turbo.parser.Code);

dojo.provide("turbo.widgets.TurboTree");
dojo.provide("turbo.widgets.HtmlTurboTree");

dojo.require("turbo.widgets.TurboWidget");

dojo.declare("turbo.TreeNode", null, 
	function(inTree) {
		this.tree = inTree;
		this.id = this.tree.makeNodeId();
	}, {
	/*
	 	node architecture
		=================
		tree :== [domNode[treeDiv[treeNode0, treeNode1, ..., treeNodeN]]]
		treeNode :== [div
								 	[span (gutter)]
									[|- vs. `- (connector)][img (+/-) (button)]
									[span (content)]
									[div (children)
										[childNode0]
										[...]
										[childNodeN]
									]
								 ]							 	
	*/
	//
	childCount: 0,
	domNode: null,
	elements: { gutter: 0, connector:1, button: 1, content: 2, children: 3 },
	hasChildren: false,
	childrenInited: false,
	parent: null,
	hasConnectors: true,
	hasButton: true,
	canClose: true,
	// methods
	getNodeElement: function(inElement) {
		return (this.domNode ? this.domNode.childNodes[inElement] : null);
	},
	getImg: function(inPath) {
		var i = document.createElement('img');
		i.setAttribute('src', inPath);
		return i;
	},
	getGutterSrc: function(inParent) {
		return turbo.cat(this.tree.imageRoot, 
			(!inParent.hasConnectors && !inParent.hasButton ? this.tree.treeImages.none : 
			(inParent.isLastChildNode() || !inParent.hasConnectors ? this.tree.treeImages.blank : this.tree.treeImages.gutterBar)));
	},
	buildGutter: function(inParent) {
		if (inParent)	{
			var g = inParent.getNodeElement(this.elements.gutter).cloneNode(true);
			g.appendChild(this.getImg(this.getGutterSrc(inParent)));
			return g;
		} else 
			return document.createElement('span');
	},
	updateNode: function() {
		this.updateOpen();
		this.updateConnector();
	},
	updateConnector: function() {
		if (this.tree.cacheUpdates)
			return;
		var i = this.getNodeElement(this.elements.connector);
		i.style.backgroundImage = i.connector;
	},
	updateOpen: function() {
		if (this.tree.cacheUpdates)
			return;
		var c = this.getNodeElement(this.elements.children);
		turbo.showHide(c, c.show);
	},
	// 0 == this, 1 == this.parent, 2 == grandparent
	getAncestor: function(inGeneration) {
		for (var i=0, n=this; n && i<inGeneration; i++)
	    n = n.parent;
		return n;	
	},
	getGutterPartForAncestor: function(inGeneration) {
		var g = this.getNodeElement(this.elements.gutter);
    if (turbo.isGoodIndex(g.childNodes, g.childNodes.length - inGeneration))
			return g.childNodes[g.childNodes.length - inGeneration];
  },
	updateGutterPartForAncestor: function(inGeneration) {
		var a = this.getAncestor(inGeneration);
		var g = this.getGutterPartForAncestor(inGeneration);
		if (!g)
			return;
		if (g.getAttribute('src'))
			g.setAttribute('src', this.getGutterSrc(a));
	},
	updateChildGutters: function(inGeneration) {
		inGeneration = (inGeneration ? inGeneration : 1);
		for (var i=0, l= this.getChildCount(), c; i< l; i++)	{
			c = this.getChildNode(i);
			if (c) {
				c.updateGutterPartForAncestor(inGeneration);
				c.updateChildGutters(inGeneration+1);
			}
		}
	},
	buildContent: function() {
		var s = document.createElement('span');
		s.innerHTML = 'Node';
		s.className = this.tree.classTag + '-content';
		s.style.cursor = 'default';
		return s;
	},
	buildDomNode: function(inParent) {
		var node = document.createElement('div');
		// gutter
		node.appendChild(this.buildGutter(inParent));
		// button / connector
		var b = document.createElement('img');
		node.appendChild(b);
		// content
		node.appendChild(this.buildContent());
		// children
		var c = document.createElement('div');
		c.show = true;
		node.appendChild(c);
		// node attributes
		node.setAttribute('turboTreeNode', 'true');
		node.id = this.id;
		this.domNode = node;
		return node;
	},
	setConnector: function(inFirstRoot, inLast) {
		var i = this.getNodeElement(this.elements.connector);
		var b;
		if (!this.hasConnectors)
			b = this.tree.treeImages.none;
		else if (inFirstRoot)
			b = (inLast ? this.tree.treeImages.connectorRoot : this.tree.treeImages.connectorRootFirst);
		else
			b = (inLast ? this.tree.treeImages.connectorLast : this.tree.treeImages.connectorLeaf);
		i.connector = "url(" + this.tree.imageRoot + b + ")";
		this.updateConnector();
	},
	isFirstRootNode: function() {
		return (this.isRootNode() && this.domNode == this.domNode.parentNode.firstChild);
	},
	isRootNode: function() {
		return (this.domNode.parentNode == this.tree.treeDiv);
	},
	isLastChildNode: function() {
		return (this.domNode.parentNode && this.domNode.parentNode.lastChild == this.domNode);
	},
	selectConnector: function() {
		this.setConnector(this.isFirstRootNode(), this.isLastChildNode());
	},
	getButton: function() {
		return this.getNodeElement(this.elements.button);
	},
	getButtonSrcFile: function(inHasChildren, inOpen) {
		return (!this.hasButton ? this.tree.treeImages.none : 
			(!inHasChildren ? this.tree.treeImages.blank : 
			(inOpen ? this.tree.treeImages.buttonOpen : this.tree.treeImages.buttonClosed)));
	},
	getButtonSrc: function(inHasChildren, inOpen) {
		return turbo.cat(this.tree.imageRoot, this.getButtonSrcFile(inHasChildren, inOpen));
	},
	setButton: function(inHasChildren, inOpen) {
		var f = this.getButtonSrcFile(inHasChildren, inOpen);
		var s = this.getButton();
		var src = s.getAttribute('src');
		if (src == null || (src.indexOf && src.indexOf(f) == -1))
			s.setAttribute('src', this.getButtonSrc(inHasChildren, inOpen));
	},
	selectButton: function() {
		this.setButton(this.hasChildren || (this.getChildCount() > 0), this.getOpen());
	},
	getOpen: function() {
		var n = this.getNodeElement(this.elements.children);
		return (n && (n.show !== null ? n.show : (n.style.display == '')));
	},
	canOpen: function() {
		return (this.getChildCount() > 0);	
	},
	setOpen: function(inOpen) {
		inOpen = (this.canClose ? inOpen : true);
		var n = this.getNodeElement(this.elements.children);
		var wasOpen = (n && (n.show !== null ? n.show : (n.style.display == '')));
		if (n && wasOpen != inOpen) {
			n.show = inOpen;
			this.updateOpen();
		}
		if (inOpen && !this.childrenInited && this.hasChildren)
			this.tree.initChildren(this);
		// close node if no children after initialization
		if (inOpen && !this.canOpen() && this.canClose)
			this.setOpen(false);
		if (wasOpen != inOpen)	
			this.selectButton();
	},
	toggleNode: function() {
		this.setOpen(!this.getOpen());
	},
	setContent: function(inHtml) {
		this.getNodeElement(this.elements.content).innerHTML = inHtml;
	},
	appendTo: function(inParent, inAtChild) {
		this.parent = inParent;
		var p = (inParent ? inParent.getNodeElement(this.elements.children) : this.tree.treeDiv);
		this.index = p.childNodes.length;
		if (inAtChild && inAtChild.domNode)
			p.insertBefore(this.domNode, inAtChild.domNode);
		else
			p.appendChild(this.domNode);
		if (this.parent)
			this.parent.selectButton();
		var n = this.tree.nodeFromDomNode(this.domNode.previousSibling);
		if (n) {
			n.selectConnector();
			// when a node is added, must update the gutters of all the children of its previous sibling.
			n.updateChildGutters();
		}
		this.selectConnector();
		this.updateNode();
	},
	setSelected: function(inSelect) {
		if (inSelect === undefined)
			inSelect = true;
		var c = this.tree.classTag + '-content'			
		var content = this.getNodeElement(this.elements.content);
		if (content)
			content.className = c + (inSelect ? ' ' + c + '-selected' : '');
	},
	getChildCount: function() {
		var c = this.getNodeElement(this.elements.children);
		return (c && c.childNodes.length);
	},
	getChildNode: function(inIndex) {
		var childDomNodes = this.getNodeElement(this.elements.children).childNodes;
		if (turbo.isGoodIndex(childDomNodes, inIndex))
			return this.tree.nodeFromDomNode(childDomNodes[inIndex]);
	},
	_deleteChildNodes: function(inNode) {
		var childrenDomNodes = inNode.getNodeElement(this.elements.children).childNodes;
		for (var i=0, l=childrenDomNodes.length; i < l; i++) {
			var el = childrenDomNodes[i];
			this._deleteChildNodes(this.tree.nodeFromDomNode(el));
			delete this.tree.nodeList[el.id];
		}	
	},
	removeChildren: function() {
		this._deleteChildNodes(this);
		this.domNode.replaceChild(document.createElement('div'), this.getNodeElement(this.elements.children));
		this.childCount = 0;
		this.hasChildren = false;
		this.setOpen(false);
		this.selectButton();
	},
	refreshChildren: function() {
		if (this.getChildCount() <= 0)
			return;
		this.removeChildren();
		this.childrenInited = false;
		this.tree.initChildren(this);
		this.setOpen(true);
	},
	// checks of node has self initializing data for child of index
	canSelfInitChild: function(inIndex) {
		return (this.canSelfInit && this.children && this.children[inIndex]);
	},
	childRemoved: function() {
		this.childCount = Math.max(0, this.childCount-1);
		this.hasChildren = (this.childCount > 0);
		this.setOpen(this.getOpen());
		this.selectButton();
		this.selectConnector();
	},
	nextNodeRemoved: function() {
		this.selectConnector();
		this.selectButton();
		this.updateChildGutters();
	},
	prevNodeRemoved: function() {
		this.selectConnector();
	}});

dojo.widget.defineWidget("dojo.widget.html.TurboTree", dojo.widget.html.TurboWidget, 
	function() {
		this.imageRoot = dojo.uri.dojoUri(this.themeRoot + 'default/images/');
		this.nodeList = [ ];
	}, {
	templateString: '<div><div dojoattachpoint="treeDiv" class="turbo-tree-scroller"></div></div>',
	styleRoot: 'TurboTree',
	classTag: 'turbo-tree',
	// DOM nodes
	treeDiv: null,
	// properties
	treeImages: {
		none: 'tree_none.gif',
		blank: 'tree_blank.gif',
		gutterBar: 'tree_bar.gif',
		connectorRoot: 'tree_root.gif',
		connectorRootFirst: 'tree_root_first.gif',
		connectorLast: 'tree_last_leaf.gif', 
		connectorLeaf: 'tree_leaf.gif',
		buttonOpen: 'tree_open.gif', 
		buttonClosed: 'tree_closed.gif'
	},
	hasRootConnectors: true,
	nodeId: 0,
	nodes: '',
	selected: null,
	// events
	onInitNode: function() { },
	onInitChildren: function() { },
	onCanUnselect: function(inNode) { },
	onCanSelect: function(inNode) { },
	onSelect: function(inNode) { },
	onUnselect: function(inNode) { },
	onOpen: function(inNode) { },
	onClose: function(inNode) { },
	// methods
	fillInTemplate: function(inArgs, inFrag) {
		// prevents the aligner from forcing a value before the stylesheet is ready
		// this is supposed to be fixed now
		//this.domNode.style.overflow = "auto";
		this.setTheme('');
		dojo.event.connect(this.domNode, 'onclick', this, 'domClick');
		dojo.event.connect(this.domNode, 'ondblclick', this, 'domDblClick');
		//
		// disable text selection so no conflict with node selection
		if (dojo.render.html.ie)
			this.domNode.onselectstart = function() { return false; }; 
		else	
			dojo.html.disableSelection(this.domNode);
		if (this.extraArgs['onnodeselected'])
			this.onNodeSelected = eval(this.extraArgs['onnodeselected']);
		//	
		if (dojo.lang.isString(inArgs['treeImages']))	
			this.treeImages = eval(inArgs['treeImages']);
		if (dojo.lang.isString(inArgs['nodes']))	
		this.nodes = eval(inArgs['nodes']);
		if (this.nodes)
			this.buildNodes(this.nodes);
	},
	styleChanged: function() {
		this.setStyledClass(this.domNode, '');
	},
	// utility
	makeNodeId: function() {
		return this.widgetId + ':' + this.nodeId++;
	},
	nodeFromDomNode: function(inDomNode) {
		return (inDomNode && inDomNode.id ? this.nodeList[inDomNode.id] : null);
	},
	// tree operations
	clear: function() {
		this.nodeId = 0;
		this.nodeList = [];
		this.treeDiv.innerHTML = '';
	},
	setRootCount: function(inCount) {
		this.clear();	
		for (var i=0; i < inCount; i++)		
			this.newNode(null);
	},
	setRootNodeConnectors: function(inNode) {
		if (!this.hasRootConnectors) {
			inNode.hasConnectors = false;
			inNode.hasButton = false;
		}		
	},
	newTreeNode: function(inParent) {
		var n = new turbo.TreeNode(this);
		this.nodeList[n.id] = n;
		if (inParent == null)
			this.setRootNodeConnectors(n);
		n.buildDomNode(inParent);
		return n;
	},
	// called only if inNode has initializing data
	initNodeBySelf: function(inNode) {
		inNode.childCount = (inNode.children ? inNode.children.length : 0);
		inNode.hasChildren = (inNode.childCount > 0 || inNode.onInitChildren);
		inNode.setContent(inNode.content ? inNode.content : (inNode.name ? inNode.name : ''));
		inNode.setOpen(inNode.initOpen !== undefined ? inNode.initOpen : true); // open by default
	},
	newNode: function(inParent, inInitData) {
		if (!dojo.lang.isArray(inInitData))
			 	return this._newNode(inParent, inInitData);
			else
				for (var i=0,l=inInitData.length; i<l; i++)
					this._newNode(inParent, inInitData[i]);
	},
	// node can optionally be created with a set of initializing data (a la TurboTree1 format)
	_newNode: function(inParent, inInitData) {
		var n = this.newTreeNode(inParent);
		n.appendTo(inParent);
		return this.initNode(n, inInitData);
	},
	initNode: function(inNode, inInitData) {
		if (inInitData)	{
			inNode.canSelfInit = true;
			if (dojo.lang.isObject(inInitData))
				inNode = turbo.swiss(inInitData, inNode);
			else
				inNode.name = inInitData;
		}
		if (!inNode.canSelfInit)
			this.onInitNode(inNode);
		else if (inNode.onInitNode)
			inNode.onInitNode();
		if (inNode.canSelfInit)
			this.initNodeBySelf(inNode);
		else	
			inNode.setOpen(inNode.isRootNode());
		inNode.selectConnector();
		return inNode;
	},
	addNodeBefore: function(inSibling, inInitData) {
		var p = (inSibling && inSibling.parent ? inSibling.parent : null);
		var n = this.newTreeNode(p);
		n.appendTo(p, inSibling);
		this.initNode(n, inInitData);
		if (!p)
			inSibling.selectConnector();		
		return n;
	},
	addNodeAfter: function(inSibling, inInitData) {
		var p = (inSibling && inSibling.parent ? inSibling.parent : null);
		var n = this.newTreeNode(p);
		var s = this.getNextSiblingNode(inSibling);
		n.appendTo(p, (s && s.parent == p ? s : null));
		this.initNode(n, inInitData);
		return n;
	},
	initChildren: function(inNode) {
		if (!inNode.canSelfInit)
			this.onInitChildren(inNode);
		else if (inNode.onInitChildren)
			inNode.onInitChildren(inNode);
		var c = inNode.childCount; 
		for (var i=0; i < c; i++)
			this.newNode(inNode, (inNode.canSelfInitChild(i) ? inNode.children[i] : null));		
		if (inNode.children)
			inNode.children = null;
		inNode.childrenInited = true;
		inNode.hasChildren = (c > 0);
	},
	selectNode: function(inNode) {
		if (inNode && this.onCanSelect(inNode) === false)
			return false;
		if (this.selected) {
			if (this.onCanUnselect(this.selected) === false)
				return false;
			this.onUnselect(this.selected);
			this.selected.setSelected(false);
		}
		this.selected = inNode;
		if (this.selected) 
			this.selected.setSelected(true);
		return true;	
	},
	eventSelectNode: function(inNode) {
		if (!this.selectNode(inNode))
			return;
		// for bc
		if (this.onNodeSelected)
			this.onNodeSelected(this.selected);
		else	
			this.onSelect(this.selected);
	},
	forEach: function(inMap, inEvery) {
		var nodes = [ ], n = null;
		while (n = this.getNextNode(n))	{
			if (!inMap(n))
				continue; 
			if (!inEvery)
				return n;
			nodes.push(n);
		}
		return (inEvery ? nodes : null);
	},
	getNodeByValue: function(inValue, inProperty) {
		return this.getNodesByValue(inValue, inProperty, true);
	},
	getNodesByValue: function(inValue, inProperty, inFirst) {
		return this.forEach(function(inNode) {
			return (inNode[inProperty] && inNode[inProperty] == inValue);
		}, !Boolean(inFirst));
	},
	//
	// events
	isTreeNode: function(inNode) {
		return inNode && inNode.getAttribute && inNode.getAttribute('turboTreeNode');
	},
	toggleNode: function(inNode) {
		inNode.toggleNode();
		if (inNode.getOpen())
			this.onOpen(inNode);
		else
			this.onClose(inNode);
	},
	nodeClick: function(inNode, inEvent) {
		if (!inNode)
			return;
		if (inNode.getButton() == inEvent.target) 
			this.toggleNode(inNode);
		this.eventSelectNode(inNode);
	},
	nodeDblClick: function(inNode, inEvent) {
		if (!inNode)
			return;
		this.toggleNode(inNode);
		this.eventSelectNode(inNode);
	},
	domClick: function(inEvent) {
		var n = inEvent.target;
		while (n && !this.isTreeNode(n))
			n = n.parentNode;
		this.nodeClick((n ? this.nodeFromDomNode(n) : null), inEvent);
	},
	domDblClick: function(inEvent) {
		var n = inEvent.target;
		while (n && !this.isTreeNode(n))
			n = n.parentNode;
		this.nodeDblClick((n ? this.nodeFromDomNode(n) : null), inEvent);
	},
	getRootNodeByIndex: function(inIndex) {
		var childDomNodes = this.treeDiv.childNodes;
		if (turbo.isGoodIndex(childDomNodes, inIndex))
			return this.nodeFromDomNode(childDomNodes[inIndex]);
	},
	getNodeByIndex: function() {
	  if (!(arguments.length > 0)) 
    	return null; 
    var node = this.getRootNodeByIndex(arguments[0]);
		for (var i=1, l=arguments.length; i < l; i++)
			node = node.getChildNode(arguments[i]);
		return node;
	},
	getPrevSiblingNode: function(inNode) {
		return (inNode.domNode.previousSibling ? this.nodeFromDomNode(inNode.domNode.previousSibling) : null);
	},
	getNextSiblingNode: function(inNode) {
		return (inNode.domNode.nextSibling ? this.nodeFromDomNode(inNode.domNode.nextSibling) : null);
	},
	getNextNode: function(inNode) {
		// first node
		if (!inNode)
			return this.getRootNodeByIndex(0);
		// first child	
		if (inNode.getChildCount() > 0)
			return inNode.getChildNode(0);
		// next sibling	
		if (!inNode.isLastChildNode())
			return this.getNextSiblingNode(inNode);
		// parent's next sibling	
		while (inNode.parent && inNode.parent.isLastChildNode())
			inNode = inNode.parent;
		return (inNode.parent ? this.getNextSiblingNode(inNode.parent) : null);
	},
	removeNode: function(inNode) {
		this.selected.setSelected(false);
		this.selected = null;
		var parent = inNode.parent;
		var prevNode = this.getPrevSiblingNode(inNode);
		var nextNode = this.getNextSiblingNode(inNode);
		inNode.removeChildren();
		delete this.nodeList[inNode.id];
		turbo.remove(inNode.domNode);
		// cleanup
		if (parent)
			parent.childRemoved();
		if (prevNode)
			prevNode.nextNodeRemoved();
		if (nextNode)
			nextNode.prevNodeRemoved();
	},
	buildNodes: function(inNodeData) {
		this.newNode(null, inNodeData);
	}
});
dojo.provide("turbo.widgets.TurboLayout.Layout");
dojo.require("turbo.widgets.TurboWidget");

dojo.declare("turbo.mixin.aligner", null, {
	descentAttribute: 'layoutchildren',
	getAttribute: function(inNode, inAttribute) {
		return (inNode.getAttribute)&&(inNode.getAttribute(inAttribute));
	},
	shouldAlign: function(inNode) {
		return (inNode.nodeType==1)&&(dojo.html.isShowing(inNode)); 
	},
	getAlignment: function(inElement) {
		return inElement.getAttribute("layout") || 'top';
	},
	aligned: function(inElement, inAlign) {
		return this.getAlignment(inElement) == inAlign; 
	},
	listAlignedChildren: function(inNode) {
		var result = { none:[], top:[], left:[], client:[], right:[], bottom:[] };
		for (var node=inNode.firstChild; node; node=node.nextSibling)	{
			if (this.shouldAlign(node))
				result[this.getAlignment(node)].push(node);
		}
		return result;
	},
	setStylePosition: function(inElement, inPosition)	{
		with (inElement.style)
			if (position != inPosition)
				position = inPosition;
	},
	normalizeAlignedElement: function(inElement) {
		this.setStylePosition(inElement, 'absolute');
	},
	alignElement: function(inElement, inL, inT, inW, inH) {
		this.normalizeAlignedElement(inElement);
		turbo.setBounds(inElement, inL, inT, inW, inH);
		//console.log("alignElement: ", arguments);
	},
	alignTop: function(inNodes, inBox)	{
		for (var i=0, n; n=inNodes[i]; inBox.t+=n.offsetHeight, i++)
			this.alignElement(n, inBox.l, inBox.t, inBox.w);
	},
	alignBottom: function(inNodes, inBox)	{
		for (var i=inNodes.length-1, b=inBox.h, n; n=inNodes[i]; b-=n.offsetHeight, i--)
			this.alignElement(n, inBox.l, b - n.offsetHeight, inBox.w);
		inBox.h = b - inBox.t;			
	},
	alignLeft: function(inNodes, inBox)	{
		for (var i=0, n; n=inNodes[i]; inBox.l+=n.offsetWidth, i++)
			this.alignElement(n, inBox.l, inBox.t, -1, inBox.h);
	},
	alignRight: function(inNodes, inBox)	{
		for (var i=inNodes.length-1, r=inBox.w, n; n=inNodes[i]; r-=n.offsetWidth, i--)
			this.alignElement(n, r-n.offsetWidth, inBox.t, -1, inBox.h);
		inBox.w = r - inBox.l;			
	},
	alignClient: function(inNodes, inBox)	{
		for (var i=0, n=inNodes[i]; n; n = null)
			this.alignElement(n, inBox.l, inBox.t, inBox.w, inBox.h);
	},
	getBox: function(inNode) {
		var box = turbo.getContentSize(inNode);
		box.t = dojo.html.getPixelValue(inNode, 'padding-top', true);
		box.l = dojo.html.getPixelValue(inNode, 'padding-left', true);
		return box;
	},
	alignGrandchildren: function(inRoot, inGenerations) {
		//if (inGenerations > 0)
		//	console.log("alignGrandchildren: ", inRoot, inGenerations);
		if (--inGenerations != -1)
			for (var node=inRoot.firstChild; node; node=node.nextSibling)	{
				if (this.getAttribute(node, this.descentAttribute))
				//if (this.getAlignment(node))
					this.alignNode(node, inGenerations);
			}
	},
	alignChildren: function(inBranch, inGenerations)	{
		//console.log("alignChildren: ", inBranch, inGenerations);
		var box = this.getBox(inBranch);
		//console.log("alignChildren: %1.o %o", box, inBranch);
		var alignables = this.listAlignedChildren(inBranch);
		this.alignTop(alignables.top, box);
		this.alignBottom(alignables.bottom, box);
		this.alignLeft(alignables.left, box);
		this.alignRight(alignables.right, box);
		this.alignClient(alignables.client, box);
		this.alignGrandchildren(inBranch, inGenerations);
	},
	alignNode: function(inNode, inGenerations) {
		if (this.shouldAlign(inNode))
			this.alignChildren(inNode, inGenerations);
	}
});

dojo.widget.defineWidget("dojo.widget.TurboLayout", [ dojo.widget.html.TurboWidget, turbo.mixin.aligner ], {
	// useful properties
	classTag: 'turbo-layout',
	themeable: false,
	// methods
	fillInTemplate: function(inArgs, inFrag) {
		dojo.event.topic.subscribe("turboresize", this, "reflow");
		this.createChildWidgets(inFrag);
		this.domNode.style.cssText += this.style + "overflow: hidden; position: relative;";
	},
	postCreate: function() {
		var node = this.domNode;
		turbo.deferUntil(
			function() { return Boolean(node.offsetHeight); }, 
			turbo.bind(this, 'reflow')
		);
	},
	isLayoutChild: function() { 
		var n = this.domNode.parentNode;
		return (this.getAttribute(n, "dojotype")=="TurboLayout") || Boolean(this.getAttribute(n, this.descentAttribute));
 	},
	fitToParent: function() {
		if ((!this.isLayoutChild())&&(this.layout=="client")) {
			this.normalizeAlignedElement(this.domNode);
			var box = this.getBox(this.domNode.parentNode);
			turbo.setBounds(this.domNode, box.l, box.t, box.w, box.h);
		}
	},
	align: function() {
		this.alignNode(this.domNode, this.generations);
	},
	reflow: function() {
		this.defaultReflow();
	},
	defaultReflow: function() {
		this.fitToParent();
		this.align();
	},
	dummy: 0
});
dojo.provide("turbo.widgets.TurboFloater");
dojo.provide("turbo.widgets.HtmlTurboFloater");

dojo.require("turbo.lib.scrim");
dojo.require("turbo.lib.drag");
dojo.require("turbo.widgets.TurboWidget");

dojo.declare("turbo.stack", null, 
	function(inItems) {
		this.items = (inItems ? inItems : [ ]);
	}, {
		add: function(inItem) {
			if (inItem != this.getTop())
				this.items.push(inItem);
		},
		remove: function(inItem) {
			this.removeIndex(this.getIndex(inItem));
		},
		removeIndex: function(inIndex) {
			if (turbo.isGoodIndex(this.items, inIndex))
				this.items.splice(inIndex, 1);
		},
		get: function(inIndex) {
			if (turbo.isGoodIndex(this.items, inIndex))
				return this.items[inIndex];
		},
		getTop: function() {
			return this.items[this.count()-1];
		},
		moveToTop: function(inItem) {
			if (inItem == this.getTop())
				return;
			var i = this.getIndex(inItem);
			if (i < 0)
				return;
			this.removeIndex(i);
			this.add(inItem);	
		},
		count: function() {
			return this.items.length;
		},
		getIndex: function(inItem) {
			for (var i = 0, l=this.count(); i<l; i++)
				if (inItem == this.get(i))
					return i;
			return -1;
		},
		map: function(inStart, inEnd, inMethodName) {
			inStart = (turbo.isGoodIndex(this.items, inStart) ? inStart : this.count()-1);
			inEnd = (turbo.isGoodIndex(this.items, inEnd) ? inEnd : 0);
			var lt = (inStart <= inEnd);
			for (var i=inStart, o; (lt ? i<=inEnd : i>=inEnd); (lt ? i++ : i--)) {
				o = this.get(i);
				if (o && o[inMethodName])
					o[inMethodName]();
			}
		}
	}
);	
		
turbo.floaters = new turbo.stack();

// FIXME: put in turbo.js
// node must be in dom
turbo.getNodeRect = function(inNode) {
	var toggleShow = !turbo.isShowing(inNode);
	if (toggleShow)
		turbo.show(inNode);
	var nt = turbo.getAbsoluteY(inNode, true);
	var nl = turbo.getAbsoluteX(inNode, true);
	var box = dojo.html.getBorderBox(inNode);
	var nodeRect = {
		t: nt,
		r: nl + box.width,
		b: nt + box.height,
		l: nl
	};
	if (toggleShow)
		turbo.hide(inNode);
	return nodeRect;	
}

turbo.measureNode = function(inNode) { 
 	var domNode = document.createElement("div");
 	with (domNode.style) {
		top = "0px";
		left = "0px";
		position = "absolute";
		visibility = "hidden";
	}
	turbo.append(turbo.$(inNode), domNode);
	turbo.append(domNode);
 	var result = dojo.html.getContentBox(domNode);
	turbo.remove(domNode);
	turbo.remove(inNode);
	return result;
};

turbo.measureHtml = function(inHtml) {
	 var n = document.createElement('div');
	 n.innerHTML = inHtml;
	 return turbo.measureNode(n);
}


// dragging
turbo.floater = {};

dojo.declare("turbo.floater.drag", null, 
	function(inNode, inOnDrag, inOnStartDrag, inOnEndDrag, inSnap) {
		this.domNode = inNode;
		turbo.tie(this, '_onDrag', '_onEndDrag', '_onStartDrag');
		if (inOnDrag)
			this.onDrag = inOnDrag;
		if (inOnStartDrag)
			this.onStartDrag = inOnStartDrag;
		if (inOnEndDrag)
			this.onEndDrag = inOnEndDrag;	
		if (inSnap)
			this.snap = inSnap;
		this.connectNode();
	}, {
	snap: null,
	onStartDrag: function(inEvent) {},
	onDrag: function(inEvent) {},
	onEndDrag: function() {},
	connectNode: function() {
		dojo.event.kwConnect({
			srcObj:     this.domNode, 
			srcFunc:    'onmousedown', 
			targetObj:  this, 
			targetFunc: '_beginDrag',
			once:       true
		});
	},
	_beginDrag: function(inEvent) {
		turbo.drag.start(this.domNode, this._onDrag, this._onEndDrag, inEvent, this._onStartDrag);
	},
	setDragDelta: function(inEvent) {
		var p = dojo.html.getAbsolutePosition(this.domNode, true);
		this.dragDelta = {left: inEvent.pageX - p.x, top: inEvent.pageY - p.y};
	},
	_onStartDrag: function(inEvent) {
		this.domNode.style.cursor = 'move';
		this.setDragDelta(inEvent);
		this.parentPosition = dojo.html.getAbsolutePosition(this.domNode.parentNode, true);
		dojo.html.disableSelection(this.domNode);
		this.processEvent(inEvent);
		this.onStartDrag(this.dragEvent);
		this.lastDragEvent = this.dragEvent;		
	},
	isOkDrag: function() {
		return (this.dragEvent && 
			(!this.lastDragEvent || this.lastDragEvent.domNodeX != this.dragEvent.domNodeX || this.lastDragEvent.domNodeY != this.dragEvent.domNodeY));
	},
	setSnappedDrag: function() {
		if (!this.lastDragEvent || !this.dragEvent)
			return;
		var x = Math.round(this.dragEvent.domNodeX / this.snap.x) * this.snap.x;
		var y = Math.round(this.dragEvent.domNodeY / this.snap.y) * this.snap.y;
		this.dragEvent.domNodeX = x;
		this.dragEvent.domNodeY = y;
	},
	processEvent: function(inEvent) {
		this.dragEvent = inEvent;
		this.dragEvent.dragX = this.dragEvent.pageX - this.parentPosition.x;
		this.dragEvent.dragY = this.dragEvent.pageY - this.parentPosition.y;
		this.dragEvent.domNodeX = this.dragEvent.dragX - this.dragDelta.left;
		this.dragEvent.domNodeY = this.dragEvent.dragY - this.dragDelta.top;
		if (this.snap)
			this.setSnappedDrag();
	},
	_onDrag: function(inEvent) {
		this.processEvent(inEvent);
		this.onDrag(this.dragEvent);
		this.lastDragEvent = this.dragEvent;	
	},
	_onEndDrag: function() { 
		dojo.html.enableSelection(this.domNode);
		this.domNode.style.cursor = 'default';
		this.lastDragEvent = null;
		this.dragEvent = null;
		this.onEndDrag();
 	}
});

dojo.widget.defineWidget("dojo.widget.html.TurboFloater", dojo.widget.html.TurboWidget, 
	function() {
		this.stack = turbo.floaters;
		this.windowBounds = {w: 0, h: 0};
		// internal storage
		this._dimensions = {w: 0, h: 0};
		this._minDimensions = {w: 0, h: 0};
		this._sizing = {};
		this.snap = null;
	}, {		
	templateString: '<div></div>',
	// useful properties
	classTag: 'turbo-floater',
	windowMargin: 8,
	showTimeoutMs: 5000,
	_showing: false,
	applyScrimToParent: false,
	bgOpacity: 0,
	bgColor: '',
	useScrim: true,
	autoSize: true,
	// set of: trbl
	sizeable: '',
	dragable: '',	
	sizingNubWidth: 4,
	sizingDelta: 4,
	baseZ: 11,	
	left: 0,
	top: 0,
	height: 0,
	width: 0,
	// events
	onShow: turbo.nop,
	onHide: turbo.nop,
	onBeforeShow: turbo.nop,
	onBeforeHide: turbo.nop,
	onStartDrag: turbo.nop,
	onDrag: turbo.nop,
	onEndDrag: turbo.nop,
	// methods
	fillInTemplate: function(inArgs, inFrag) {
		this.installChildren(inFrag);
		dojo.event.topic.subscribe('turboresize', this, 'resize');
		dojo.event.browser.addListener(this.domNode, 'onmousedown', turbo.bind(this, this.doMouseDown)) ;
		if (this.sizeable)
			dojo.event.browser.addListener(this.domNode, 'onmousemove', turbo.bind(this, this.doMouseMove));
	},
	styleChanged: function() {
		this.setStyledClass(this.domNode, '');
		this.domNode.className = this.domNode.className + (this.className ? ' ' + this.className : '');
	},
	postCreate: function() {
		this.setDimensions(this.width, this.height);
		this.setMinDimensions(this.minWidth, this.minHeight);
		this.initNodes();		
		if (this.dragable || this.sizeable)
			this.initDragSize();
	},
	// dragging and sizing
	initDragSize: function() {
		// IE needs slight defer to process nodes
		turbo.defer(this, this._initDragSize, 0);
	},
	_initDragSize: function() {
		this.dragger = new turbo.floater.drag(this.domNode, turbo.bind(this, this.doDrag), turbo.bind(this, this.doStartDrag), turbo.bind(this, this.doEndDrag), this.snap);
	},
	processEvent: function(inEvent) {
		if (inEvent.dragX && inEvent.dragY)
			return;
		var p = dojo.html.getAbsolutePosition(this.domNode.parentNode, true);	
		inEvent.dragX = inEvent.pageX - p.x;
		inEvent.dragY = inEvent.pageY - p.y;
	},
	overTopResizeArea: function(inEvent) {
		return Math.abs(this.top - inEvent.dragY) < this.sizingNubWidth;
	},
	overBottomResizeArea: function(inEvent) {
		return Math.abs(this.top + this.getHeight() - inEvent.dragY) < this.sizingNubWidth;
	},
	overLeftResizeArea: function(inEvent) {
		return Math.abs(this.left - inEvent.dragX) < this.sizingNubWidth;
	},
	overRightResizeArea: function(inEvent) {
		return Math.abs(this.left + this.getWidth() - inEvent.dragX) < this.sizingNubWidth;
	},
	charInString: function(inString, inChar) {
		return (inString.indexOf(inChar) != -1);
	},
	canSizeRight: function() {
		return this.charInString(this.sizeable, 'r');
	},
	canSizeLeft: function() {
		return this.charInString(this.sizeable, 'l');
	},
	canSizeTop: function() {
		return this.charInString(this.sizeable, 't');
	},
	canSizeBottom: function() {
		return this.charInString(this.sizeable, 'b');
	},
	setSizing: function(inEvent) {
		this._sizing.r = this.overRightResizeArea(inEvent) && this.canSizeRight();
		this._sizing.l = this.overLeftResizeArea(inEvent) && this.canSizeLeft();
		this._sizing.t = this.overTopResizeArea(inEvent) && this.canSizeTop();
		this._sizing.b = this.overBottomResizeArea(inEvent) && this.canSizeBottom();
	},
	dragSizeRight: function(inEvent) {
		var w = inEvent.dragX - this.left + this.sizingDelta;
		w = Math.min(w, this.windowBounds.w - this.left)
		this.setDimensions(w, null);		
	},
	dragSizeLeft: function(inEvent) {
		var l = Math.max(this.snapX(inEvent.dragX - this.sizingDelta), 0);
		var w = this.getWidth() + this.left - l;
		this.setDimensions(w, null);
		this.setPosition(l, null);
	},
	dragSizeTop: function(inEvent) {
		var t = Math.max(this.snapY(inEvent.dragY - this.sizingDelta), 0);
		var h = this.getHeight() + this.top - t;
		this.setDimensions(null, h);
		this.setPosition(null, t);
	},
	dragSizeBottom: function(inEvent) {
		var h = inEvent.dragY - this.top + this.sizingDelta;
		h = Math.min(h, this.windowBounds.h - this.top)
		this.setDimensions(null, h);				
	},
	doMouseDown: function(inEvent) {
		this.setZIndexes();
		this.stack.moveToTop(this);
	},
	doMouseMove: function(inEvent) {
		this.processEvent(inEvent);
		if (this.sizeable && !turbo.drag.dragging)
			this.setSizing(inEvent);
		this.setCursor(inEvent);
	},
	setCursor: function(inEvent) {
		var c = 'default';
		if (this.isSizing())
			with (this._sizing)
				if ((t || b) && !(l || r))
					c = 'n-resize';
				else if ((l || r) && !(t || b))	
					c = 'e-resize';
				else if (t && l)
					c = 'nw-resize';
				else if (t && r)
					c = 'ne-resize';
				else if (b && l)
					c = 'sw-resize';
				else if (b && r)
					c = 'se-resize';
		this.domNode.style.cursor = c;
	},
	doDragSize: function(inEvent) {
		if (!this.sizeable)
			return false;
		if (this._sizing.t)	
			this.dragSizeTop(inEvent);
		if (this._sizing.b)	
			this.dragSizeBottom(inEvent);
		if (this._sizing.l)	
			this.dragSizeLeft(inEvent);
		if (this._sizing.r)	
			this.dragSizeRight(inEvent);
		if (this.isSizing()) {
			this.setCursor(inEvent);
			this.renderBounds();
			return true;
		}
	},
	isSizing: function() {
		for (var i in this._sizing)
			if (this._sizing[i])
				return true;
	},
	doDrag: function(inEvent) {
		if (!this.doDragSize(inEvent) && this.dragable) {
			this.setPosition(inEvent.domNodeX, inEvent.domNodeY, true);
			this.onDrag(inEvent);	
		}
	},
	doStartDrag: function(inEvent) {
		//turbo.debug('doStartDrag');
		if (!this.dragable && !this.isSizing())
			return;
		if (this.dragable)
			this.onStartDrag(inEvent);	
	},
	doEndDrag: function() {
		//turbo.debug('doEndDrag');
		if (this.isSizing()) {
			this._sizing = {};
			this.align();			
		}
		this.onEndDrag();
	},
	// floater
	initNodes: function() {
		this.domNode.isFloater = true;
		this._hide();
		this.styleChanged();
	},
	detachNode: function(inNode) {
		//return;
		turbo.remove(inNode);
	},
	attachNode: function(inNode) {
		//return;
		var p = this.getParentNode();
		try {
			p.appendChild(inNode);
		}	catch(e) {
			turbo.debug(this.widgetId + ' failed to append node to ');
			turbo.debugObject(p);
		}
	},
	setZIndexes: function() {
		var z = (this.domNode.style.zIndex && this.domNode.style.zIndex != 'auto' ? this.domNode.style.zIndex : this.baseZ);
		var active = this.stack.getTop();
		if (active && active != this)
			z = 2 + (parseInt(dojo.html.getStyle(active.domNode, "z-index")) || z);
		this.domNode.style.zIndex = z;
		//turbo.debugf('%f z: %f', this.widgetId, this.domNode.style.zIndex);
		this.setScrimZ();	
	},
	setScrimZ: function() {
		if (this.scrim)
			this.scrim.style.zIndex = (1 || parseInt(dojo.html.getStyle(this.domNode, "z-index"))) - 1;
	},
	getParentNode: function() {
		return document.body;
	},
	getRect: function() {
		if (!this._showing) {
			this.attachNode(this.domNode);
			turbo.show(this.domNode);
		}
		var box = dojo.html.getBorderBox(this.domNode);
		if (!this._showing)
			this.hideDomNode();
		return {t: this.top, r: this.left + box.width, b: this.top + box.height, l: this.left};
	},
	snapX: function(inX) {
		return (this.snap && this.snap.x ? inX  - inX % this.snap.x : inX);
	},
	snapY: function(inY) {
		return (this.snap && this.snap.y ? inY - inY % this.snap.y : inY);
	},
	setPosition: function(inLeft, inTop, inUpdatePosition) {
		if (inLeft || inLeft === 0)
			this.left = this.snapX(inLeft);
		if (inTop  || inTop === 0)
			this.top = this.snapY(inTop);
		if (inUpdatePosition)
			this.updatePosition();
	},
	getCssDimensions: function() {
		var c = dojo.html.getContentBox(this.domNode);
		return (!c.height || !c.width ? false : {w: c.width, h: c.height});
	},
	setCssDimensions: function() {
		var cssD = this.getCssDimensions();
		if (!cssD) {
			//turbo.debug('failed to getCssDimensions');
			return false;
		}
		var w = this.getWidth();
		var h = this.getHeight();
		//console.log('width', w, cssD.w, 'height', h, cssD.h)		
		this.setDimensions((w ? w : cssD.w), (h ? h : cssD.h));
		return true;
	},
	hasDimensions: function() {
		return (this.getWidth() > 0 && this.getHeight() > 0);
	},
	getScrim: function() {
		if (!this.useScrim)
			return null;
		var props = {	backgroundColor: this.bgColor, opacity: this.bgOpacity, zIndex: parseInt(dojo.html.getStyle(this.domNode, "z-index")) - 1	};
		var bounds = (this.applyScrimToParent ? null : {t: this.top, l: this.left, w: this.getWidth(), h: this.getHeight() });
		return turbo.scrim.get(this.getParentNode(), this.widgetId + '_scrim', props, bounds);
	},
	setDimensions: function(inWidth, inHeight) {
		if (inWidth > 0) {
			this._dimensions.w = this.snapX(Math.max(inWidth, this._minDimensions.w));
		}
		if (inHeight > 0) {
			this._dimensions.h = this.snapY(Math.max(inHeight, this._minDimensions.h));	
		}
	},
	setMinDimensions: function(inWidth, inHeight) {
		if (inWidth > 0)
			this._minDimensions.w = this.snapX(inWidth);
		if (inHeight > 0)
			this._minDimensions.h = this.snapY(inHeight);	
	},
	getWidth: function() {
		return this._dimensions.w;
	},
	getHeight: function() {
		return this._dimensions.h;
	},
	applyPrePositioning: function() {
		// move domNode to 0, 0 so can get best real cssdimenions
		turbo.setStyleLeftPx(this.domNode, 0);
		turbo.setStyleTopPx(this.domNode, 0);
		//turbo.debugf('applyPrePositioning: %f', this.domNode.style.left);
		// size domNode to any predefined dimensions
		if (this.getWidth())
			this.renderWidth();
		if (this.getHeight())
			this.renderHeight();
	},
	toggleShow: function() {
		if (this.isShowing())
			this.hide();
		else
			this.show();
	},
	doOnShow: function() {
		this.onShow.apply(this, arguments);
	},	
	isShowing: function() {
		return (turbo.isShowing(this.domNode) || this._showing);
	},
	show: function(inX, inY) {
		this.ensureWindowBounds();
		if (this.isShowing() || this._initShow) {
			if (inX || inY)
				this.setPosition(inX, inY, true);
				this.doOnShow.apply(this, arguments);
			return;
		}
		this._initShow = true;
		this.setPosition(inX, inY);
		this.onBeforeShow.apply(this, arguments);
		this.attachNode(this.domNode);
		this.timeShown = turbo.time();	
		var cb = turbo.bindArgs(this, this.doOnShow, arguments);
		if (this.autoSize) {
			this.resetExtent();	
			this.applyPrePositioning();
			turbo.defer(this, this.attemptShow, cb, 10);
		} else
			this.attemptShow(cb);
	},
	showTimedOut: function() {
		var timedOut = (this.timeShown + this.showTimeoutMs < turbo.time());
		if (timedOut)
			turbo.debug('Timed out attempting to show.');
		return timedOut;	
	},
	attemptShow: function(inCallback) {
		// handle hide during show process
		if (!this._initShow) {
			this._hide();
			return;
		}
		turbo.setVisibility(this.domNode, false);
		if (!turbo.isShowing(this.domNode))
			turbo.show(this.domNode);
		if (this.autoSize && !this.setCssDimensions()) {
			turbo.defer(turbo.bindArgs(this, this.attemptShow, inCallback), 10);
			return;
		}
		this._showing = true;
		if (this.useScrim)
			this.scrim = this.getScrim();
		this.doRenderBounds(inCallback);
	},
	doRenderBounds: function(inCallback) {
		if (!this._showing)
			return;
		this.renderBounds();
		this.finishShow(inCallback);
	},	
	putOnStack: function() {
		if (this.stack.getIndex(this) < 0)
			this.stack.add(this);
		else
			this.stack.moveToTop(this);
	},
	finishShow: function(inCallback) {
		this.setZIndexes();
		this.putOnStack();		
		var self = this;
		var cb = function() {
			turbo.setVisibility(self.domNode, true);
			if (inCallback)
				inCallback();
		}
		this.align(cb);
	},	
	delayedHide: function() {
		turbo.defer(this, this.hide, 10);
	},
	hide: function() {
		this.doHide();
	},
	hideStack: function() {
		if (this.stack)
			this.stack.map(this.stack.count()-1, 0, 'doHide');
	},
	doHide: function() {
		if (this.onBeforeHide)
			this.onBeforeHide();
		this._hide();
		this.stack.remove(this);
		if (this.onHide)
			turbo.defer(this, 'onHide', 10);
	},
	_hide: function() {
		this.hideDomNode();
		this.hideScrim();
		this._initShow = false;
		this._showing = false;
	},
	hideDomNode: function() {
		turbo.hide(this.domNode);
		//this.detachNode(this.domNode);
	},
	hideScrim: function() {
		if (!this.scrim)
			return;
		turbo.hide(this.scrim);
		turbo.scrim.release(this.scrim);
	},
	hasContent: function() {
		return Boolean(this.domNode.innerHTML);
	},
	setContent: function(inHtml, inParseWidgets) {
		turbo.clean(this.domNode);
		this.domNode.innerHTML = inHtml;
		if (inParseWidgets)
			turbo.parseWidgets(this.domNode);
		if (this.autoSize)
			this.resetExtent();	
		if (this.isShowing()) {
			this.applyPrePositioning();
			this.attemptShow();
		}
	},
	resetHeight: function() {
		this._dimensions.h = 0;
		this.domNode.style.height = '';
	},
	resetWidth: function() {
		this._dimensions.w = 0;
		this.domNode.style.width = '';
	},
	resetExtent: function() {
		this.resetWidth();
		this.resetHeight();
	},
	getWindowBounds: function() {
		var p = this.getParentNode();
		var b = (p == document.body ? dojo.html.getViewport() : dojo.html.getMarginBox(p));
		return {w: b.w || b.width, h: b.h || b.height};
	},	
	resize: function() {
		if (this.isShowing())
			this.doResize();
	},
	doResize: function() {
		this.windowBounds = this.getWindowBounds();
	},
	ensureWindowBounds: function() {
		if (!this.windowBounds.w || !this.windowBounds.h)
			this.windowBounds = this.getWindowBounds();
	},
	boundPosition: function(inLeft, inTop) {
		inLeft = (inLeft ? inLeft : this.left);
		inTop = (inTop ? inTop: this.top);
		this.ensureWindowBounds();
		//console.log('before', inLeft, inTop);
		inLeft = Math.max(0, Math.min(inLeft,  this.windowBounds.w - this.getWidth() - this.windowMargin));
		inTop = Math.max(0, Math.min(inTop, this.windowBounds.h - this.getHeight() - this.windowMargin));
		//console.log('after', inLeft, inTop);
		this.setPosition(inLeft, inTop);
		if (this.top + this.getHeight() > this.windowBounds.h)
			this.setDimensions(null, this.windowBounds.h);
		if (this.left + this.getWidth() > this.windowBounds.w)
			this.setDimensions(this.windowBounds.w, null);
	},
	renderBounds: function(inX, inY, inW, inH) {
		this.boundPosition(inX, inY);
		if (inW && inH)
			this.setDimensions(inW, inH);
		if (this.hasDimensions()) {
			// FIXME: sizing will be wrong without considering margin / border
			//turbo.setBounds(this.domNode, this.left, this.top, this.getWidth(), this.getHeight());
			this.renderLeft();
			this.renderTop();
			this.renderWidth();
			this.renderHeight();
			this.setScrimBounds();
		}
		else
			this.updatePosition();
	},
	renderLeft: function() {
		if (this.left >=0)
			turbo.setStyleLeftPx(this.domNode, this.left);
	},
	renderTop: function() {
		if (this.top >=0)
		turbo.setStyleTopPx(this.domNode, this.top);
	},
	renderWidth: function() {
		turbo.setStyleWidthPx(this.domNode, this.getWidth());
	},
	renderHeight: function() {
		turbo.setStyleHeightPx(this.domNode, this.getHeight());
	},
	setScrimBounds: function() {
		if (!this.scrim)
			return;
		if (this.applyScrimToParent)
			turbo.scrim.size(this.scrim);	
		else	
			turbo.setBounds(this.scrim, this.left, this.top, this.getWidth(), this.getHeight());
	},
	updatePosition: function() {
		this.boundPosition();
		this.renderLeft();
		this.renderTop();
		this.updateScrimPosition();	
	},
	updateScrimPosition: function() {
		if (!this.scrim)
			return;
		if (!this.applyScrimToParent) {
			if (this.left >=0)
				turbo.setStyleLeftPx(this.scrim, this.left);
			if (this.top >=0)	
				turbo.setStyleTopPx(this.scrim, this.top);
		}
	},
	align: function(inCallback) {
		// FIXME: testing
		//inCallback();
		//return;
		turbo.defer(turbo.bindArgs(this, '_align', inCallback), 10);
	},
	_align: function(inCallback) {
		turbo.aligner.alignFrom(this.domNode);
		if (inCallback)
			turbo.defer(inCallback, 10);
	}
});


dojo.provide("turbo.widgets.TurboTooltip");
dojo.provide("turbo.widgets.HtmlTurboTooltip");

dojo.require("turbo.widgets.TurboFloater");

dojo.widget.defineWidget("dojo.widget.html.TurboTooltip", dojo.widget.html.TurboFloater, 
	function() {
		this.boundedNodes = [ ];
		this.tipPosition = {x: 0, y: 0 };
		this.tipTarget = null;
	}, {
	tipDelay: 300,
	tipOffsetLeft: 12,
	tipOffsetTop: 12,
	boundedNodes: null,
	parseWidgets: false,
	fillInTemplate: function(inArgs, inFrag) {
		this.inherited("fillInTemplate", arguments);
		this.styleRoot = 'TurboFloater';
		dojo.event.connect(this.domNode, 'onmouseout', this, 'tipOut');
	},
	setTipPosition: function(inEvent) {
		var n = (dojo.lang.inArray(this.boundedNodes, this.tipTarget));
		var x = this.tipPosition.x + (n ? 0 : this.tipOffsetLeft);
		var y = this.tipPosition.y + (n ? 0 : this.tipOffsetTop);
		this.setPosition(x, y);
		if (dojo.lang.inArray(this.boundedNodes, inEvent.target))
			this.pushOutsideNode(inEvent.target);
		this.boundPosition();	
	},
	canShow: function() { 
		return true;
	},
	show: function(inEvent) {
		this.hideTip();
		inEvent.tooltipNode = this.currentTarget;
		if (!this.canShow(inEvent))
			return false;
		this.setTipPosition(inEvent);
		this.inherited('show', [null, null, inEvent]);
	},
	onBeforeShow: function(inX, inY, inEvent) {
		var c = this.onGetContent(inEvent);
		if (c)
			this.setContent(c, this.parseWidgets);
	},
	pushOutsideNode: function(inNode) {
		var moved = false;
		var nodeRect = turbo.getNodeRect(inNode);
		var tipRect = this.getRect();
		var tipDims = {w: tipRect.r - tipRect.l, h: tipRect.b - tipRect.t};
		var tipMaybe = {
			l: nodeRect.l - tipDims.w - this.tipOffsetLeft,
			r: nodeRect.r + this.tipOffsetLeft,
			t: nodeRect.t - tipDims.h - this.tipOffsetTop,
			b: nodeRect.b + this.tipOffsetLeft
		}
		// try to push left or right
		if (this.isBetweenNums(tipRect.l, nodeRect.l, nodeRect.r) || this.isBetweenNums(tipRect.r, nodeRect.l, nodeRect.r)) {
			moved = true;
			var canGoLeft = this.fitsBetween(tipMaybe.l, tipMaybe.l + tipDims.w, 0, nodeRect.l);
			var canGoRight = this.fitsBetween(tipMaybe.r, tipMaybe.r + tipDims.w, nodeRect.r, this.windowBounds.w);	
			var shouldGoLeft = tipRect.l < nodeRect.l + ((nodeRect.r - nodeRect.l) / 2);
			if (canGoLeft && shouldGoLeft || !canGoRight)
				this.setPosition(tipMaybe.l, this.top + this.tipOffsetTop);
			else if (canGoRight)	
				this.setPosition(tipMaybe.r, this.top + this.tipOffsetTop);
			else
				moved = false;
		}
		// try to push top or bottom
		if (!moved && (this.isBetweenNums(tipRect.t, nodeRect.t, nodeRect.b) || 
			this.isBetweenNums(tipRect.b, nodeRect.t, nodeRect.b))) {
			moved = true;
			var canGoUp = this.fitsBetween(tipMaybe.t, tipMaybe.t + tipDims.h, 0, nodeRect.t);
			var canGoDown = this.fitsBetween(tipMaybe.b, tipMaybe.b + tipDims.h, nodeRect.b, this.windowBounds.h);	
			var shouldGoUp = tipRect.t < nodeRect.t + ((nodeRect.b - nodeRect.t) / 2);
			if (canGoUp && shouldGoUp || !canGoDown)
				this.setPosition(this.left + this.tipOffsetLeft, tipMaybe.t);
			else if (canGoDown)
				this.setPosition(this.left + this.tipOffsetLeft, tipMaybe.b);
			else
				moved = false;	
		}
		return moved;
	},
	fitsBetween: function(inSx, inSy, inBx, inBy) {
		return (inSx > inBx && inSy < inBy); 
	},
	isBetweenNums: function(inA, inX, inY) {
		return (inA > inX && inA <inY);
	},
	_toolTipOver: function(inEvent) {
		this.currentTarget = inEvent.currentTarget;
		if (!this.ttPending) {
			this.updateTipPosition(inEvent);
			this.ttPending = turbo.defer(this, 'show', inEvent, this.tipDelay);
		}
		turbo.killEvent(inEvent);	
	},
	_toolTipOut: function(inEvent) {	
		if (!dojo.dom.isDescendantOf(inEvent.relatedTarget, this.domNode))
			this.hideTip();
	},
	_toolTipMove: function(inEvent) {
		if (this.ttPending && !this.isShowing())
			this.updateTipPosition(inEvent);
	},
	hideTip: function() {
		//console.log('hideTip');
		window.clearTimeout(this.ttPending);
		delete this.ttPending;
		this.positionEvent = null;
		if (this.isShowing())
			this.hide();																											 
	},
	// FIXME: don't use kwconnect... 
	connectOnce: function(inSrcObj, inSrcFunc, inMethodName, inDisconnect) {
		dojo.event[inDisconnect ? 'kwDisconnect' : 'kwConnect']({
			srcObj:     inSrcObj, 
			srcFunc:    inSrcFunc, 
			targetObj:  this, 
			targetFunc: inMethodName,
			once:       true
		});
	},
	initToolTip: function() {
		if (arguments.length == 0)
			return;
		// we're tooltipping a node
		if (arguments.length < 2)
			this.toolTipNode(arguments[0]);
		// custom tooltip with over and out
		else {
			var inObj = arguments[0];
			var inMouseOver = arguments[1];
			var inMouseMove = arguments[2];
			var inMouseOut = arguments[3];
			this.connectToolTip(inObj, inMouseOver, inMouseMove, inMouseOut);
		}
	},
	initToolTipNode: function(inNode, inNodeOutOfBounds) {
		inNode = turbo.$(inNode);
		if (inNodeOutOfBounds && !dojo.lang.inArray(this.boundedNodes, inNode))
			this.boundedNodes.push(inNode);
		this.connectToolTip(inNode, 'onmouseover', 'onmousemove', 'onmouseout');
	},
	connectToolTip: function(inObj, inOver, inMove, inOut) {
		if (inOver)
			this.connectOver(inObj, inOver);
		if (inMove)	
			this.connectMove(inObj, inMove);
		if (inOut)	
			this.connectOut(inObj, inOut);
	},
	connectOver: function(inObj, inMethod) {
		this.connectOnce(inObj, inMethod, '_toolTipOver');
	},
	connectMove: function(inObj, inMethod) {
		this.connectOnce(inObj, inMethod, '_toolTipMove');
	},
	connectOut: function(inObj, inMethod) {
		this.connectOnce(inObj, inMethod, '_toolTipOut');
	},
	updateTipPosition: function(inEvent) {
		this.tipPosition = {x: inEvent.pageX, y: inEvent.pageY};
		this.tipTarget = inEvent.target;
	},
	tipOut: function(inEvent) {
		if (!dojo.dom.isDescendantOf(inEvent.relatedTarget, this.domNode))
			this.hideTip();
	},
	onGetContent: turbo.nop
});																										 

