dojo.provide('turbo.lib.scroller');
dojo.require('dojo.event.*');

// virtual scrollbox

// Content must in /rows/
// Rows are managed in contiguous sets called /pages/
// There are a fixed # of rows per page
// The minimum rendered unit is a page

dojo.declare('turbo.scroller.base', null, 
	function() {
		this.pageHeights = [ ];
		this.stack = [ ];
	}, {
	// specified
	rowCount: 0, // total number of rows to manage
	defaultRowHeight: 10, // default height of a row
	keepRows: 100, // maximum number of rows that should exist at one time
	contentNode: null, // node to contain pages
	scrollboxNode: null, // node that controls scrolling
	// calculated
	defaultPageHeight: 0, // default height of a page
	keepPages: 10, // maximum number of pages that should exists at one time
	pageCount: 0,
	windowHeight: 0,
	firstVisibleRow: 0,
	lastVisibleRow: 0,
	// private
	page: 0,
	pageTop: 0,
	// init
	init: function(inRowCount, inKeepRows, inRowsPerPage) {
		switch (arguments.length) {
			case 3: this.rowsPerPage = inRowsPerPage;
			case 2: this.keepRows = inKeepRows;
			case 1: this.rowCount = inRowCount;
		}
		this.defaultPageHeight = this.defaultRowHeight * this.rowsPerPage;
		//this.defaultPageHeight = this.defaultRowHeight * Math.min(this.rowsPerPage, this.rowCount);
		this.pageCount = Math.ceil(this.rowCount / this.rowsPerPage);
		this.keepPages = Math.max(Math.ceil(this.keepRows / this.rowsPerPage), 2);
		this.invalidate();
		if (this.scrollboxNode) {
			this.scrollboxNode.scrollTop = 0;
			this.scroll(0);
			this.scrollboxNode.onscroll = turbo.bind(this, 'onscroll');
		}
	},
	// updating
	invalidate: function() {
		this.invalidateNodes();
		this.pageHeights = [ ];
		this.height = (this.pageCount ? (this.pageCount - 1)* this.defaultPageHeight + this.calcLastPageHeight() : 0);
		this.resize();
	},
	updateRowCount: function(inRowCount) {
		this.invalidateNodes();
		this.rowCount = inRowCount;
		// update page count, adjust document height
		oldPageCount = this.pageCount;
		this.pageCount = Math.ceil(this.rowCount / this.rowsPerPage);
		if (this.pageCount < oldPageCount)
			for (var i=oldPageCount-1; i>=this.pageCount; i--) {
				this.height -= this.getPageHeight(i);
				delete this.pageHeights[i]
			}
		else if (this.pageCount > oldPageCount)  {
			this.height += this.defaultPageHeight * (this.pageCount - oldPageCount - 1) + this.calcLastPageHeight();
		}
		this.resize();
	},
	// abstract interface
	pageExists: function(inPageIndex) {
	},
	measurePage: function(inPageIndex) {
	},
	positionPage: function(inPageIndex, inPos) {
	},
	repositionPages: function(inPageIndex) {
	},
  installPage: function(inPageIndex) {
	},
	preparePage: function(inPageIndex, inPos, inReuseNode) {
	},
	renderPage: function(inPageIndex) {
	},
	removePage: function(inPageIndex) {
	},
	pacify: function(inShouldPacify) {
	},
	// pacification
	pacifying: false,
	pacifyTicks: 200,
	setPacifying: function(inPacifying) {
		if (this.pacifying != inPacifying) {
			this.pacifying = inPacifying; 
			this.pacify(this.pacifying);
		}
	},
	startPacify: function() {
		this.startPacifyTicks = new Date().getTime();
	},
	doPacify: function() {
		var result = (new Date().getTime() - this.startPacifyTicks) > this.pacifyTicks;
		this.setPacifying(true);
		this.startPacify();
		return result;
	},
	endPacify: function() {
		this.setPacifying(false);
	},
	// default sizing implementation
	resize: function() {
		if (this.scrollboxNode)
			this.windowHeight = this.scrollboxNode.clientHeight;
		turbo.setStyleHeightPx(this.contentNode, this.height);
	},
	calcLastPageHeight: function() {
		if (!this.pageCount)
			return 0;
		var lastPage = this.pageCount - 1;
		var lastPageHeight = ((this.rowCount % this.rowsPerPage)||(this.rowsPerPage)) * this.defaultRowHeight;
		this.pageHeights[lastPage] = lastPageHeight;
		return lastPageHeight;
	},
	updateContentHeight: function(inDh) {
		//turbo.debugf('updateContentHeight: old=%d, delta=%d, new=%d', this.height, inDh, this.height + inDh);
		this.height += inDh;
		this.resize();
	},
	updatePageHeight: function(inPageIndex) {
		if (this.pageExists(inPageIndex)) {
			var oh = this.getPageHeight(inPageIndex);
			var h = (this.measurePage(inPageIndex))||(oh);
			this.pageHeights[inPageIndex] = h;
			//turbo.debugf('building: page %d height = %d (%d)', inPageIndex, h, inPos);
			if ((h)&&(oh != h)) {
				this.updateContentHeight(h - oh)
				this.repositionPages(inPageIndex);
			}
		}
	},
	rowHeightChanged: function(inRowIndex) {
		this.updatePageHeight(Math.floor(inRowIndex / this.rowsPerPage));
	},
	// scroller core
	invalidateNodes: function() {
		while (this.stack.length) 
			this.destroyPage(this.popPage());
	},
	createPageNode: function() {
		var p = document.createElement('div');
		p.style.position = 'absolute';
		//p.style.width = '100%';
		p.style.left = '0';
		return p;
	},
	getPageHeight: function(inPageIndex) {
		var ph = this.pageHeights[inPageIndex];			
		return (ph !== undefined ? ph : this.defaultPageHeight);
	},
	// FIXME: this is not a stack, it's a FIFO list
	pushPage: function(inPageIndex) {
		return this.stack.push(inPageIndex);
	},
	popPage: function() {
		return this.stack.shift();
	},
	findPage: function(inTop) {
		var i = 0, h = 0;
		for (var ph = 0; i<this.pageCount; i++, h += ph) {
			ph = this.getPageHeight(i);
			if (h + ph >= inTop)
				break;
		}
		this.page = i;
		this.pageTop = h;
	},
	buildPage: function(inPageIndex, inReuseNode, inPos) {
		this.preparePage(inPageIndex, inReuseNode);
		this.positionPage(inPageIndex, inPos);
		// order of operations is key below
		this.installPage(inPageIndex);
		this.renderPage(inPageIndex);
		// order of operations is key above
		this.pushPage(inPageIndex);
	},
	needPage: function(inPageIndex, inPos) {
		//turbo.debugf('scroller.needPage(%d, %d)', inPageIndex, inPos);
		var h = this.getPageHeight(inPageIndex), oh = h;
		if (!this.pageExists(inPageIndex)) {
			this.buildPage(inPageIndex, (this.keepPages)&&(this.stack.length >= this.keepPages), inPos);
			//turbo.debugf('scroller.needPage: page %d, measurePage = %d, h = %d', inPageIndex, this.measurePage(inPageIndex), h);
			h = this.measurePage(inPageIndex) || h;
			this.pageHeights[inPageIndex] = h;
			//turbo.debugf('building: page %d height = %d (%d)', inPageIndex, h, inPos);
			if (h && (oh != h))
				this.updateContentHeight(h - oh)
		} 
		else {
			//turbo.debugf('positioning: page %d height = %d (%d)', inPageIndex, h, inPos);
			this.positionPage(inPageIndex, inPos);
		}
		return h;		
	},
	onscroll: function() {
		this.scroll(this.scrollboxNode.scrollTop);		
	},
	scroll: function(inTop) {
		this.startPacify();
		this.findPage(inTop);
		var h = this.height;
		var b = this.getScrollBottom(inTop);
		for (var p=this.page, y=this.pageTop; (p<this.pageCount)&&((b<0)||(y<b)); p++)
			y += this.needPage(p, y);
		//turbo.debug(this.pageTop, y);
		this.firstVisibleRow = this.getFirstVisibleRow(this.page, this.pageTop, inTop);
		this.lastVisibleRow = this.getLastVisibleRow(p - 1, y, b);
		// indicates some page size has been updated
		if (h != this.height)
			this.repositionPages(p-1);
		this.endPacify();
	},
	getScrollBottom: function(inTop) {
		return (this.windowHeight >= 0 ? inTop + this.windowHeight : -1);
	},
	// events
	processNodeEvent: function(e, inNode) {
		var t = e.target;
		while (t && (t != inNode) && t.parentNode && (t.parentNode.parentNode != inNode)) 
			t = t.parentNode;
		if (!t || !t.parentNode || (t.parentNode.parentNode != inNode))
			return false;
		var page = t.parentNode;			
		e.topRowIndex = page.pageIndex * this.rowsPerPage;
		e.rowIndex = e.topRowIndex + turbo.indexInParent(t);
		e.rowTarget = t;						
		return true;
	},
	processEvent: function(e) {
		return this.processNodeEvent(e, this.contentNode);
	},
	dummy: 0
});

dojo.declare('turbo.scroller', turbo.scroller.base, 
	function() {								
		this.pageNodes = [ ];
	},{
	// virtual rendering interface
	renderRow: function(inRowIndex, inPageNode) {
	},
	removeRow: function(inRowIndex) {
	},
	// page node operations
	getDefaultNodes: function() {
		return this.pageNodes;
	},
	getDefaultPageNode: function(inPageIndex) {
		return this.getDefaultNodes()[inPageIndex];
	},
	positionPageNode: function(inNode, inPos) {
		inNode.style.top = inPos + 'px';
	},
	getPageNodePosition: function(inNode) {
		return inNode.offsetTop;
	},
	repositionPageNodes: function(inPageIndex, inNodes) {
		var last = 0;
		for (var i=0; i<this.stack.length; i++)
			last = Math.max(this.stack[i], last);
		//	
		var n = inNodes[inPageIndex];
		var y = (n ? this.getPageNodePosition(n) + this.getPageHeight(inPageIndex) : 0);
		//console.log('detected height change, repositioning from #%d (%d) @ %d ', inPageIndex + 1, last, y, this.pageHeights[0]);
		//
		for (var p=inPageIndex+1; p<=last; p++) {
			n = inNodes[p];
			if (n) {
				//console.log('#%d @ %d', inPageIndex, y, this.getPageNodePosition(n));
				if (this.getPageNodePosition(n) == y)
					return;
				//console.log('placing page %d at %d', p, y);
				this.positionPage(p, y);
			}
			y += this.getPageHeight(p);
		}
	},
	invalidatePageNode: function(inPageIndex, inNodes) {
		var p = inNodes[inPageIndex];
		if (p) {
			delete inNodes[inPageIndex];
			this.removePage(inPageIndex, p);
			turbo.clean(p);
			p.innerHTML = '';
		}
		return p;
	},
	preparePageNode: function(inPageIndex, inReusePageIndex, inNodes) {
		var p = (inReusePageIndex === null ? this.createPageNode() : this.invalidatePageNode(inReusePageIndex, inNodes));
		p.pageIndex = inPageIndex;
		p.id = 'page-' + inPageIndex;
		inNodes[inPageIndex] = p;
	},
	// implementation for page manager
	pageExists: function(inPageIndex) {
		return Boolean(this.getDefaultPageNode(inPageIndex));
	},
	measurePage: function(inPageIndex) {
		return this.getDefaultPageNode(inPageIndex).offsetHeight;
	},
	positionPage: function(inPageIndex, inPos) {
		this.positionPageNode(this.getDefaultPageNode(inPageIndex), inPos);
	},
	repositionPages: function(inPageIndex) {
		this.repositionPageNodes(inPageIndex, this.getDefaultNodes());
	},
	preparePage: function(inPageIndex, inReuseNode) {
		this.preparePageNode(inPageIndex, (inReuseNode ? this.popPage() : null), this.getDefaultNodes());
	},
  installPage: function(inPageIndex) {
		this.contentNode.appendChild(this.getDefaultPageNode(inPageIndex));
	},
	destroyPage: function(inPageIndex) {
		var p = this.invalidatePageNode(inPageIndex, this.getDefaultNodes());
		turbo.remove(p);
	},
	// rendering implementation
	renderPage: function(inPageIndex) {
		var node = this.pageNodes[inPageIndex];
		for (var i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++)
			this.renderRow(j, node);
	},
	removePage: function(inPageIndex) {
		for (var i=0, j=inPageIndex*this.rowsPerPage; i<this.rowsPerPage; i++, j++)
			this.removeRow(j);
	},
	// scroll control
	getPageRow: function(inPage) {
		return inPage * this.rowsPerPage;
	},
	getLastPageRow: function(inPage) {
		return Math.min(this.rowCount, this.getPageRow(inPage + 1)) - 1;
	},
	getFirstVisibleRowNodes: function(inPage, inPageTop, inScrollTop, inNodes) {
		var row = this.getPageRow(inPage);
		var rows = turbo.divkids(inNodes[inPage]);
		for (var i=0,l=rows.length; i<l && inPageTop<inScrollTop; i++, row++)
			inPageTop += rows[i].offsetHeight;			
		return (row ? row - 1 : row);
	},
	getFirstVisibleRow: function(inPage, inPageTop, inScrollTop) {
		//turbo.debugf('scroller.getFirstVisibleRow(%d, %d, %d)', inPage, inPageTop, inScrollTop);
		if (!this.pageExists(inPage)) 
			return 0;
		return this.getFirstVisibleRowNodes(inPage, inPageTop, inScrollTop, this.getDefaultNodes());
	},
	getLastVisibleRowNodes: function(inPage, inBottom, inScrollBottom, inNodes) {
		var row = this.getLastPageRow(inPage);
		var rows = turbo.divkids(inNodes[inPage]);
		for (var i=rows.length-1; i>=0 && inBottom>inScrollBottom; i--, row--)
			inBottom -= rows[i].offsetHeight;			
		return row + 1;
	},
	getLastVisibleRow: function(inPage, inBottom, inScrollBottom) {
		if (!this.pageExists(inPage)) 
			return 0;
		return this.getLastVisibleRowNodes(inPage, inBottom, inScrollBottom, this.getDefaultNodes());
	},
	findTopRowForNodes: function(inScrollTop, inNodes) {
		var rows = turbo.divkids(inNodes[this.page]);
		for (var i=0,l=rows.length,t=this.pageTop,h; i<l; i++) {
			h = rows[i].offsetHeight;
			t += h;			
			if (t >= inScrollTop) {
				this.offset = h - (t - inScrollTop);
				return i + this.page * this.rowsPerPage;
			}
		}
		return -1;
	},
	findScrollTopForNodes: function(inRow, inNodes) {
		var rowPage = Math.floor(inRow / this.rowsPerPage);
		var t = 0;
		for (var i=0; i<rowPage; i++)
			t += this.getPageHeight(i);
		this.pageTop = t;
		this.needPage(rowPage, this.pageTop);
		var rows = turbo.divkids(inNodes[rowPage]);
		var r = inRow - this.rowsPerPage * rowPage;
		for (var i=0,l=rows.length; i<l && i<r; i++)
			t += rows[i].offsetHeight;			
		return t;
	},
	findTopRow: function(inScrollTop) {
		return this.findTopRowForNodes(inScrollTop, this.getDefaultNodes());
	},
	findScrollTop: function(inRow) {
		return this.findScrollTopForNodes(inRow, this.getDefaultNodes());
	},
	dummy: 0
});

dojo.declare('turbo.scroller.columns', turbo.scroller, 
	function(inContentNodes) {								
		this.setContentNodes(inContentNodes);
	},{
	// nodes
	setContentNodes: function(inNodes) {
		this.contentNodes = inNodes;
		this.colCount = (this.contentNodes ? this.contentNodes.length : 0);
		this.pageNodes = [ ];
		for (var i=0; i<this.colCount; i++)
			this.pageNodes[i] = [ ];
	},
	getDefaultNodes: function() {
		return this.pageNodes[0] || [ ];
	},
	scroll: function(inTop) {
		if (this.colCount)
			turbo.scroller.prototype.scroll.call(this, inTop);
	},
	// resize
	resize: function() {
		if (this.scrollboxNode)
			this.windowHeight = this.scrollboxNode.clientHeight;
		for (var i=0; i<this.colCount; i++)
			turbo.setStyleHeightPx(this.contentNodes[i], this.height);
	},
	// implementation for page manager
	positionPage: function(inPageIndex, inPos) {
		for (var i=0; i<this.colCount; i++) {
			this.positionPageNode(this.pageNodes[i][inPageIndex], inPos);
		}
	},
	preparePage: function(inPageIndex, inReuseNode) {
		var p = (inReuseNode ? this.popPage() : null);
		for (var i=0; i<this.colCount; i++)
			this.preparePageNode(inPageIndex, p, this.pageNodes[i]);
	},
  installPage: function(inPageIndex) {
		for (var i=0; i<this.colCount; i++)
			this.contentNodes[i].appendChild(this.pageNodes[i][inPageIndex]);
	},
	destroyPage: function(inPageIndex) {
		for (var i=0; i<this.colCount; i++)
			turbo.remove(this.invalidatePageNode(inPageIndex, this.pageNodes[i]));
	},
	// rendering implementation
	renderPage: function(inPageIndex) {
		var nodes = [ ];
		for (var i=0; i<this.colCount; i++)
			nodes[i] = this.pageNodes[i][inPageIndex];			
		//this.renderRows(inPageIndex*this.rowsPerPage, this.rowsPerPage, nodes);
		for (var i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++)
			this.renderRow(j, nodes);
	}
});

dojo.provide("turbo.data.fields");

turbo.data.mixer = function() {
	this.defaultValue = { };
	this.values = [ ];
	this.count = function()	{
		return this.values.length;
	}
	this.clear = function()	{
		this.values = [ ];
	}
	//this.init = function(inIndex, inValue) {
	//}
	this.build = function(inIndex)	{
		var result = turbo.swiss(this.defaultValue, { owner: this });
		result.key = inIndex;
		//this.init(inIndex, result);
		this.values[inIndex] = result;
		return result;
	}
	this.getDefault = function(){
		return this.defaultValue;
	}
	this.setDefault = function(inField /*[, inField2, ... inFieldN] */) {
		for (var i=0,l=arguments.length; i<l; i++)
			turbo.swiss(arguments[i] || {}, this.defaultValue);
	}
	this.get = function(inIndex)	{
		return (this.values[inIndex] || this.build(inIndex));
	}
	this._set = function(inIndex, inField /*[, inField2, ... inFieldN] */) {
		// each field argument can be a single field object of an array of field objects
		var v = this.get(inIndex);
		for (var i=1; i<arguments.length; i++)
			turbo.swiss(arguments[i], v);
		this.values[inIndex] = v;
	}
	this.set = function(/* inIndex, inField [, inField2, ... inFieldN] | inArray */)	{
		if (arguments.length < 1)
			return;
		var a = arguments[0];
		if (!dojo.lang.isArray(a))
			this._set.apply(this, arguments); 
		else {
			if (a.length && a[0]["default"])
				this.setDefault(a.shift());
			for (var i=0, l=a.length; i<l; i++)
				this._set(i, a[i]);
		}
	}
	this.insert = function(inIndex, inProps) {
		if (inIndex >= this.values.length)
			this.values[inIndex] = inProps;
		else
			this.values.splice(inIndex, 0, inProps);
	}
	this.remove = function(inIndex)	{
		this.values.splice(inIndex, 1);
	}
	this.swap = function(inIndexA, inIndexB) {
		turbo.arraySwap(this.values, inIndexA, inIndexB);
	}
	this.move = function(inFromIndex, inToIndex)	{
		turbo.arrayMove(this.values, inFromIndex, inToIndex);
	}
}

turbo.data.field = function(inName) {
	this.name = inName;
	this.compare = turbo.data.compare;
	this.na = '...';
}

turbo.data.fields = function(inFieldClass) {
	turbo.data.mixer.call(this);
	var fieldClass = (inFieldClass ? inFieldClass : turbo.data.field);
	this.defaultValue = new fieldClass();
}

turbo.data.compare = function(a, b) {
	return (a > b ? 1 : (a == b ? 0 : -1));
}

turbo.data.generateComparator = function(inCompare, inColumn, inSubCompare) {
	return function(a, b) { 
		var c = Math.abs(inColumn) - 1;				
		var ineq = inCompare(a[c], b[c]);
		return (ineq ? (inColumn > 0 ? ineq : -ineq) : (inSubCompare && inSubCompare(a, b)));
	}
}
dojo.provide('turbo.data.model');
dojo.require('turbo.data.fields');

dojo.declare("turbo.data.model", null, 
	function(inFields, inData) {									
		this.fields = new turbo.data.fields();
		if (inFields)
			this.fields.set(inFields);
		this.set(inData);
	}, {
	count: 0,
	updating: 0,
	// observer
	observe: function(inMethod, inContext, inFunction, inStop) {
		dojo.event[inStop ? 'disconnect' : 'connect'](this, inMethod, inContext, inFunction);
	},
	unobserve: function(inMethod, inContext, inFunction) {
		this.observe(inMethod, inContext, inFunction, true);
	},
	/*
	// FIXME: use this kind of listener model instead of connecting anonymously
	notify: function(inEvent, inArgs) {
		for (var f, l, i=0;(l=this.listeners[i]);i++) {
			f = l[inEvent];
			if (f) f.apply(l, inArgs);
		}
	},
	*/
	// updates
	clear: function() {
		this.fields.clear();
		this.clearData();
	},
	beginUpdate: function() {
		this.updating++;
	},
	endUpdate: function() {
		if (this.updating) {
			this.updating--;
			if (!this.updating) 
				this.change();
		}
	},
	isUpdating: function() {
		return Boolean(this.updating);
	},
	// data
	clearData: function() {
		this.set(null);
	},
	// observer hooks
	change: function() {		
	},
	insertion: function(/* index */) {		
	},
	removal: function(/* keys */) {		
	},
	// low-level 
	_get: function() {
	},
	_set: function(inDatum /*, index */) {
	},
	_setData: function() {
	},
	_update: function(inOldData, inNewData /*, index*/) {
		return false;
	},
	_insert: function(inData /*, index */) {
		return false;
	},
	_remove: function(/* index */) {
		return false;
	},
	// data access
	get: function(/* index */) {
		return this._get.apply(this, arguments);
	},
	set: function(inDatum /*, index */) {
		this._set.apply(this, arguments);
		this.change.apply(this, turbo.cloneArguments(arguments, 1));
		return true;
	},
	// modify (public API for posting a row modification ... exists so we can keep "update" virtual and abstract)
	compare: function(inOldData, inNewData) {
		return turbo.arrayCompare(inOldData, inNewData);
	},
	modify: function(inOldData, inNewData /*, index*/) {
		if (!this.compare(inOldData, inNewData))
			this.update.apply(this, arguments);
	},
	// update (internal implementation for a row update)
	update: function(inOldData, inNewData, inRowIndex) {
		return this.defaultUpdate.apply(this, arguments);
	},
	defaultUpdate: function(inOldData, inNewData /*, index*/) {
		if (!this._update.apply(this, arguments))
			return false;
		this.change.apply(this, turbo.cloneArguments(arguments, 2));
	},
	// insert
	insert: function(inData /*, index */) {
		return this.defaultInsert.apply(this, arguments);
	},
	defaultInsert: function(inData /*, index */) {
		if (!this._insert.apply(this, arguments))
			return false;
		this.insertion.apply(this, turbo.cloneArguments(arguments, 1));
		return true;
	},
	// remove
	remove: function(inData /*, index */) {
		return this.defaultRemove.apply(this, arguments);
	},
	defaultRemove: function(/* keys */) {
		if (!this._remove.apply(this, arguments))
			return false;
		this.removal.apply(this, arguments);
		return true;
	},
	// sort
	canSort: function(/* (+|-)column_index+1, ... */) {
		return this.sort != null;
	},
	sort: null,
	dummy: 0
});

dojo.declare("turbo.data.table", turbo.data.model, 
	function() { 
		this.cache = [ ];
	},{
	colCount: 0, // tables introduce cols
	data: null,
	cache: null,
	// morphology
	measure: function() {
		this.count = this.getRowCount();
		this.colCount = this.getColCount();
	},
	getRowCount: function() {
		return (this.data ? this.data.length : 0);
	},
	getColCount: function() {
		return (this.data && this.data.length ? this.data[0].length : this.fields.count());
	},
	badIndex: function(inCaller, inDescriptor) {
		turbo.debugf('%s.%s: bad index (%d, %d)', this.className, inCaller, inDescriptor[0], (inDescriptor[1]||'n/a'));
	},
	isGoodIndex: function(inRowIndex, inColIndex){
		return (inRowIndex >= 0 && inRowIndex < this.count && (arguments.length < 2 || (inColIndex >= 0 && inColIndex < this.colCount)));
	},
	// access
	_get: function(inRowIndex, inColIndex) {
		// NOTE: is tricky-get damaging performance? it's probably not worth it.
		var row = this.data[inRowIndex];
		return (inColIndex == undefined ? row : row[inColIndex])
	},
	// modify
	_setData: function(inData) {
		this.data = (inData || [ ]);
	},
	_setRow: function(inData, inRowIndex) {
		this.data[inRowIndex] = inData;	
	},
	_setCell: function(inDatum, inRowIndex, inColIndex) {
		this.data[inRowIndex][inColIndex] = inDatum;
	},
	setters: [ '_setData', '_setData', '_setRow', '_setCell' ],
	_set: function(inDatum, inRowIndex, inColIndex) {
		this[this.setters[arguments.length]].apply(this, arguments);
	},
	setRows: function(inData, inRowIndex) {
		for (var i=0, l=inData.length, r=inRowIndex; i<l; i++, r++)
			this.set(inData[i], r);
	},
	// update
	beginModifyRow: function(inRowIndex) {
		if (!this.cache[inRowIndex])
			this.cache[inRowIndex] = turbo.arrayDup(this.get(inRowIndex));
	},
	endModifyRow: function(inRowIndex) {
		var cache = this.cache[inRowIndex];
		if (cache) {
			var data = this.get(inRowIndex);
			if (!turbo.arrayCompare(cache, data))
				this.modify(cache, data, inRowIndex);
		}
	},
	_update: function(inOldData, inNewData, inRowIndex) {
		this._set(inNewData, inRowIndex);
		delete this.cache[inRowIndex];	
		return true;
	},
	// insert
	_insert: function(inData, inRowIndex) {
		turbo.arrayInsert(this.data, inRowIndex, inData);
		this.count++;
		return true;
	},
	// remove
	_remove: function(inKeys) {
		for (var i=inKeys.length-1; i>=0; i--)	
			turbo.arrayRemove(this.data, inKeys[i]);
		this.count -= inKeys.length;
		return true;
	},
	// sort
	makeComparator: function(inIndices) {
		var result = null;
		for (var i=inIndices.length-1, col; i>=0; i--) {	
			col = Math.abs(inIndices[i]) - 1;
			if (col >= 0) 
				result = turbo.data.generateComparator(this.fields.get(col).compare, inIndices[i], result);
		}
		return result;
	},
	sort: function(/* (+|-)column_index+1, ... */) {
		//if (inComparator)
		this.data.sort(this.makeComparator(arguments));
	},
	swap: function(inIndexA, inIndexB) {
		turbo.arraySwap(this.data, inIndexA, inIndexB);
		this.change(inIndexA);
		this.change(inIndexB);
	},
	dummy: 0
});

dojo.declare("turbo.data.dynamic", turbo.data.table, 
	function() {
		this.page = [ ];									
		this.pages = [ ];
	},{
	page: null,									
	pages: null,
	rowsPerPage: 100,
	requests: 0,
	bop: -1,
	eop: -1,
	// data 
	clearData: function() {
		this.pages = [ ];
		this.bop = this.eop = -1;
		this.set([ ]);
	},
	getRowCount: function() { 
		return this.count;
	},	
	getColCount: function() {
		return this.fields.count();
	},
	setRowCount: function(inCount) { 
		this.count = inCount;
		this.change();
	},	
	// paging
	requestsPending: function(inBoolean) {
	},
	rowToPage: function(inRowIndex) {
		return (this.rowsPerPage ? Math.floor(inRowIndex / this.rowsPerPage) : inRowIndex);
	},
	pageToRow: function(inPageIndex) {
		return (this.rowsPerPage ? this.rowsPerPage * inPageIndex : inPageIndex);
	},
	requestRows: function(inRowIndex, inCount) {
	},
	rowsProvided: function(inRowIndex, inCount) {
		this.requests--;
		if (this.requests == 0)
			this.requestsPending(false);
	},
	requestPage: function(inPageIndex) {
		var row = this.pageToRow(inPageIndex);
		var count = Math.min(this.rowsPerPage, this.count - row);
		if (count > 0) {
			this.requests++;
			this.requestsPending(true);
			turbo.defer(this, "requestRows", row, count, 1);
			//this.requestRows(row, count);
		}
	},
	needPage: function(inPageIndex) {
		if (!this.pages[inPageIndex]) {
			this.pages[inPageIndex] = true;
			this.requestPage(inPageIndex);
		}
	},
	preparePage: function(inRowIndex, inColIndex) {
		if (inRowIndex < this.bop || inRowIndex >= this.eop) {
			var pageIndex = this.rowToPage(inRowIndex);
			this.needPage(pageIndex);
			this.bop = pageIndex * this.rowsPerPage;
			this.eop = this.bop + (this.rowsPerPage || this.count);
		}
	},
	isRowLoaded: function(inRowIndex) {
		return Boolean(this.data[inRowIndex]);
	},
	// access
	getRow: function(inRowIndex) {
		var row = this.data[inRowIndex];
		if (!row)
			this.preparePage(inRowIndex);
		return row;
	},
	_get: function(inRowIndex, inColIndex) {
		var row = this.getRow(inRowIndex);
		return (inColIndex == undefined ? row : (row ? row[inColIndex] : this.fields.get(inColIndex).na));
	},
	// modify
	/*_setRow: function(inData, inRowIndex) {
		this.data[inRowIndex] = inData;
	},*/
	_setCell: function(inDatum, inRowIndex, inColIndex) {
		var row = this.getRow(inRowIndex);
		if (!row) 
			turbo.debugf('[%s] turbo.data.dynamic.set: cannot set data on an unloaded row', this.declaredClass);
		if (row)
			row[inColIndex] = inDatum;
	},
	// sort
	canSort: function() {
		return false;
	}
});

dojo.provide("turbo.widgets.TurboGrid.builder");

dojo.declare("turbo.grid.builderBase", null, 	
	function(inGrid) {
		this.grid = inGrid;
	}, {
 	isCellNode: function(inNode) {
		return (inNode)&&(inNode.getAttribute)&&(Boolean(inNode.getAttribute(turbo.grid.CELL_INDEX_TAG)));
	},
	getCellNodeIndex: function(inCellNode) {
		return (inCellNode ? Number(inCellNode.getAttribute(turbo.grid.CELL_INDEX_TAG)) : -1);
	},
	getCellNode: function(inRowNode, inCellIndex) {
		for (var i=0, row; row=turbo.getTableRow(inRowNode.firstChild, i); i++)
			for (var j=0, cell; cell=row.cells[j]; j++)
				if (this.getCellNodeIndex(cell) == inCellIndex)
					return cell;
	},
	findTarget: function(inSourceNode, inTopNode) {
		var n = inSourceNode;
		while ((n)&&(!this.isCellNode(n))&&(n!=inTopNode))
			n = n.parentNode;
		return ((n!=inTopNode) ? n : null) 
	},
	decorateEvent: function(e) {
		e.cellNode = this.findTarget(e.target, e.rowNode);
		e.cellIndex = this.getCellNodeIndex(e.cellNode);
		return true;
	}
});

dojo.declare("turbo.grid.builder", turbo.grid.builderBase, {});

turbo.grid.CELL_INDEX_TAG = 'idx';


dojo.provide("turbo.widgets.TurboGrid.header");
dojo.require("turbo.widgets.TurboGrid.builder");
dojo.require("turbo.lib.drag");

ascendDom = function(inNode, inWhile) {
	var n = inNode;
/*	
	// best, but hard to match parens
	while ((n)&&(inWhile(n)))
	// whitespace
	while (n && inWhile(n))
	// can't read it
	while (n&&inWhile(n))
	// too much of a good thing
	while ((n) && (inWhile(n)))
*/	
	while (n && inWhile(n))
		n = n.parentNode;
	return n;
}

mkNotNode = function(inNode) {
	return function(n) { return n!=isNode; };
}

mkNotTagName = function(inTagName) {
	return function(n) { return turbo.getTagName(n)!=inTagName; };
}

getProp = function(inObjectOrNull, inProp) {
	return (inObjectOrNull)&&(inObjectOrNull[inProp])||null;
}

dojo.declare("turbo.grid.header", turbo.grid.builderBase, 
	function() {
	}, {
	bogusClickTime: 0,
	overResizeWidth: 4,
	getCellNode: function(inView, inCellIndex) {
		turbo.grid.header.superclass.getCellNode.apply(this, [ inView.headerContentNode, inCellIndex ]);
	},
	// event helpers
	getCellX: function(e) {
		var x = e.layerX;
		if ((dojo.render.html.moz)/*&&(turbo.getTagName(e.target) == "td")*/)
			x -= getProp(ascendDom(e.target, mkNotTagName("td")), "offsetLeft") || 0;
		var n = ascendDom(e.target, function() {
			if (!n || n == e.cellNode)
				return false;
			// Mozilla 1.8 (FF 1.5) has a bug that makes offsetLeft = -parent border width
			// when parent has border, overflow: hidden, and position: relative
			// handle this problem here ... not a general solution!
			x += (n.offsetLeft < 0 ? 0 : n.offsetLeft);
			return true;
		});
		/*																		 
		var n = e.target;
		while ((n)&&(n != e.cellNode)) {
			// Mozilla 1.8 (FF 1.5) has a bug that makes offsetLeft = -parent border width
			// when parent has border, overflow: hidden, and position: relative
			// handle this problem here ... not a general solution!
			x += (n.offsetLeft < 0 ? 0 : n.offsetLeft);
			n = n.parentNode;
		}
		*/
		return x;
	},
	// event decoration
	decorateEvent: function(e) {
		if (turbo.drag.dragging)
			return true;
		if ((e.type=="click")&&(turbo.time() < this.bogusClickTime)) 
			return true;
		if (!turbo.grid.header.superclass.decorateEvent.apply(this, arguments))
			return false;
		e.cellX = this.getCellX(e);
		e.cell = e.sourceView.layout.getCell(e.cellIndex);
		return true;
	},
	dispatchEvent: function(e) {
		if (e.dispatch in this) 
			return this[e.dispatch](e);
	},
	// event handlers
	// clicks
	/*doclick: function(e) {
		if (turbo.time() > this.bogusClickTime) 
			e.sourceView.grid.dispatchHeaderEvent(e);
	},*/
	// resizing
	prepareLeftResize: function(e) {
		var i = turbo.getCellIndex(e.cellNode);
		e.cellNode = (i ? e.cellNode.parentNode.cells[i-1] : null);
		e.cellIndex = (e.cellNode ? this.getCellNodeIndex(e.cellNode) : -1);
		return Boolean(e.cellNode);
	},
	canResize: function(e) {
		if ((!e.cellNode)||(e.cellNode.colSpan > 1))
			return false;
		var layoutCell = e.sourceView.layout.getCell(e.cellIndex);
		return (!layoutCell.noresize)&&(!turbo.grid.layout.isFlexColumn(layoutCell));
	},
	overLeftResizeArea: function(e)	{
		return (e.cellIndex>0)&&(e.cellX < this.overResizeWidth)&&(this.prepareLeftResize(e));
	},
	overRightResizeArea: function(e) {
		return (e.cellNode)&&(e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth);
	},
	/*
	// overs
	lastOverCellNode: null,
	domouseover: function(e) {
		if (e.cellNode != this.lastOverCellNode) {
			this.lastOverCellNode = e.cellNode;
			this.grid.onHeaderCellMouseOver(e);
		}
	},
	domouseout: function(e) {
		if (this.lastOverCellNode == e.target) {
			this.lastOverCellNode = null;
			this.grid.onHeaderCellMouseOut(e);	
		}
	},
	*/
	domousemove: function(e) {
		var c = (this.overRightResizeArea(e) ? 'e-resize' : (this.overLeftResizeArea(e) ? 'w-resize' : ''));
		if ((c)&&(!this.canResize(e))) c = 'not-allowed';
		e.sourceView.headerNode.style.cursor = c || ''; //'default';
	},
	domousedown: function(e) {
		if (!turbo.drag.dragging)
			if ((this.overRightResizeArea(e)||this.overLeftResizeArea(e))&&this.canResize(e))
				this.beginColumnResize(e);
			//else
			//	this.beginMoveColumn(e);
	},
	// column resizing
	beginColumnResize: function(e) {
		turbo.killEvent(e);
		var spanners = [ ], nodes = e.sourceView.layout.tableMap.findOverlappingNodes(e.cellNode);
		for (var i=0, cell; cell=nodes[i]; i++)
			spanners.push({ node: cell, index: this.getCellNodeIndex(cell), width: cell.offsetWidth });
		var drag = { 
			view: e.sourceView,
			node: e.cellNode,
			index: e.cellIndex, 
			w: e.cellNode.clientWidth,
			spanners: spanners
		};
		turbo.drag.start(e.cellNode, turbo.bindArgs(this, 'doResizeColumn', drag), turbo.bindArgs(this, 'endResizeColumn', drag), e);
	},
	minColWidth: 1,
	doResizeColumn: function(inDrag, inEvent)	{
		var w = inDrag.w + inEvent.deltaX;
		if (w >= this.minColWidth) {
			for (var i=0, s, sw; (s=inDrag.spanners[i]); i++) {
				sw = s.width + inEvent.deltaX;
				s.node.style.width = sw + 'px';
				inDrag.view.setColWidth(s.index, sw);
			}
			inDrag.node.style.width = w + 'px';
			inDrag.view.setColWidth(inDrag.index, w);
		}
		if (inDrag.view.layout.flexColumns)
			if (inDrag.view.layout.testFlexColumns() == false) {
				var t = turbo.getTable(inDrag.node);
				(t)&&(t.style.width = '');
			}
	},
	endResizeColumn: function(inDrag)	{
		this.bogusClickTime = turbo.time() + 30;
		turbo.defer(inDrag.view, "update", 50);
	}
});

dojo.provide("turbo.widgets.TurboGrid.layout");

dojo.declare("turbo.grid.layout", null,
	function(inView) {
		this.view = inView;
		this.dataRow = {
			index: -1
		}
	},{
	//defaultHeight: 4,
	defaultWidth: '6em',
	view: null,
	viewDef: null,
	cells: [],
	rows: null,
	tableMap: null,
	dataRow: null,	
	flexColumns: false,
	// public api
	setViewDef: function(inViewDef, inContextNode) {
		this.viewDef = inViewDef;
		this.contextNode = inContextNode;
		if ((inViewDef)&&(inViewDef.cells)) 
			this._loadDef();
	},
	_loadDef: function() {
		this.prepareRows();
		this.tableMap = new turbo.grid.tableMap(this.rows);
		this.update();
	},
	getCell: function(inIndex) {
		return this.cells[inIndex];
	},
	update: function() {
		if (!this.rows)
			return;
		//this.measure();
		this.testFlexColumns();
		this.prepareHtml();
	},
	// convert layout definition to internal structure
	prepareRows: function() {
		//this.flexColumns = false;
		this.cells = [ ];
		this.rows = [ ];
		var common = {
			view: this.view,
			grid: this.view.grid,
			getCellNode: this.cellGetCellNode,
			getContentNode: this.cellGetContentNode
		}
		dojo.lang.mixin(common, this.viewDef.defaultCell || {});
		var subrows = this.viewDef.subrows;
		for (var j=0, k=0, row, cells; (cells=subrows[j]); j++) {
			//console.log(j);
			row = [];
			row.def = cells;
			row.viewDef = this.viewDef;
			row.index = j;
			for (var i=0, cell, w, c; (cell=cells[i]); i++, k++) {
				if (cell.colSpan > 1)	
					w = 0;
				else if (!isNaN(cell.width))
					w = cell.width + "em";
				else 
					w = (cell.width)||(this.defaultWidth);
				c = dojo.lang.mixin({}, common, cell, {
					subrow: row,
					layoutIndex: k,
					def: cell,
					unitWidth: w,
					styles: cell.styles || ''
				});
				this.cells.push(c);
				row.push(c);
			}
			this.rows.push(row);
		}
		this.cellCount = this.cells.length;
	},
	testFlexColumns: function() {
		this.flexColumns = false;
		for (var j=0, row; (row=this.rows[j]); j++)
			for (var i=0, cell; (cell=row[i]); i++)
				if (turbo.grid.layout.isFlexColumn(cell)) {
					this.flexColumns = true;
					break;
				}
		return this.flexColumns;				
	},
	//: private
	// utility functions referenced by cell
	cellGetCellNode: function(inRowIndex) { 
		return this.view.getCellNode(inRowIndex, this.layoutIndex);
	},
	cellGetContentNode: function(inRowIndex) { 
		return this.getCellNode(inRowIndex);
	},
	// generate starting tags for a cell
	generateCellMarkup: function(inCell, inMoreStyles, inMoreClasses) {
		var result = [ ];
		var html = [ '<td' ];
		(inCell.colSpan)&&(html.push(' colspan="', inCell.colSpan, '"'));
		(inCell.rowSpan)&&(html.push(' rowspan="', inCell.rowSpan, '"'));
		html.push(' class="');
		(inCell.classes)&&(html.push(inCell.classes, ' '));
		(inMoreClasses)&&(html.push(inMoreClasses, ' '));
		// result[0] => td opener, style
		result.push(html.join(''));
		//
		// SLOT: result[1] => td classes 
		result.push('');
		//
		html = ['turbogrid-cell" idx="', inCell.layoutIndex, '" style="'];
		html.push(inCell.styles, inMoreStyles||'');
		(inCell.unitWidth)&&(html.push('width:', inCell.unitWidth, ';'));
		// result[2] => markup
		result.push(html.join(''));
		//
		// SLOT: result[3] => td style 
		result.push('');
		//
		html = [ '"' ];
		(inCell.attrs)&&(html.push(" ", inCell.attrs));
		html.push('>');
		// result[4] => td postfix
		result.push(html.join(''));
		//
		// SLOT: result[5] => content
		result.push('');
		//
		// result[6] => td closes
		result.push('</td>');
		return result;
	},
	// generate a custom table 
	generateHeaderTable: function(inGetValue, inValue) {
		this.dataRow.index = -1;
		(this.viewDef.onBeforeRow)&&(this.viewDef.onBeforeRow(-1, this.rows));
		var html = [ '<table class="turbogrid-row-table" border="0" cellspacing="0" cellpadding="0">' ];
		for (var j=0, row; (row=this.rows[j]); j++) {
			if (row.hidden)
				continue;
			html.push('<tr>');
			for (var i=0, cell, markup; (cell=row[i]); i++) {
				cell.customClasses = '';
				cell.customStyles = '';
				markup = this.generateCellMarkup(cell, cell.headerStyles, cell.headerClasses);
				// content
				markup[5] = (inValue != undefined ? inValue : inGetValue(this.dataRow, cell));
				// styles
				markup[3] = (cell.customStyles)||('');
				// classes
				markup[1] = (cell.customClasses ? ' ' + cell.customClasses : '');
				html.push(markup.join(''));
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join('');				
	},
	// cache html for rendering data rows
	prepareHtml: function() {
		var defaultGet = this.view.grid.get;
		for (var j=0, row; (row=this.rows[j]); j++)
			for (var i=0, cell; (cell=row[i]); i++) {
				cell.get = cell.get || defaultGet;
				cell.markup = this.generateCellMarkup(cell, cell.cellStyles, cell.cellClasses);
				cell.customClasses = cell.customStyles = '';
			}
	},
	// time-critical: generate html using cache and data source
	generateDataTable: function(inDataIndex, inRowIndex) {
		this.dataRow.index = inDataIndex;
		(this.view.onBeforeRow)&&(this.view.onBeforeRow(inDataIndex, this.rows));
		var html = [ '<table class="turbogrid-row-table" border="0" cellspacing="0" cellpadding="0">' ];
		for (var j=0, row; (row=this.rows[j]); j++) {
			if (row.hidden)
				continue;
			html.push('<tr>');
			for (var i=0, cell; (cell=row[i]); i++) {
				with (cell) {
					customClasses = '';
					customStyles = '';
					// content
					markup[5] = (cell.get)&&(get(this.dataRow, cell)) || (cell.value);
					// classes
					markup[1] = (customClasses ? ' ' + customClasses : '');
					// styles
					markup[3] = customStyles;
					//if (cell.cellStyles) console.log(markup.join(''));
					html.push.apply(html, markup);
					//html.push(markup.join(''));
				}
				//if (cell.cellStyles) console.log(cell.markup.join(''));
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join('');
	}
});

turbo.grid.layout.isFlexColumn = function(inCell) {
	return ((inCell.unitWidth)&&((inCell.unitWidth == 'auto')||(inCell.unitWidth.slice(-1) == '%')));
}

dojo.declare("turbo.grid.tableMap", null, 
	function(inRows) {
		this.mapRows(inRows);
	},{
	map: null,
	// map table topography
	mapRows: function(inRows) {
		//console.log('mapRows');
		// # of rows
		var rowCount = inRows.length;
		// count # of cells in first row
		// we assume the table is square
		var colCount = 0, row = inRows[0];
		for (var i=0, cell; (cell=row[i]); i++)
			colCount += (cell.def.colSpan)||(1);
		// map which columns and rows fill which cells
		this.map = [ ];
		for (var j=0, row; (row=inRows[j]); j++)
			this.map[j] = [];
		for (var j=0, row; (row=inRows[j]); j++)
			for (var i=0, x=0, cell, colSpan, rowSpan; (cell=row[i]); i++) {
				while (this.map[j][x]) x++;
				this.map[j][x] = { c: i, r: j };
				rowSpan = (cell.def.rowSpan)||(1);
				colSpan = (cell.def.colSpan)||(1);
				for (var y=0; y<rowSpan; y++) 
					for (var s=0; s<colSpan; s++)
						this.map[j+y][x+s] = this.map[j][x];
				x += colSpan;
			}
		if (0) // dump
			for (var j=0, row; (row=this.map[j]); j++) {
				var h = '';
				for (var i=0, cell; (cell=row[i]); i++)
					h += cell.r + ',' + cell.c + '   ';
				console.log(h);
			}
	},
	// find node's map coords by it's structure coords
	getMapCoords: function(inRow, inCol) {
		for (var j=0, row; (row=this.map[j]); j++)
			for (var i=0, cell; (cell=row[i]); i++) 
				if ((cell.c == inCol)&&(cell.r == inRow))
					return { j: j, i: i };
				//else console.log(inRow, inCol, ' : ', i, j, " : ", cell.r, cell.c);
		return { j: -1, i: -1 };					
	},
	// find a node in inNode's table with the given structure coords
	getNode: function(inTable, inRow, inCol) {
		var row = (inTable)&&(inTable.rows[inRow]);
		return (row)&&(row.cells[inCol]);
	},
	_findOverlappingNodes: function(inTable, inRow, inCol) {
		var nodes = [ ];
		var m = this.getMapCoords(inRow, inCol);
		//console.log("node j: %d, i: %d", m.j, m.i);
		var row = this.map[m.j];
		for (var j=0, row; (row=this.map[j]); j++) {
			if (j == m.j) continue;
			//********************************************************************			
			// TOQUE POR INCIDENTE N� 2233
			if (row[m.i] === undefined) {
				continue;
			}
			//********************************************************************
			with (row[m.i]) {
				//console.log("overlaps: r: %d, c: %d", r, c);
				var n = this.getNode(inTable, r, c);
				if (n) nodes.push(n);
			}
		}
		//console.log(nodes);
		return nodes;
	},
	findOverlappingNodes: function(inNode) {
		//console.log("layout %o findOverlappingNodes", this);
		var r = turbo.getRowIndex(inNode.parentNode);
		var c = turbo.getCellIndex(inNode);
		//console.log("node r: %d, c: %d", r, c);
		return this._findOverlappingNodes(turbo.getTable(inNode), r, c);
	}
});
dojo.provide("turbo.widgets.TurboGrid.view");
dojo.require("turbo.widgets.TurboGrid.builder");
dojo.require("turbo.widgets.TurboGrid.header");
dojo.require("turbo.widgets.TurboGrid.layout");

turbo.grid.rowIndexTag = "gridRowIndex";

dojo.widget.defineWidget('dojo.widget.TurboGridView', dojo.widget.html.TurboWidget, 
	function() {
		this.rowNodes = [ ];
	}, {
	//: public
	defaultWidth: "18em",
	viewWidth: "",												 
	//: private
	templateString: '<div class="turbogrid-view"><div class="turbogrid-header" dojoAttachPoint="headerNode"><div style="width: 9000em"><div dojoAttachPoint="headerContentNode"></div></div></div><div class="turbogrid-scrollbox" dojoAttachPoint="scrollboxNode"><div class="turbogrid-content" dojoAttachPoint="contentNode" hidefocus="hidefocus"></div></div></div>',
	themeable: false,	
	classTag: 'turbogrid',
	colCount: 0,
	cells: null,		
	marginBottom: 0,
	rowPad: 2,
	viewDef: null,
	fillInTemplate: function() {
		dojo.event.browser.addListener(this.scrollboxNode, "onscroll", turbo.bind(this, "doscroll"));
		turbo.funnelEvents(this.contentNode, this, "doContentEvent", [ 'mouseover', 'mouseout', 'click', 'dblclick', 'contextmenu' ]);
		turbo.funnelEvents(this.headerNode, this, "doHeaderEvent", [ 'dblclick', 'mouseover', 'mouseout', 'mousemove', 'mousedown', 'click', 'contextmenu' ]);
		this.layout = new turbo.grid.layout(this);
		dojo.html.disableSelection(this.contentNode);
	},
	destroy: function() {
		turbo.remove(this.headerNode);
		this.inherited("destroy");
	},
	//: protected
	setViewDef: function(inViewDef) {
		// fixup for backward compat
		inViewDef.cells = inViewDef.subrows = inViewDef.subrows||inViewDef.cells;
		this.viewDef = inViewDef;
		if ((inViewDef.width)&&(dojo.lang.isNumber(inViewDef.width)))
			this.viewWidth = inViewDef.width + 'em';
		else
			this.viewWidth = inViewDef.width || this.viewWidth ; //|| this.defaultWidth;
		this.colCount = 0;
		this.cells = inViewDef.cells||inViewDef.subrows;
		if (this.cells)
			for (var i=0, r; (r=this.cells[i]); i++)
				for (var j=0, c; (c=r[j]); j++, this.colCount++);
		this.onBeforeRow = inViewDef.onBeforeRow;				
		this.layout.setViewDef(inViewDef, this.contentNode);
		this.noscroll = inViewDef.noscroll;
		if (this.noscroll)
			this.scrollboxNode.style.overflow = "hidden";
	},
	getScrollbarWidth: function() {
		return (this.noscroll ? 0 : turbo.getScrollbarWidth());
	},
	getColumnsWidth: function() {
		return this.headerContentNode.firstChild.offsetWidth;
		//return Math.round(this.grid.builder.getWidth(this.cells) * this.grid.contentPixelToEmRatio) + 2;
	},
	getWidth: function() {
		return (this.viewWidth)||((this.getColumnsWidth()+this.getScrollbarWidth()) +'px');
	},
	getContentWidth: function() {
		return Math.max(0, dojo.html.getContentBox(this.domNode).width - this.getScrollbarWidth()) + 'px';
	},
	render: function() {
		this.scrollboxNode.style.height = '';
		this.renderHeader();
	},
	renderHeader: function() {


		this.headerContentNode.innerHTML = this.layout.generateHeaderTable(this._getHeaderContent);
	},
	// note: not called in 'view' context
	_getHeaderContent: function(inDataIndex, inCell) {
		var h = inCell.name;
		if (inCell.index != inCell.grid.getSortIndex())
			return h;
		h = [ '<div class="', '', '">', h, '</div>' ];
		if (inCell.grid.sortInfo > 0)
			h[1] = 'turbogrid-sort-down';
		else
			h[1] = 'turbogrid-sort-up';
		return h.join('');
	},
	resize: function() {
		this.resizeHeight();
		this.resizeWidth();
	},
	hasScrollbar: function() {
		return (this.scrollboxNode.clientHeight != this.scrollboxNode.offsetHeight);
	},
	resizeHeight: function() {
		if (!this.grid.autoHeight)  {
			var h = this.domNode.clientHeight;
			if (!this.hasScrollbar()) // no scrollbar is rendered
				h -= turbo.getScrollbarWidth();
			turbo.setStyleHeightPx(this.scrollboxNode, h);
		}
	},
	resizeWidth: function() {
		if (this.layout.flexColumns) {
			// the view content width
			this.contentWidth = this.getContentWidth();
			this.headerContentNode.firstChild.style.width = this.contentWidth;
		}
		// FIXME: it should be easier to get w from this.scrollboxNode.clientWidth, 
		// but clientWidth seemingly does not include scrollbar width in some cases
		var w = this.scrollboxNode.offsetWidth - this.getScrollbarWidth();
		w = Math.max(w, this.getColumnsWidth()) + 'px';
		with (this.contentNode) {
			style.width = '';
			offsetWidth;
			style.width = w;
		}
	},
	setSize: function(w, h) {
		with (this.domNode.style) {
			width = w;
			height = (h >= 0 ? h + 'px' : '');
		}
		with (this.headerNode.style) {
			width = w;
		}
	},
	// FIXME: exchange 'render' and 'build' idioms
	renderRow: function(inRowIndex, inHeightPx) {
		var rowNode = this.createRowNode(inRowIndex);
		this.buildRow(inRowIndex, rowNode, inHeightPx);
		return rowNode;
	},
	createRowNode: function(inRowIndex) {
		var node = document.createElement("div");
		node.className = this.classTag + '-row';
		node[turbo.grid.rowIndexTag] = inRowIndex;
		this.rowNodes[inRowIndex] = node;
		return node;
	},
	buildRow: function(inRowIndex, inRowNode) {
		this.buildRowContent(inRowIndex, inRowNode);
		this.styleRow(inRowIndex, inRowNode);
	},
	buildRowContent: function(inRowIndex, inRowNode) {
		inRowNode.innerHTML = this.layout.generateDataTable(inRowIndex); // data index
		if (this.layout.flexColumns) {
			// FIXME: accessing firstChild here breaks encapsulation
			inRowNode.firstChild.style.width = this.contentWidth;
		}
	},
	rowRemoved: function(inRowIndex) {
		delete this.rowNodes[inRowIndex];
	},
	getRowNode: function(inRowIndex) {
		return this.rowNodes[inRowIndex];
	},
	getCellNode: function(inRowIndex, inCellIndex) {
		var row = this.getRowNode(inRowIndex);
		return (row ? this.grid.builder.getCellNode(row, inCellIndex) : null);
	},
	// styling
	styleRow: function(inRowIndex, inRowNode) {
		inRowNode._style = turbo.getStyleText(inRowNode);
		this.styleRowNode(inRowIndex, inRowNode);
	},
	styleRowNode: function(inRowIndex, inRowNode) {
		(inRowNode)&&(this.doStyleRowNode(inRowIndex, inRowNode));
	},
	doStyleRowNode: function(inRowIndex, inRowNode) {
		this.grid.styleRowNode(inRowIndex, inRowNode, this.cells);
	},
	// updating
	updateRow: function(inRowIndex, inHeightPx, inPageNode) {
		var rowNode = this.getRowNode(inRowIndex);
		if (rowNode) {
			rowNode.style.height = '';
			this.buildRow(inRowIndex, rowNode);
		}
		return rowNode;
	},
	updateRowStyles: function(inRowIndex) {
		this.styleRowNode(inRowIndex, this.getRowNode(inRowIndex));
	},
	// scrolling
	lastTop: 0,
	doscroll: function(inEvent) {
		this.headerNode.scrollLeft = this.scrollboxNode.scrollLeft;
		// 'lastTop' is a semaphore to prevent feedback-loop with setScrollTop below
		var top = this.scrollboxNode.scrollTop;
		if (top != this.lastTop)
			this.grid.scrollTo(top);
	},
	setScrollTop: function(inTop) {
		// 'lastTop' is a semaphore to prevent feedback-loop with doScroll above
		this.lastTop = inTop;
		this.scrollboxNode.scrollTop = inTop;
		return this.scrollboxNode.scrollTop;
	},
	// event helpers
	findTarget: function(inSource, inTag) {
		var n = inSource;
		while ((n)&&!(inTag in n)&&(n!=this.domNode))
			n = n.parentNode;
		return (n != this.domNode ? n : null) 
	},
	findRowTarget: function(inSource) {
		return this.findTarget(inSource, turbo.grid.rowIndexTag);
	},
	isIntraNodeEvent: function(e) {
		try {
			return (e.cellNode && e.relatedTarget && dojo.dom.isDescendantOf(e.relatedTarget, e.cellNode));
		} catch(x) { 
			// e.relatedTarget has permission problem in FF if it's an input: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
			return false;	
		}
	},
	isIntraRowEvent: function(e) {
		try {	
			var row = (e.relatedTarget)&&(this.findRowTarget(e.relatedTarget));
			return (!row)&&(e.rowIndex==-1) || (row)&&(e.rowIndex==row.gridRowIndex);			
		} catch(x) { 
			// e.relatedTarget has permission problem in FF if it's an input: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
			return false;
		}
	},
	// DOM event handlers
	doContentEvent: function(e) {
		if (this.decorateContentEvent(e))
			this.grid.onContentEvent(e);
	},
	doHeaderEvent: function(e) {
		if (this.decorateHeaderEvent(e))
			this.grid.onHeaderEvent(e);
	},
	// event decoration
	decorateEvent: function(e) {
		e.sourceView = this;
		e.dispatch = 'do' + e.type;
	},
	decorateContentEvent: function(e) {
		this.decorateEvent(e);
		e.rowNode = this.findRowTarget(e.target);
		if (!e.rowNode) return false;
		e.rowIndex = e.rowNode[turbo.grid.rowIndexTag];
		this.grid.builder.decorateEvent(e);
		e.cell = this.layout.getCell(e.cellIndex);
		return true;
	},
	decorateHeaderEvent: function(e) {
		this.decorateEvent(e);
		e.rowIndex = -1;
		return this.grid.header.decorateEvent(e);
	},
	// event dispatch
	dispatchEvent: function(m, e) {
		if (m in this) 
			return this[m](e);
	},
	dispatchContentEvent: function(e) {
		this.dispatchEvent(e.dispatch, e);
	},
	dispatchHeaderEvent: function(e) {
		this.dispatchEvent(e.dispatch, e);
	},
	// dispatched event handlers
	domouseover: function(e) {
		if ((e.cellNode)&&(e.cellNode!=this.lastOverCellNode)) {
			this.lastOverCellNode = e.cellNode;
			this.grid.onMouseOver(e);
		}
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e) {
		if ((e.cellNode)&&(e.cellNode==this.lastOverCellNode)&&(!this.isIntraNodeEvent(e, this.lastOverCellNode))) {
			this.lastOverCellNode = null;
			this.grid.onMouseOut(e);	
			if (!this.isIntraRowEvent(e))
				this.grid.onMouseOutRow(e);
		}
	},
	// column resizing
	setColWidth: function(inIndex, inWidth) {
		var cell = this.layout.getCell(inIndex);
		cell.unitWidth = inWidth + 'px';
	},
	update: function() {
		var left = this.scrollboxNode.scrollLeft;
		this.layout.update();
		this.grid.update();
		this.scrollboxNode.scrollLeft = left;
	},
	// focus renderer
	focusClass: "turbogrid-cell-focus",
	borkNode: function(inRowIndex, inCellIndex, inBork) {
		turbo.addRemoveClass(this.getCellNode(inRowIndex, inCellIndex), this.focusClass, inBork);	
	},
	blurCell: function(inRowIndex, inCellIndex) {
		this.borkNode(inRowIndex, inCellIndex, false);
	},
	focusCell: function(inRowIndex, inCellIndex) {
		this.borkNode(inRowIndex, inCellIndex, true);
	}
});
dojo.provide("turbo.widgets.TurboGrid.views");

dojo.declare('turbo.grid.views', null, 
	function(inGrid) {
		this.grid = inGrid;
	}, {	
	defaultWidth: 200,
	views: [ ],						 	
	// operations
	resize: function() {
		this.onEach("resize");
	},
	render: function() {
		this.onEach("render");
		this.normalizeHeaderNodeHeight();
	},
	// views
	addView: function(inView) {
		inView.idx = this.views.length;
		this.views.push(inView);
	},
	destroyViews: function() {
		for (var i=0, v; v=this.views[i]; i++)
			v.destroy();
		this.views = [];
	},
	getContentNodes: function() {
		var nodes = [ ];
		for (var i=0, v; v=this.views[i]; i++)
			nodes.push(v.contentNode);
		return nodes;
	},
	forEach: function(inCallback) {
		for (var i=0, v; v=this.views[i]; i++)
			inCallback(v, i);
	},
	onEach: function(inMethod, inArgs) {
		inArgs = inArgs || [ ];
		for (var i=0, v; v=this.views[i]; i++)
			if (inMethod in v) 
				v[inMethod].apply(v, inArgs);
	},
	// layout
	normalizeHeaderNodeHeight: function() {
		var rowNodes = [ ];
		for (var i=0, v; (v=this.views[i]); i++)
			(v.headerContentNode.firstChild)&&(rowNodes.push(v.headerContentNode));
		this.normalizeRowNodeHeights(rowNodes);
	},
	normalizeRowNodeHeights: function(inRowNodes) {
		var h = 0; 
		for (var i=0, n, o; (n=inRowNodes[i]); i++)
			h = Math.max(h, (n.firstChild.clientHeight)||(n.firstChild.offsetHeight));
		h = (h >= 0 ? h : 0) + 'px';
		//
		for (var i=0, n; (n=inRowNodes[i]); i++)
			n.firstChild.style.height = h;
		//
		//console.log('normalizeRowNodeHeights ', h);
		//
		// querying the height here seems to help scroller measure the page on IE
		(inRowNodes[0])&&(inRowNodes[0].parentNode.offsetHeight);
	},
	renormalizeRow: function(inRowIndex) {
		var rowNodes = [];
		for (var i=0, v, n; (v=this.views[i])&&(n=v.getRowNode(inRowIndex)); i++) {
			n.firstChild.style.height = '';
			rowNodes.push(n);
		}
		/*
		var h = 0; 
		for (var i=0, n, o; (n=rowNodes[i]); i++)
			h = Math.max(h, (n.firstChild.clientHeight)||(n.firstChild.offsetHeight));
		h = (h >= 0 ? h : 0) + 'px';
		console.log('** renormalize row ', inRowIndex, '=', h);
		for (var i=0, n; (n=rowNodes[i]); i++)
			console.log(n.firstChild.style.height);
		*/
		this.normalizeRowNodeHeights(rowNodes);
	},
	getViewWidth: function(inIndex) {
		return this.views[inIndex].getWidth() || this.defaultWidth;
	},
	measureHeader: function() {
		this.forEach(function(inView) {
			inView.headerContentNode.style.height = '';
		});
		var h = 0;
		this.forEach(function(inView) {
			h = Math.max(inView.headerNode.offsetHeight, h);
		});
		return h;
	},
	measureContent: function() {
		var h = 0;
		this.forEach(function(inView) {	h = Math.max(inView.domNode.offsetHeight, h); });
		return h;
	},
	findClient: function(inAutoWidth) {
		// try to use user defined client
		var c = this.grid.elasticView || -1;
		// attempt to find implicit client
		if (c < 0)
			for (var i=1, v; (v=this.views[i]); i++) {
				if (v.viewWidth) {
					for (i=1; (v=this.views[i]); i++)
						if (!v.viewWidth) {
							c = i;
							break;
						}
					break;
				}
			}
		// client is in the middle by default
		if (c < 0)
			c = Math.floor(this.views.length / 2);
		return c;
	},
	_arrange: function(l, t, w, h) {
		var i, v, vw, len = this.views.length;
		// find the client
		var c = (w <= 0 ? len : this.findClient());
		// layout views
		var setPosition	= function(v, l, t) {
			with (v.domNode.style) {
				left = l + 'px';
				top = t + 'px';
			}
			with (v.headerNode.style) {
				left = l + 'px';
				top = 0;
			}
		}
		// for views left of the client
		for (i=0; (v=this.views[i])&&(i<c); i++) {
			// get width
			vw = this.getViewWidth(i);
			// process boxes
			v.setSize(vw, h);
			setPosition(v, l, t);
			vw = v.domNode.offsetWidth;
			// update position
			l += vw;
		}
		// next view (is the client, i++ == c) 
		i++;
		// start from the right edge
		var r = w;
		// for views right of the client (iterated from the right)
		for (var j=len-1; (v=this.views[j])&&(i<=j); j--) {
			// get width
			vw = this.getViewWidth(j);
			// set size
			v.setSize(vw, h);
			// measure in pixels
			vw = v.domNode.offsetWidth;
			// update position
			r -= vw;
			// set position
			setPosition(v, r, t);
		}
		if (c<len) {
			v = this.views[c];
			// position the client box between left and right boxes	
			vw = Math.max(1, r-l);
			// set size
			v.setSize(vw + 'px', h);
			setPosition(v, l, t);
		}
		return l;
	},
	arrange: function(l, t, w, h) {
		var w = this._arrange(l, t, w, h);
		this.resize();
		return w;
	},
	// rendering
	renderRow: function(inRowIndex, inNodes) {
		var rowNodes = [ ];
		for (var i=0, v, n, rowNode; (v=this.views[i])&&(n=inNodes[i]); i++) {
			rowNode = v.renderRow(inRowIndex);
			turbo.append(rowNode, n);
			rowNodes.push(rowNode);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	rowRemoved: function(inRowIndex) {
		this.onEach("rowRemoved", [ inRowIndex ]);
	},
	// updating
	updateRow: function(inRowIndex, inHeight) {
		for (var i=0, v; v=this.views[i]; i++)
			v.updateRow(inRowIndex, inHeight);
		this.renormalizeRow(inRowIndex);
	},
	updateRowStyles: function(inRowIndex) {
		this.onEach("updateRowStyles", [ inRowIndex ]);
	},
	// scrolling
	setScrollTop: function(inTop) {
		var top = inTop;
		for (var i=0, v; v=this.views[i]; i++)
			top = v.setScrollTop(inTop);
		return top;
		//this.onEach("setScrollTop", [ inTop ]);
	},
	// indexing
	cellToColIndex: function(inCellIndex, inView) {
		var result = inCellIndex;
		for (var i=0, v; (v=this.views[i])&&(v!=inView); i++)
			result += ((v.cells)&&(v.colCount))||0;
		return result;
	},
	colToCellIndex: function(inColIndex) {
		for (var i=0, ci=inColIndex, v; (v=this.views[i]); i++)
			if (v.cells) {
				if (ci < v.colCount)
					return { view: v, index: ci };
				ci -= v.colCount;					
			}
		return {};
	},
	getFirstScrollingView: function() {
		for (var i=0, v; (v=this.views[i]); i++)
			if (v.hasScrollbar())
				return v;
	}
	//,
	// events
	//decorateEvent: function(inEvent) {
	//	inEvent.colIndex = this.cellToColIndex(inEvent.cellIndex, inEvent.sourceView);
	//}
});
dojo.provide("turbo.widgets.TurboGrid.rows");

dojo.declare("turbo.grid.rows", null, 
	function(inGrid) {
		this.grid = inGrid;
	}, {
	linesToEms: 2,
	defaultRowHeight: 1, // lines
	overRow: -2,
	// metrics
	getHeight: function(inRowIndex) {
		return '';
	},
	getDefaultHeightPx: function() {
		return 32;
		//return Math.round(this.defaultRowHeight * this.linesToEms * this.grid.contentPixelToEmRatio);
	},
	// styles
	prepareStylingRow: function(inRowIndex, inRowNode) {
		return { 
			index: inRowIndex, 
			node: inRowNode,
			odd: Boolean(inRowIndex&1),
			selected: this.grid.selection.isSelected(inRowIndex),
			over: this.isOver(inRowIndex),
			customStyles: "",
			customClasses: "turbogrid-row"
		}
	},
	styleRowNode: function(inRowIndex, inRowNode) {
		var row = this.prepareStylingRow(inRowIndex, inRowNode);
		this.grid.onStyleRow(row);
		this.applyStyles(row);
	},
	applyStyles: function(inRow) {
		with (inRow) {
			node.className = customClasses;
			var h = node.style.height;
			turbo.setStyleText(node, customStyles + ';' + (node._style||''));
			node.style.height = h;
		}
	},
	updateStyles: function(inRowIndex) {
		this.grid.updateRowStyles(inRowIndex);
	},
	// states and events
	setOverRow: function(inRowIndex) {
		var last = this.overRow;
		this.overRow = inRowIndex;
		if ((last!=this.overRow)&&(last >=0))
			this.updateStyles(last);
		this.updateStyles(this.overRow);
	},
	isOver: function(inRowIndex) {
		return (this.overRow == inRowIndex);
	}
});
dojo.provide("turbo.widgets.TurboGrid.focus");

// focus management
dojo.declare("turbo.grid.focus", null,
 	function(inGrid) {
		this.grid = inGrid;
		this.info = {};
		this.cell = {row: 0, col: -1};
		this.canBlur = {previous: false, next: false};
	}, {
	isLastFocusCell: function() {
		//********************************************************************		
		// TOQUE POR INCIDENTE N� 2239		
		if (this.grid === undefined || this.grid === null) {
			return;
		}
		return (this.cell.row == this.grid.rowCount-1 && this.cell.col == this.grid.colCount-1);
		//********************************************************************		
	},
	isFirstFocusCell: function() {
		return (this.cell.row == 0 && this.cell.col == 0);
	},
	isNoFocusCell: function() {
		return (this.cell.row < 0 || this.cell.col < 0);
	},
	resetBlur: function() {
		this.canBlur.next = false;
		this.canBlur.previous = false;
	},
	doSetFocus: function(inMethod) {
		if (this.info.view)
			with (this.info) 
				view[inMethod].call(view, this.cell.row, index);
	},
	setFocusCell: function(inRowIndex, inColIndex) {
		this.doSetFocus('blurCell');
		this.cell = { row: inRowIndex, col: inColIndex };
		this.info = this.grid.views.colToCellIndex(inColIndex);
		this.doSetFocus('focusCell');
		//this.grid.edit.editCellNode(inRowIndex, inColIndex);
		this.resetBlur();
		this.grid.onCellFocus(inRowIndex, inColIndex);
	},
	next: function() {
		var row = this.cell.row;
		var col = this.cell.col + 1;
		if (col >= this.grid.colCount) {
			col = 0;
			row++;
		}
		if (row > this.grid.rowCount-1) {
			col = this.grid.colCount - 1;
			row = this.grid.rowCount - 1;
		}
		this.setFocusCell(row, col);
	},
	previous: function() {
		var row = (this.cell.row || 0);
		var col = (this.cell.col || 0) - 1;
		if (col < 0) {
			col = this.grid.colCount-1;
			row--;
		}
		if (row < 0) {
			row = 0;
			col = 0;
		}
		this.setFocusCell(row, col);
	}
});
dojo.provide('turbo.widgets.TurboGrid.selection');

dojo.declare("turbo.grid.selection", null, 
	function(inGrid) {
		this.grid = inGrid;
		this.selected = [ ];
	},{
	multiSelect: true,
	selected: null,
	selectedIndex: -1,
	onCanSelect: function(inIndex) { 
		return this.grid.canSelect(inIndex);
	},
	onCanDeselect: function(inIndex) {
		return this.grid.canDeselect(inIndex);
	},
	onSelected: function(inIndex) {
		return this.grid.selected(inIndex);
	},
	onDeselected: function(inIndex) {
		return this.grid.deselected(inIndex);
	},
	//onSetSelected: function(inIndex, inSelect) { };
	//onChanging: function() { };
	onChanged: function() {
		//return this.grid.selectionChanged();
	},
	isSelected: function(inIndex) 	{
		return this.selected[inIndex];
	},
	getFirstSelected: function() {
		for (var i=0, l=this.selected.length; i<l; i++)
			if (this.selected[i])
				return i;
		return -1;
	},
	getNextSelected: function(inPrev) {
		for (var i=inPrev+1, l=this.selected.length; i<l; i++)
			if (this.selected[i])
				return i;
		return -1;
	},
	getSelected: function() {
		var result = [ ];
		for (var i=0, l=this.selected.length; i<l; i++)
			if (this.selected[i])
				result.push(i);
		return result;
	},
	getSelectedCount: function() {
		var c = 0;
		for (var i=0; i<this.selected.length; i++)
			if (this.selected[i])
				c++;
		return c;
	},
	beginUpdate: function() { 
		if (this.updating == 0)
			this.onChanging();
		this.updating++;
	},
	endUpdate: function() { 
		this.updating--;
		if (this.updating == 0)
			this.onChanged();
	},
	select: function(inIndex)	{
		this.unselectAll(inIndex);
		this.addToSelection(inIndex);
	},
	addToSelection: function(inIndex)	{
		inIndex = Number(inIndex);
		if (this.selected[inIndex])
			this.selectedIndex = inIndex;
		else {
			if (this.onCanSelect(inIndex) !== false) {
				this.selectedIndex = inIndex;
				this.beginUpdate();
				this.selected[inIndex] = true;
				this.grid.selected(inIndex);
				//this.onSelected(inIndex);
				//this.onSetSelected(inIndex, true);
				this.endUpdate();
			}
		}
	},
	deselect: function(inIndex)	{
		inIndex = Number(inIndex);
		if (this.selectedIndex == inIndex)
			this.selectedIndex = -1;
		if (this.selected[inIndex])	{
			if (this.onCanDeselect(inIndex) === false)
				return;
			this.beginUpdate();
			delete this.selected[inIndex];
			this.grid.deselected(inIndex);
			//this.onDeselected(inIndex);
			//this.onSetSelected(inIndex, false);
			this.endUpdate();
		}
	},
	setSelected: function(inIndex, inSelect) {
		this[(inSelect ? 'addToSelection' : 'deselect')](inIndex);
	},
	toggleSelect: function(inIndex)	{
		this.setSelected(inIndex, !this.selected[inIndex])
	},
	insert: function(inIndex)	{
		this.selected.splice(inIndex, 0, false);
		if (this.selectedIndex >= inIndex)
			this.selectedIndex++;
	},
	remove: function(inIndex)	{
		this.selected.splice(inIndex, 1);
		if (this.selectedIndex >= inIndex)
			this.selectedIndex--;
	},
	unselectAll: function(inExcept)	{
		for (var i in this.selected)
			if ((i!=inExcept)&&(this.selected[i]===true))
				this.deselect(i);
	},
	shiftSelect: function(inFrom, inTo)	{
		var s = (inFrom >= 0 ? inFrom : inTo);
		var e = inTo;
		if (s > e) {
			e = s;
			s = inTo;
		}
		for (var i=s; i<=e; i++)
			this.addToSelection(i);
	},
	clickSelect: function(inIndex, inCtrlKey, inShiftKey)	{
		this.beginUpdate();
		if (!this.multiSelect) {
			this.select(inIndex);
		} else {
			var lastSelected = this.selectedIndex;
			if (!inCtrlKey) 
				this.unselectAll(inIndex);
			if (inShiftKey) 
				this.shiftSelect(lastSelected, inIndex);
			else //if (inCtrlKey)
				this.toggleSelect(inIndex);
			//else
			//	this.addToSelection(inIndex)
		}
		this.endUpdate();
	},
	clickSelectEvent: function(e)	{
		this.clickSelect(e.rowIndex, e.ctrlKey, e.shiftKey);
	},
	clear: function() {
		this.beginUpdate();
		this.unselectAll();
		this.endUpdate();
	}
});
dojo.provide("turbo.widgets.TurboGrid.edit");

dojo.declare("turbo.grid.edit", null, 
	function(inGrid) {
		this.grid = inGrid;
	}, {
	shouldEdit: function(e) {
		return ((!e.ctrlKey)&&(!e.shiftKey));
	},
	doclick: function(e) {		
		if (!this.shouldEdit(e))
			return;
		this.editCellNode(e.rowIndex, e.colIndex);
	},
	editCellNode: function(inRowIndex, inColIndex) {
		this.applyEdit();
		this.editCell = this.grid.views.colToCellIndex(inColIndex);
		this.editCell.col = inColIndex;
		this.editCell.row = inRowIndex;
		this.editNode = (this.editCell.view)&&(this.editCell.view.getCellNode(this.editCell.row, this.editCell.index));
		if (!this.editNode) {
			this.cancelEdit();
			return;
		}
		this.editBak = this.editNode.innerHTML;
		this.editNode.innerHTML = "[EDITOR GOES HERE]";
		/*	
		// editor.edit might need this info
		this.setEditCell(inRowIndex, inColIndex);
		// get an editor
		this.editor = (inEditor || this.getEditor(inRowIndex, inColIndex));
		var value = (inValue || this.getValue(inRowIndex, inColIndex));
		if (this.editor && this.editor.edit(editNode, value, inEvent) === false)
			this.editor = null;
		if (!this.editor)
			this.grid.editRow = -1; 
		else
			this.grid.editStarted();
		this.updateRowStyles(inRowIndex);
		*/
	},
	saveState: function(inRowIndex) {
		if ((this.editNode)&&(this.editCell.row==inRowIndex)&&(this.edtiHtml))
			this.editHtml = this.editNode.innerHTML;
	},
	restoreState: function(inRowIndex) {
		if ((this.editNode)&&(this.editCell.row==inRowIndex)) {
			this.editNode.innerHTML = this.editHtml;
			this.editHtml = '';
		}
	},
	editDone: function() {
		if (this.editNode)
			this.editNode.innerHTML = this.editBak;
		this.editNode = null;
	},
	applyEdit: function() {
		this.editDone();
	},
	cancelEdit: function() {
		this.editDone();
	}
});
dojo.provide("turbo.widgets.TurboGrid.rowbar");
dojo.require("turbo.widgets.TurboGrid.view");

dojo.widget.defineWidget('dojo.widget.TurboGridRowView', dojo.widget.TurboGridView, {
	defaultWidth: "3em",				
	noscroll: true,
	padBorderWidth: 2, // FIXME: magic number
	fillInTemplate: function() {
		this.inherited("fillInTemplate");
		this.scrollboxNode.style.overflow = "hidden";
		dojo.html.setVisibility(this.headerNode, false);
	},	
	getWidth: function() {
		return (this.viewWidth)||(this.defaultWidth);
	},
	buildRowContent: function(inRowIndex, inRowNode) {
		var w = this.contentNode.offsetWidth - this.padBorderWidth 
		inRowNode.innerHTML = '<div style="width:' + w + 'px; height: 8px;"><div class="turbogrid-rowbar-inner"></div></div>';
	},
	renderHeader: function() {
	},
	resize: function() {
		this.resizeHeight();
	},	
	// styling
	doStyleRowNode: function(inRowIndex, inRowNode) {
		var n = [ "turbogrid-rowbar" ];
		if (this.grid.rows.isOver(inRowIndex))
			n.push("turbogrid-rowbar-over")
		if (this.grid.selection.isSelected(inRowIndex))
			n.push("turbogrid-rowbar-selected")
		inRowNode.className = n.join(" ");
	},
	// event handlers
	domouseover: function(e) {
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e) {
		if (!this.isIntraRowEvent(e))
			this.grid.onMouseOutRow(e);
	}
});
dojo.require("turbo.lib.more");
dojo.require("turbo.lib.scroller");
dojo.require("turbo.widgets.TurboWidget");
dojo.require("turbo.widgets.TurboGrid.view");
dojo.require("turbo.widgets.TurboGrid.views");
dojo.require("turbo.widgets.TurboGrid.rows");
dojo.require("turbo.widgets.TurboGrid.focus");
dojo.require("turbo.widgets.TurboGrid.selection");
dojo.require("turbo.widgets.TurboGrid.edit");
dojo.require("turbo.widgets.TurboGrid.rowbar");
dojo.provide("turbo.widgets.TurboGrid.Grid");

dojo.widget.defineWidget('dojo.widget.TurboGrid', dojo.widget.html.TurboWidget, {	
	// plumbing
	//: private
	templateString: '<div class="turbogrid" hidefocus="hidefocus"><div class="turbogrid-master-header" dojoAttachPoint="headerNode"></div><div class="turbogrid-master-view" dojoAttachPoint="viewsNode"></div><span dojoAttachPoint="lastFocusNode" tabindex="0"></span></div>',
	classTag: 'turbogrid',
	styleRoot: 'TurboGrid',
	// settings
	//: public
	//$ Default data getter.
	get: turbo.nop,
	//$ Number of rows to display. 
	rowCount: 5,
	//$ Number of rows to keep in the rendering cache.
	keepRows: 75, 
	//$ Number of rows to render at a time.
	rowsPerPage: 25,
	//$ If <em>autoWidth</em> is true, grid width is automatically set to fit the data.
	autoWidth: false,
	//$ If <em>autoHeight</em> is true, grid height is automatically set to fit the data.
	autoHeight: false,
	//$ If <em>autoRender</em> is true, grid will <a href="#dojo.widget.TurboGrid.prototype.update">render</a> itself after initialization.
	autoRender: true,
	//$ <em>defaultHeight</em> of the grid.
	defaultHeight: '15em',
	//$ View layout defintion. Can be set to a layout object, or to the (string) name of a layout object.
	structure: '',
	//$ Override defaults and make this view elastic.
	elasticView: -1,
	//: private
	focusView: null,
	sortInfo: 0,
	themeable: true,
	columnsWithOrder: null,	
	// initialization
	fillInTemplate: function() {
		if (!this.domNode.getAttribute('tabIndex'))
			this.domNode.tabIndex = "0";
		//this.domNode.style.visibility = 'hidden';
		this.domNode.onReveal = turbo.bind(this, "revealed");
		//
		this.createScroller();			
		this.createBuilder();			
		this.createHeader();			
		this.createViews();
		this.createRowManager();
		this.createFocusManager();
		this.createSelectionManager();
		this.createEditManager();
		//
		turbo.watchTextSizePoll(this, 'textSizeChanged');
		turbo.funnelEvents(this.domNode, this, 'doKeyEvent', turbo.keyEvents);
		turbo.connect(this.domNode, 'onFocus', this, this.doFocus);	
	},
	postCreate: function() {
		this.buildViews();
		(this.autoRender)&&(this.render());
		//(this.autoRender)&&(turbo.defer(this, "render", 250));
		/*if (this.autoRender) {
			var node = this.domNode;
			turbo.deferUntil(function() { return node.offsetHeight > 0 }, turbo.bind(this, 'render'));
		}*/
		this.styleChanged = this.defaultStyleChanged;
	},
	defaultStyleChanged: function() {
		this.setStyledClass(this.domNode, '');
		this.update();
	},
	destroy: function() {
		this.domNode.onReveal = null;
		turbo.unwatchTextSizePoll(this, 'textSizeChanged');
		this.views.destroyViews();
		this.inherited("destroy");
	},
	revealed: function() {
		this.render();
	},
	textSizeChanged: function() {
		var self = this;
		turbo.defer(function() {
			if (!self.domNode)
				return;
			var t = new Date().getTime();
			self.views.forEach(function(v) { 
				v.layout.update(); 
			});
			self.render();
		}, 1);
	},
	// rows
	createRowManager: function() {
		this.rows = new turbo.grid.rows(this);
	},
	// focus manager
	createFocusManager: function() {
		this.focus = new turbo.grid.focus(this);
	},
	// selection manager
	createSelectionManager: function() {
		this.selection = new turbo.grid.selection(this);
	},
	// edit manager
	createEditManager: function() {
		this.edit = new turbo.grid.edit(this);
	},
	// builder
	createBuilder: function() {
		this.builder = new turbo.grid.builder(this);
	},
	// header
	createHeader: function() {
		this.header = new turbo.grid.header(this);
	},
	// virtual scroller
	createScroller: function() {
		this.scroller = new turbo.scroller.columns();
		this.scroller.renderRow = turbo.bind(this, "renderRow");
		this.scroller.removeRow = turbo.bind(this, "rowRemoved");
	},
	// views
	createViews: function() {
		this.views = new turbo.grid.views(this);
		this.views.createView = turbo.bind(this, "createView");
	},
	createView: function(inClass) {
		var view = dojo.widget.createWidget(inClass, { grid: this });
		this.viewsNode.appendChild(view.domNode);
		this.headerNode.appendChild(view.headerNode);
		this.views.addView(view);
		return view;
	},
	buildViews: function() {
		if (this.cells) {
			this.structure = this.cells;
			this.cells = null;
		}
		if ((this.structure)&&(dojo.lang.isString(this.structure)))
			this.structure=window[this.structure];
		if (!this.structure)
			this.structure=window["layout"]
		if (!this.structure)
			return;
		this.indexColumns();
		this.views.destroyViews();
		for (var i=0, v; (v=this.structure[i]); i++)
			this.createView(v.type || "TurboGridView").setViewDef(v);
		this.scroller.setContentNodes(this.views.getContentNodes());
	},
	indexColumns: function() {
		var idx = 0;
		for (var i=0, view; (view=this.structure[i]); i++)
			for (var j=0, row; (view.cells)&&(row=view.cells[j]); j++)
				for (var k=0, cell; (cell=row[k]); k++)
					cell.index = idx++;
		this.colCount = idx;
	},
	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////	
	//Estas 2 funciones las agregue para que en el evento onHeaderCellClick solo aparezca el icono de orden en las columnas que tienen byorder.
	setColumnsWithOrder: function(columnsWithOrder) {
		this.columnsWithOrder = columnsWithOrder;	
	},
	mustOrder: function(cell) {
		if (cell === undefined || cell === null) {
			return false;
		}
		var columnIndex = cell.index;
		if (columnIndex === undefined) {
			return false;
		}
		if (this.columnsWithOrder === null || this.columnsWithOrder === undefined) {	
			return false;
		}
		var gridName = cell.grid.widgetId;
		if (this.columnsWithOrder.gridName !== gridName) {
			return false;
		}		
		if (this.columnsWithOrder[columnIndex] === undefined) {
			return false;
		}
		return this.columnsWithOrder[columnIndex];
	},	
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	//: public
	//$ Install a new structure and rebuild the grid.
	setStructure: function(inStructure) {
		this.structure = inStructure;
		this.buildViews();
		this.render();
	},
	// sizing
	//: protected
	resize: function() {
		// useful measurement
		var padBorder = dojo.html.getPadBorder(this.domNode);
		// grid height
		if (this.autoHeight) {
			this.domNode.style.height = 'auto';
			this.viewsNode.style.height = '';
		} else if (this.domNode.clientHeight <= padBorder.height) {
			if (this.domNode.parentNode == document.body) 
				this.domNode.style.height = this.defaultHeight;
			else 
				this.fitTo = "parent";
		}
		if (this.fitTo == 'parent') {
			var h = dojo.html.getContentBox(this.domNode.parentNode).height;
			dojo.html.setMarginBox(this.domNode, { height: Math.max(0, h) });
		}
		// header height
		var t = this.views.measureHeader();
		this.headerNode.style.height = t + 'px';
		// content extent		
		var l = 1, h = (this.autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
		if (this.autoWidth) 
			// grid width set to total width
			this.domNode.style.width = this.views.arrange(l, 0, 0, h) + 'px';
		else {
			// views fit to our clientWidth
			var w = this.domNode.clientWidth || (this.domNode.offsetWidth - padBorder.width);
			this.views.arrange(l, 0, w, h);
		}
		// virtual scroller height
		this.scroller.windowHeight = h; 
		// default row height (FIXME: use running average(?), remove magic #)
		this.scroller.defaultRowHeight = this.rows.getDefaultHeightPx() + 1;
	},
	// render 
	//: public
	//$ Render the grid, headers, and views. Edit and scrolling states are reset.
	//$ Compare to <a href="#dojo.widget.TurboGrid.prototype.update">Update</a>.
	render: function() {
		//turbo.watchTextSizePoll(this, 'textSizeChanged');
		if (!this.domNode) {
			console.log("Grid.render: domNode is null", this);
			return;
		}
		this.update = this.defaultUpdate;
		//this.contentPixelToEmRatio = turbo.getPixelToEmRatio(this.viewsNode) || 12;
		this.scroller.init(this.rowCount, this.keepRows, this.rowsPerPage);
		this.prerender();
		this.setScrollTop(0);
		this.postrender();
		//this.domNode.style.visibility = '';
	},
	//: protected
	//$ Renders views and calls <a href="#dojo.widget.TurboGrid.prototype.resize">resize</a>.
	prerender: function() {
		this.views.render();
		this.resize();
	},
	//$ Post-render tasks
	postrender: function() {
		// views are position absolute, so they do not inflate the parent
		if (this.autoHeight) {
			//turbo.defer(this, function() { this.viewsNode.style.height = this.views.measureContent() + 'px'; }, 100);
			//this.domNode.style.height = 'auto';
			this.viewsNode.style.height = this.views.measureContent() + 'px';
		}
		this.setFocusView();
	},
	//: private
	renderRow: function(inRowIndex, inNodes) {
		this.views.renderRow(inRowIndex, inNodes);
		//this.edit.restoreState(inRowIndex);
	},
	rowRemoved: function(inRowIndex) {
		//this.edit.saveState(inRowIndex);
		this.views.rowRemoved(inRowIndex);
	},
	// update
	//: private
	defaultUpdate: function() {
		//this.edit.saveState(inRowIndex);
		this.prerender();
		this.scroller.invalidateNodes();
		this.setScrollTop(this.scrollTop);		
		this.postrender();
		//this.edit.restoreState(inRowIndex);
	},
	//: public
	//$ Re-render the grid, retaining edit and scrolling states.
	update: function() {
		this.render();
	},
	//$ Re-render a single row.
	updateRow: function(inRowIndex) {
		this.edit.saveState(inRowIndex);
		this.views.updateRow(inRowIndex, this.rows.getHeight(inRowIndex));
		this.scroller.rowHeightChanged(inRowIndex);
		//this.rowHeightChanged(inRowIndex);
		this.edit.restoreState(inRowIndex);
	},
	rowHeightChanged: function(inRowIndex) {
		this.views.renormalizeRow(inRowIndex);
		this.scroller.rowHeightChanged(inRowIndex);
	},
	//$ Change the number of rows.
	updateRowCount: function(inRowCount) {
		//this.edit.saveState(inRowIndex);
		this.rowCount = inRowCount;
		this.scroller.updateRowCount(inRowCount);
		this.setScrollTop(this.scrollTop);		
		//this.edit.restoreState(inRowIndex);
	},
	//$ Update the styles for a row after it's state has changed.
	updateRowStyles: function(inRowIndex) {
		this.views.updateRowStyles(inRowIndex);
	},
	//$ Perform row styling 
	onStyleRow: function(inRow) {
		with (inRow)
			customClasses += (odd?" turbogrid-row-odd":"") + (selected?" turbogrid-row-selected":"") + (over?" turbogrid-row-over":"");
	},
	//: protected
	// scrolling
	fastScroll: true,
	delayScroll: false,
	scrollRedrawThreshold: (dojo.render.html.ie ? 100 : 50),
	// scroll methods
	scrollTo: function(inTop) {
		if (!this.fastScroll) {
			this.setScrollTop(inTop);
			return;
		}
		var delta = Math.abs(this.lastScrollTop - inTop);
		this.lastScrollTop = inTop;
		if (delta > this.scrollRedrawThreshold || this.delayScroll) {
			this.delayScroll = true;
			this.scrollTop = inTop;
			this.views.setScrollTop(inTop);
			turbo.job('turbogrid-scroll', 200, turbo.bind(this, "finishScrollJob"));
		} else {
			this.setScrollTop(inTop);
		}
	},
	finishScrollJob: function() {
		this.delayScroll = false;
		this.setScrollTop(this.scrollTop);		
	},
	setScrollTop: function(inTop) {
		this.scrollTop = this.views.setScrollTop(inTop);
		this.scroller.scroll(this.scrollTop);
	},
	scrollToRow: function(inRow) {
		this.setScrollTop(this.scroller.findScrollTop(inRow) + 1);
	},
	// styling
	styleRowNode: function(inRowIndex, inRowNode) {
		if (inRowNode)
			this.rows.styleRowNode(inRowIndex, inRowNode);
	},
	// selection
	canSelect: function(inRowIndex) {
		return true;
	},
	canDeselect: function(inRowIndex) {
		return true;
	},
	selected: function(inRowIndex) {
		this.updateRowStyles(inRowIndex);
	},
	deselected: function(inRowIndex) {
		this.updateRowStyles(inRowIndex);
	},
	selectionChanged: function() {
	},
	// sorting
	setSortInfo: function(inSortInfo) {
		this.sortInfo = inSortInfo;
		this.update();
	},
	getSortIndex: function(inIndex) {
		return Math.abs(this.sortInfo) - 1;
	},
	setSortIndex: function(inIndex) {
		if (this.getSortIndex() == inIndex)
			this.setSortInfo(-this.sortInfo)
		else
			this.setSortInfo(inIndex + 1);
	},
	// DOM event handler
	doKeyEvent: function(e) {
		e.dispatch = 'do' + e.type;
		this.onKeyEvent(e);
	},
	// top level (overridable) handlers
	//: public
	onKeyEvent: function(e) {
		this.dispatchKeyEvent(e);
	},
	onContentEvent: function(e) {
		this.dispatchContentEvent(e);
	},
	onHeaderEvent: function(e) {
		this.dispatchHeaderEvent(e);
	},
	// event dispatch
	//: protected
	_dispatch: function(m, e) {
		if (m in this) return this[m](e);
	},
	dispatchContentEvent: function(e) {
		e.sourceView.dispatchContentEvent(e) || this._dispatch(e.dispatch, e);
	},
	dispatchKeyEvent: function(e) {
		this._dispatch(e.dispatch, e);
	},
	dispatchHeaderEvent: function(e) {
		e.sourceView.dispatchHeaderEvent(e) || this.header.dispatchEvent(e) || this._dispatch('doheader' + e.type, e);
	},
	// dispatch handlers
	dokeydown: function(e) {
		this.onKeyDown(e);
	},
	doclick: function(e) {
		if (e.cellNode)
			this.onCellClick(e);
		else
			this.onRowClick(e);
	},
	dodblclick: function(e) {
		if (e.cellNode)
			this.onCellDblClick(e);
		else
			this.onRowDblClick(e);
	},
	docontextmenu: function(e) {
		if (e.cellNode)
			this.onCellContextMenu(e);
		else
			this.onRowContextMenu(e);
	},
	doheaderclick: function(e) {
		if (e.cellNode)
			this.onHeaderCellClick(e);
		else
			this.onHeaderClick(e);
	},
	doheaderdblclick: function(e) {
		if (e.cellNode)
			this.onHeaderCellDblClick(e);
		else
			this.onHeaderDblClick(e);
	},
	doheadercontextmenu: function(e) {
		if (e.cellNode)
			this.onHeaderCellContextMenu(e);
		else
			this.onHeaderContextMenu(e);
	},
	// low-level overridable handlers
	//: public
	onKeyDown: function(e) {
		if (e.altKey || e.ctrlKey || e.metaKey /*|| !this.isEditing()*/)
			return;
		switch (e.keyCode) {
			case e.KEY_ESCAPE:
				//this.cancelEdit();
				break;
			case e.KEY_ENTER:
				//if (!e.shiftKey)
				//	this.applyEdit();
				break;
			case e.KEY_TAB:
				//console.log('onKeyDown tab');
				if (e.shiftKey)
					// if shif-tab not in first cell grid handles focus
					if (!this.focus.isFirstFocusCell()) {
						this.focus.previous();
						turbo.killEvent(e);
					} else {
						this.focus.canBlur.previous = true;
						this.domNode.focus();
					}
				else
					// if tab not in last cell grid handles focus
					if (!this.focus.isLastFocusCell()) {
						this.focus.next();		
						turbo.killEvent(e);
					} else {
						this.focus.canBlur.next = true;
						this.lastFocusNode.focus();
					}
				//console.log('onKeyDown', this.focus.canBlur);
				break;
		}
	},
	onMouseOver: function(e) {
		(e.rowIndex == -1 ? this.onHeaderCellMouseOver(e) : this.onCellMouseOver(e));
	},
	onMouseOut: function(e) {
		(e.rowIndex == -1 ? this.onHeaderCellMouseOut(e) : this.onCellMouseOut(e));
	},
	onMouseOverRow: function(e) {
		if (!this.rows.isOver(e.rowIndex)) {
			this.rows.setOverRow(e.rowIndex);
			(e.rowIndex == -1 ? this.onHeaderMouseOver(e) : this.onRowMouseOver(e));
		}
	},
	onMouseOutRow: function(e) {
		//console.log("onMouseOutRow: " + this.rows.overRow);
		if (this.rows.isOver(-1)) 
			this.onHeaderMouseOut(e)
		else if (!this.rows.isOver(-2)) {
			this.rows.setOverRow(-2);
			this.onRowMouseOut(e);
		} 
	},
	onCellMouseOver: function(e) {
		dojo.html.addClass(e.cellNode, "turbogrid-cell-over");
	},
	onCellMouseOut: function(e) {
		dojo.html.removeClass(e.cellNode, "turbogrid-cell-over");
	},
	onRowMouseOver: function(e) {
	},
	onRowMouseOut: function(e) {
	},
	onCellClick: function(e) {
		turbo.killEvent(e);
		this.focus.setFocusCell(e.rowIndex, e.cell.index);
		/*if (e.target != e.cellNode) {
			console.log("onCellclick: ", e.rowIndex, e.cell.layoutIndex, e.cell.index, e);
		}*/
		this.onRowClick(e);
	},
	onRowClick: function(e) {
		this.selection.clickSelectEvent(e);
	},
	onCellDblClick: function(e) {
		this.onRowDblClick(e);
	},
	onRowDblClick: function(e) {
	},
	onCellContextMenu: function(e) {
		this.onRowContextMenu(e);
	},
	onRowContextMenu: function(e) {
		turbo.killEvent(e);
	},
	onHeaderMouseOver: function(e) {
	},
	onHeaderMouseOut: function(e) {
	},
	onHeaderCellMouseOver: function(e) {
		dojo.html.addClass(e.cellNode, "turbogrid-cell-over");
	},
	onHeaderCellMouseOut: function(e) {
		dojo.html.removeClass(e.cellNode, "turbogrid-cell-over");
	},
	onHeaderClick: function(e) {
	},
	onHeaderCellClick: function(e) {
		//Este if lo agrego para que solo aparzca la flechita de order en las columnas que tienen byorder.
		if (this.mustOrder(e.cell)) {
			this.setSortIndex(e.cell.index);		
		}
		this.onHeaderClick(e);
	},
	onHeaderDblClick: function(e) {
	},
	onHeaderCellDblClick: function(e) {
		this.onHeaderDblClick(e);
	},
	onHeaderCellContextMenu: function(e) {
		this.onHeaderContextMenu(e);
	},
	onHeaderContextMenu: function(e) {
		turbo.killEvent(e);
	},
	onCellFocus: function(inRowIndex, inColIndex) {
		var info = this.focus.info;
		var n = info.view.getCellNode(inRowIndex, info.index);
		// KISS try to focus firstChild of cell if it has .focus
		if (n.firstChild && n.firstChild.focus){
			n.firstChild.focus();
		}		
		else {
			if (this.focusView !== undefined && this.focusView !== null && this.focusView.scrollboxNode !== null) {
				this.focusView.scrollboxNode.focus();				
			}
		}			
	},
	//: protected
	setFocusView: function() {
		this.focusView = this.views.getFirstScrollingView();
	},
	doFocus: function(e) {
		// trap focus only for grid dom node
		if (e && e.target != e.currentTarget)
			return;
		//console.log('focus domNode', this.focus.canBlur);
		// do not focus for scrolling if grid is about to blur	
		if (this.focus.canBlur.next || this.focus.canBlur.previous) {
			this.focus.resetBlur();
			//console.log('focus domNode', this.focus.canBlur);
			return;
		}
		//********************************************************************		
		// TOQUE POR INCIDENTE N� 2233
		if (this.focusView === undefined || this.focusView.scrollboxNode === null) {
			return;
		}
		//********************************************************************			
		this.focusView.scrollboxNode.focus();			
		if (this.focus.isNoFocusCell())
			this.focus.setFocusCell(0, 0);	
	}
});
dojo.provide("turbo.widgets.TurboGrid.DataGrid");
dojo.require("turbo.widgets.TurboGrid.Grid");
dojo.require("turbo.data.model");

dojo.widget.defineWidget('dojo.widget.TurboDGrid', dojo.widget.TurboGrid, {});

dojo.widget.defineWidget('dojo.widget.TurboDataGrid', dojo.widget.TurboGrid, {
	model: 'turbo.data.table',
	postCreate: function() {
		if (this.model) {
			if (typeof this.model == 'string') {
				var m = dojo.getObject(this.model);
				this.model = (typeof m == 'function' ? new m() : m);
			}
			this._setModel(this.model);
		}
		this.inherited("postCreate", arguments);
	},
	_setModel: function(inModel) {
		this.model = inModel;
		if (this.model) {
			this.model.observe("change", this, "modelChange");
			this.model.observe("insertion", this, "modelInsertion");
			this.model.observe("removal", this, "modelRemoval");
		}
		this.measureModel();
	},
	setModel: function(inModel) {
		if (this.model) {
			this.model.unobserve("change", this, "modelChange");
			this.model.unobserve("insertion", this, "modelInsertion");
			this.model.unobserve("removal", this, "modelRemoval");
		}
		this._setModel(inModel);
		this.modelChanged();
	},
	measureModel: function() {
		if (this.model) this.model.measure();
		this.rowCount = (this.model ? this.model.count : 0);
	},
	// data socket
	// called in cells context
	getDataRowIndex: function(inRow, inCell) {
		return inRow.index;
	},
	defaultGet: function(inRow, inCell) {
		var rowIndex = this.getDataRowIndex(inRow, inCell);
		if (inCell.def.keys)
			for (var i=0, d=[], k; (k=inCell.def.keys[i]) || (k !== undefined); i++)
				d.push(inCell.grid.model.get(rowIndex, k));
		else	
			var d = inCell.grid.model.get(rowIndex, (inCell.def.key != undefined ? inCell.def.key : inCell.index));
		return (inCell.def.formatter ? inCell.def.formatter.call(inCell.grid, d, inRow, inCell) : (dojo.lang.isArray(d) ? d.join() : d));
	},
	get: function(inRow, inCell) {
		return inCell.grid.defaultGet.apply(inCell.grid, arguments);
	},
	render: function() {
		this.measureModel();
		dojo.widget.TurboGrid.prototype.render.apply(this, arguments);
	},
	// model modifications
	modelChanged: function() {
		this.measureModel();
		this.updateRowCount(this.rowCount);
	},
	modelRowChanged: function(inRowIndex) {
		this.updateRow(inRowIndex);
	},
	modelColChanged: function(inRowIndex, inFieldIndex) {
		this.updateRow(inRowIndex);
	},
	modelUpdates: ['modelChanged', 'modelRowChanged', 'modelColChanged'],
	defaultModelChange: function(inRowIndex, inFieldIndex) {
		this[this.modelUpdates[arguments.length]].apply(this, arguments);
	},
	modelChange: function(inRowIndex, inColIndex) {
		this.defaultModelChange.apply(this, arguments);
	},
	// model insertion
	modelInsertion: function(inRowIndex) {
		this.updateRowCount(this.model.getRowCount());
	},
	// model removal
	modelRemoval: function(inKeys) {
		this.updateRowCount(this.model.getRowCount());
	},
	getCellDef: function(inIndex) {
		var i = this.views.colToCellIndex(inIndex);
		return i.view.layout.cells[i.index].def;
	},
	canSort: function() {
		var d = this.getCellDef(this.getSortIndex());
		return (d && !d.get && this.model.canSort());
	},
	sortData: function() {
		var k = this.getCellDef(this.getSortIndex()).key || this.getSortIndex();
		var s = (k +1) * (this.sortInfo > 0 ? 1 : -1);
		this.model.sort(s);
	},
	setSortInfo: function(inSortInfo) {
		this.sortInfo = inSortInfo;
		if (this.canSort()) {
			this.sortData();
			this.update();
		}
	},
	junk: 0
});
