dojo.provide("turbo.plumbing");

dojo.require("dojo.dom");
dojo.require("dojo.html.*");
dojo.require("dojo.html.layout");
dojo.require("dojo.html.selection");
dojo.require("dojo.io.*");
dojo.require("dojo.string.*");
dojo.require("dojo.lang.declare");

dojo.style = dojo.html;

turbo.env = dojo.render.html;

turbo.disableSelection = dojo.html.disableSelection;

turbo.getAbsoluteX = function() {
	return dojo.html.abs.apply(dojo.html, arguments).x;
}

turbo.getAbsoluteY = function() {
	return dojo.html.abs.apply(dojo.html, arguments).y;
}

turbo.setMarginBoxSize = function(inElement, inW, inH) {
	dojo.html.setMarginBox(inElement, { width: (inW >= 0 ? inW : undefined), height: (inH >= 0 ? inH : undefined) });
}


dojo.provide("turbo.lib");

turbo.global = this;

// turn off background non-caching nastiness
if (dojo.render.html.ie)
	try { 
		document.execCommand('BackgroundImageCache', false, true); 
	} catch(e) {};

// array stuff

turbo.isGoodIndex = function(inArray, inIndex) {
	return (inArray && inIndex >=0 && inIndex < inArray.length);
}
	
turbo.moveIndex = function(inIndex, inSrc, inDst) {
	if (inIndex == inSrc) 
		return inDst - (inIndex < inDst);
	return inIndex - (inIndex > inSrc) + (inIndex >= inDst);
}

turbo.filter = function(inArray, inFunc, inObj) {
	inObj = (inObj || dj_global);
	var result = [ ];
	for (var i=0, elt; i < inArray.length; i++) {
		elt = inFunc.call(inObj, inArray[i]);
		if (elt !== undefined)
			result.push(elt);
	}
	return result;
}

turbo.arrayInsert = function(inArray, inIndex, inValue) {
	if (inArray.length <= inIndex)
		inArray[inIndex] = inValue;
	else
		inArray.splice(inIndex, 0, inValue);
}

turbo.arrayRemove = function(inArray, inIndex) {
	inArray.splice(inIndex, 1);
}

turbo.arraySwap = function(inArray, inI, inJ) {
	var cache = inArray[inI];
	inArray[inI] = inArray[inJ];
	inArray[inJ] = cache;
}

turbo.arrayMove = function(inArray, inFromIndex, inToIndex) {
	var e = inArray[inFromIndex];
	inArray.splice(inFromIndex, 1);
	turbo.arrayInsert(inArray, (inToIndex < inFromIndex ? inToIndex : inToIndex - 1), e);
	//inArray.splice((inToIndex < inFromIndex ? inToIndex : inToIndex - 1), 0, e);
}

turbo.arrayDup = function(inArray) {
	return inArray.slice(0);
}

turbo.arrayCompare = function(inA, inB) {
	for (var i=0,l=inA.length; i<l; i++)	
		if (inA[i] != inB[i])
			return false;
	return (inA.length == inB.length);
}

// string stuff

turbo.cat = function() {
	if (turbo.env.ie)
		return turbo.cloneArguments(arguments).join('')
	else {
		var s = '';
		for (var i=0,l=arguments.length;i<l;i++)
			s += arguments[i];
		return s;
	}
}

turbo.stringOf = function(inCount, inLiteral) {
	if (inCount <= 0)
		return '';
	var result = new Array(inCount);
	for (var i=0; i < inCount; i++)
		result[i] = inLiteral;
	return result.join('');
}

turbo.alphabetizeIndex = function(inIndex) {
	var letter = function(c) { return String.fromCharCode('A'.charCodeAt(0) + c); };
	var a = Math.floor(inIndex / 26);
	return (a > 0 ? letter(a-1) : '') + letter(inIndex % 26);
}

turbo.pathpop = function(inPath, inDelim) {
	var delim = inPath.lastIndexOf((inDelim == undefined ? '/' : inDelim));
	return (delim >= 0 ? inPath.substring(0, delim) : '');
}

turbo.escapeText = function(inText) {
	return dojo.string.escapeXml(String(inText)).replace(/\n/g, "<br />");
}

turbo.conjoin = function(inArray, inLeft, inRight) {
	return inLeft + inArray.join(inRight + inLeft) + inRight;
}

turbo.supplant = function(s, o) {
	var i, j;
	for (;;) 
	{
		i = s.lastIndexOf('{');
		if (i < 0) 
			break;
		j = s.indexOf('}', i);
		if (i + 1 >= j) 
			break;
		s = s.substring(0, i) + o[s.substring(i + 1, j)] + s.substring(j + 1);
	}
	return s;
}

turbo.printf = function(s) {
	for (var a=1, i=0, r; a < arguments.length;) {
		i = s.indexOf('%', i);
		if (i == -1) 
			break;
		if (s.charAt(i + 1) == '%')
			r = '%';
		else
			r = arguments[a++]; 
		s = s.substring(0, i) + r + s.substring(i + 2);
		if (r == '%')
			i++;
	}
	return (s == undefined ? '' : s);
}

turbo.stringReplace = function(inStr, inRegEx, inReplaceFunc) {
	if (!turbo.env.safari) return inStr.replace(inRegEx, inReplaceFunc);
	var str = inStr, callback = inReplaceFunc, reg = inRegEx, result = [ ], lastidx = reg.lastIndex, re;
	while ((re = reg.exec(str)) != null) {
		var idx  = re.index, args = re.concat(idx, str);
		result.push(str.slice(lastidx,idx), callback.apply(null,args).toString());
		if (!reg.global) {
			lastidx += (RegExp.lastMatch ? RegExp.lastMatch.length : 0);
			break;
		} else {
			lastidx = reg.lastIndex;
		}
	}
	result.push(str.slice(lastidx));
	return result.join("")
}

turbo.macros = new function() {
	this.apply = function(inData, inMacros)	{
		return turbo.stringReplace(inData, new RegExp('%%([^%]*)%%', 'ig'),
			function(w, m) {
				return (inMacros[m] ? inMacros[m] : m);
		});	
	}
	this.interpolate = function(inMacros)	{
		for (var i in inMacros)
			inMacros[i] = this.apply(inMacros[i], inMacros);
		return inMacros;	
	}
	this.insert = function(inData, inMacros) {
		if (!inMacros)
			return inData;
		inMacros = (dojo.lang.isString(inMacros) ? eval('(' + inMacros + ')') : inMacros);
		inMacros = this.interpolate(inMacros);
		return this.apply(inData, inMacros);
	}
}

// time stuff

turbo.time = function() {
	return new Date().getTime();
}

// math stuff

turbo.clamp = function(inValue, inMin, inMax) {
	return Math.max(inMin, Math.min(inMax, inValue));
}
 
turbo.randi = function(inInteger) {
	return Math.floor(Math.random()*(inInteger + 1));
}

// object stuff

turbo.mix = function() {
	var obj = { };
	for (var i=0,l=arguments.length;i<l;i++)
		dojo.lang.mixin(obj, arguments[i]);
	return obj;
}

// mix inProps into inObj, but only those props *already* in inObj
turbo.mixover = function(inObj, inProps /*, ...*/) {
	for (var a=1, props; (props=arguments[a]); a++)
		for (var i in props)
			if (!(i in Object.prototype) && (i in inObj) && (props[i])) 
				inObj[i] = props[i];
	return inObj;
}

turbo._swiss = function(inSrc, inDst) {
	for (var i in inSrc)
		inDst[i] = inSrc[i];
	return inDst;
}

turbo.swiss = function(inSrc, inDst) {
	if (!inSrc || !inDst)
		return;
	if (!dojo.lang.isArray(inSrc))
		turbo._swiss(inSrc, inDst);
	else
		for (var i=0, l=inSrc.length; i<l; i++)
			turbo._swiss(inSrc[i], inDst);
	return inDst;			
}

turbo.stringToReference = function(inDescriptor) {
	var obj = turbo.global;
	var names = inDescriptor.split('.');
	var prop = names.pop();
	while (names.length && obj)
		obj = obj[names.shift()];
	return (prop && (prop in obj) ? obj[prop] : null);
}

// function stuff

turbo.nop = function() {};
turbo.emptyFunction = turbo.nop;

turbo.cloneArguments = function(inArguments, inSkip) {
	var i = (inSkip || 0);
	var l = inArguments.length;
	var result = new Array(l - i);		
	for (j=0; i<l; i++, j++)
		result[j] = inArguments[i];	
	return result;
}

turbo.bind = function(inObject, inMethod) {
	if (inMethod) {
		return function() {	
			if (dojo.lang.isString(inMethod))
				inMethod = inObject[inMethod];
			return inMethod.apply(inObject, arguments); 
		}
	} 
	else {
		turbo.debug('turbo.bind called with null method');
		return turbo.nop;
	}
}

turbo.tie = function(inThis, inMethodName /*, ...*/) {
	for (var i=1, a=arguments[i]; a; a=arguments[++i])
		inThis[a] = turbo.bind(inThis, inThis[a]);
	return inThis[arguments[1]];
}
	
// tying (i.e. turbo.tie) would appear to be more efficient
turbo.bindArgs = function(inObject, inMethod /*, arguments to preapply */) {
	if (inMethod) {
		var preArgs = turbo.cloneArguments(arguments, 2);
		return function() {	
			if (dojo.lang.isString(inMethod))
				inMethod = inObject[inMethod];
			return inMethod.apply(inObject, preArgs.concat(turbo.cloneArguments(arguments))); 
		}
	} else {
		turbo.debug('turbo.bindArgs called with null method');
		return turbo.nop;
	}
}

turbo.defer = function(/*inBindContext,*/ inFunc, /*inArguments,*/ inInterval) {
	if (arguments.length > 3)	{
		var args = turbo.cloneArguments(arguments);
		var interval = args.pop();		
		return window.setTimeout(turbo.bindArgs.apply(turbo, args), interval);
	}
	if (arguments.length > 2)
		return window.setTimeout(turbo.bind(arguments[0], arguments[1]), arguments[2]);
	else																																					
		return window.setTimeout(arguments[0], arguments[1]);
}

turbo.deferUntil = function(evt, action, poll_interval, timeout) {
	timeout = (timeout||5000);
	poll_interval = (poll_interval||100);
	var func = function() {
		if (evt()) {
			window.clearInterval(job);
			action();
		} else {
			timeout -= poll_interval;
			if (timeout <= 0)
				window.clearInterval(job);
		}
	}
	var job = window.setInterval(func, poll_interval);
}

turbo.cancel = function(inHandle) {
	if (inHandle)
		window.clearTimeout(inHandle);
}

turbo.jobs = [ ];
turbo.job = function(inName, inDelay, inJob) {
	turbo.cancelJob(inName);
	var job = function() {
		delete turbo.jobs[inName];
		inJob();
	}
	turbo.jobs[inName] = turbo.defer(job, inDelay);
}

turbo.cancelJob = function(inName) {
	turbo.cancel(turbo.jobs[inName]);
}

turbo.getFunction = turbo.stringToReference;

// debug stuff

turbo.debugOut = function(inMsg) {
	dojo.debug(inMsg);
}

turbo.debugArray = function(inArray) {
	for (var i=0, l=inArray.length; i<l; i++)
		if (inArray[i])
			turbo.debug(i + ':', inArray[i]);
}

turbo.debugNode = function(inNode, inDent) {
	turbo.debugOut(inDent + '(node) ' + inNode.tagName);
}

turbo.debugObjs = [ ];
turbo.debugObject = function(inObj, inDent) {
	if (!inDent) 
		inDent = '';
	if (inObj.nodeName) 
		turbo.debugNode(inObj, inDent);
	else if (inDent.length > 6 * 10)
		turbo.debugOut(inDent + 'too deep');
	else if (dojo.lang.inArray(turbo.debugObjs, inObj))
		turbo.debugOut(inDent + '(circular reference)');
	else {
		var l = turbo.debugObjs.length;
		turbo.debugObjs.push(inObj);	
		try {
			for (var name in inObj) {
				//if (name == '______array')
				//	continue;
				var obj = inObj[name];
				s = inDent + '| ' + name;
				if (obj != null && typeof(obj) == 'object') {
					turbo.debugOut(s + ' = (' + (obj instanceof Array ? 'array' : 'object') + ')');
					turbo.debugObject(obj, inDent + '......');
				} 
				else
					turbo.debugOut(s + ' = ' + obj);
			}
		} finally {
			delete turbo.debugObjs[l];
		}
	}
}

turbo.debugTop = function(inObj, inDent) {
	if (!inDent) 
		inDent = '';
	for (var name in inObj) {
		var obj = inObj[name];
		s = inDent + name;
		if (obj != null && typeof(obj) == 'object')
			turbo.debugOut(s + ' = (' + (obj instanceof Array ? 'array' : 'object') + ')');
		else
			turbo.debugOut(s + ' = ' + obj);
	}
}

turbo.debugf = function() {
	turbo.debug(turbo.printf.apply(turbo, arguments));
}

turbo.debug = function() {
	var c = arguments.length;
	for (var i=0; i < c; i++) {
		if (dojo.lang.isArray(arguments[i]))
			turbo.debugArray(arguments[i]);
		else if (dojo.lang.isObject(arguments[i]))
			turbo.debugTop(arguments[i]);
		else
	  	turbo.debugOut(arguments[i]);
	}
}

// DOM stuff

turbo.$ = function(inNodeOrId, inDoc) {
	return (!inNodeOrId ? null : (!dojo.lang.isString(inNodeOrId) ? inNodeOrId : (inDoc ? inDoc : document).getElementById(inNodeOrId)));
}

turbo.create = function(inTagName) {
	return document.createElement(inTagName);
}

turbo.remove = function(inNode) {
	inNode = turbo.$(inNode);
	if (inNode && inNode.parentNode)
		inNode.parentNode.removeChild(inNode);
	return inNode;
}

turbo.append = function(inNode, inParentNode) {
	(inParentNode ? inParentNode : document.body).appendChild(inNode);
}

turbo.addBodyNode = function(inElt) {
	document.body.appendChild(inElt);
}

turbo.addHeadNode = function(inElt) {
	document.getElementsByTagName('head').item(0).appendChild(inElt);
}

turbo.marshall = function(/* [scope,] id0, id1, ... */) {
	var id = '';
	var scope = dj_global;
	for (var i=0; i<arguments.length; i++){
		id = arguments[i];
		if ((i==0)&&(!dojo.lang.isString(id)))
			scope = id;
		else if (!scope[id])
			scope[id] = turbo.$(id);
	}
	return dj_global[id];
}

turbo.getTagName = function(inNodeOrId) {
	var node = turbo.$(inNodeOrId);
	return (node && node.tagName ? node.tagName.toLowerCase() : '');
}

turbo.indexInParent = function(inNode) {
	var i=0, n, p=inNode.parentNode;
	while (n = p.childNodes[i++]) 
		if (n == inNode)
			return i - 1;			
	return -1;
}

turbo.kids = function(inNode, inTag) {
	var result = [ ];
	var i=0, n;
	while (n = inNode.childNodes[i++]) 
		if (turbo.getTagName(n) == inTag)
			result.push(n);
	return result;
}

turbo.divkids = function(inNode) {
	return turbo.kids(inNode, 'div');
}

turbo.nthkid = function(inNode, inN, inTag) {
	var result = [ ];
	var i = 0, n;
	while (n = inNode.childNodes[i++]) 
		if (turbo.getTagName(n) == inTag)
			if (inN-- == 0)
				return n;
	return null;
}

turbo.nthdiv = function(inNode, inN) {
	return turbo.nthkid(inNode, inN, 'div');
}

turbo.capture = function(inElement) {
	//if (inElement.setCapture)
	//	turbo.debug('setCapture on ' + inElement.tagName);
	if (inElement.setCapture)
		inElement.setCapture();
	else {
		document.addEventListener("mousemove", inElement.onmousemove, true);
		document.addEventListener("mouseup", inElement.onmouseup, true);
	}
}

turbo.release = function(inElement) {
	if (inElement.releaseCapture)
		inElement.releaseCapture();
	else {
		document.removeEventListener("mousemove", inElement.onmousemove, true);
		document.removeEventListener("mouseup", inElement.onmouseup, true);
	}
}

turbo.getPixelToEmRatio = function(inNode) {
	var scale = 200;
	var node = inNode || document.body;
	var result = 12;
	try	{
		var e = document.createElement("div");
		with (e.style) {
			top = "0px";
			left = "0px";
			width = scale + "em";
			height = "1em";
			overflow = "hidden";
			position = "absolute";
			visibility = "hidden";
		}
		node.appendChild(e);
		//console.log(dojo.html.getComputedStyle(e, 'width'), e.offsetWidth);
		//result = dojo.html.getComputedStyle(e, 'width', result) / scale;
		result = e.offsetWidth / scale;
		node.removeChild(e);
		delete e;
	}	catch (ex) { }
	return result;
}
turbo.EmToPixels = turbo.getPixelToEmRatio;

turbo.getScrollbarWidth = function() {
	if (turbo._scrollBarWidth)
		return turbo._scrollBarWidth;
	turbo._scrollBarWidth = 18;
	try	{
		var e = document.createElement("div");
		with (e.style) {
			top = "0px";
			left = "0px";
			width = "100px";
			height = "100px";
			overflow = "scroll";
			position = "absolute";
			visibility = "hidden";
		}
		document.body.appendChild(e);
		turbo._scrollBarWidth = e.offsetWidth - e.clientWidth; // - (turbo.env.moz ? 1 : 0);
		document.body.removeChild(e);
		delete e;
	}	catch (ex) { }
	return turbo._scrollBarWidth;
}

turbo.hasHorizontalScrollbar = function(inNode) {
	return (inNode.scrollWidth > inNode.clientWidth);
}

turbo.watchTextSizePoll = function(inObject, inMethod, inInterval) {
	var f = document.createElement("div");
	with (f.style) {
		top = "0px";
		left = "0px";
		position = "absolute";
		visibility = "hidden";
	}
	f.innerHTML = "TheQuickBrownFoxJumpedOverTheLazyDog";
	document.body.appendChild(f);
	var fw = f.offsetWidth;
	var job = function() {
		if (f.offsetWidth != fw) {
			fw = f.offsetWidth;
			turbo.textSizeChanged();
		}
	}
	window.setInterval(job, inInterval||200);
	turbo.connectTextSizePoll(inObject, inMethod);
	turbo.watchTextSizePoll = turbo.connectTextSizePoll;
}

turbo.unwatchTextSizePoll = function(inObject, inMethod) {
	dojo.event.disconnect(turbo, "textSizeChanged", inObject, inMethod);
}

turbo.connectTextSizePoll = function(inObject, inMethod) {
	dojo.event.connect(turbo, "textSizeChanged", inObject, inMethod);
}

turbo.textSizeChanged = function() {
}
		
turbo.preloads = [];
turbo.preloadImage = function(inPath) {
	var i = new Image();
	i.src = inPath;
	turbo.preloads.push(i);
	//turbo.debug('preload: ' + inPath);
}

turbo.setCursor = function(inCursor) {
	document.body.style.cursor = inCursor;
}

turbo.setBusyCursor = function() {
	turbo.setCursor('wait');
}

turbo.setDefaultCursor = function() {
	turbo.setCursor('default');
}

turbo.getStyleText = function(inNode, inStyleText) { 
	if (dojo.lang.isUndefined(inNode.style.cssText))  
  	return inNode.getAttribute("style");  
 	return inNode.style.cssText;  
}

turbo.setStyleText = function(inNode, inStyleText) { 
	if (dojo.lang.isUndefined(inNode.style.cssText))  
  	inNode.setAttribute("style", inStyleText);  
	else 
  	inNode.style.cssText = inStyleText;  
}

// selection stuff

turbo.cancelSelectStart = function() {
	if (window.event)
		return (!window.event.ctrlKey && !window.event.shiftKey);
}

turbo.cancelSelection = function() {
	if (window.getSelection)
		window.getSelection().removeAllRanges();
}

// SELECT stuff

turbo.getSelectValue = function(inSelect) {
	var s = turbo.$(inSelect);
	if (!s)
		return '';
	with (s.options[s.selectedIndex])
		return (value ? value : innerHTML);
}

// TABLE stuff

turbo.getCellIndex = function(inTd) {
	if (inTd.cellIndex) 
		return inTd.cellIndex;
	var cells = inTd.parentNode.cells;
  for (var i=0, cell; (cell=cells[i]); i++) 
		if (inTd == cell)
			return i;
	return -1;			
}

turbo.getRowIndex = function(inTr) {
	if (inTr.rowIndex && inTr.rowIndex >= 0) 
		return inTr.rowIndex;
	var rows = inTr.parentNode.childNodes;
  for (var i=0, l=rows.length; i < l; i++) 
		if (inTr == rows[i])
			return i;
	return -1;			
}

turbo.getTableRow = function(inRowOwner, inIndex) {
	if (!inRowOwner)
		return null;
	if ((inRowOwner.rows)&&(inRowOwner.rows.length > 0))
		return inRowOwner.rows[inIndex];
	else if (inRowOwner.childNodes.length > inIndex)
		return inRowOwner.childNodes[inIndex];
	else
		return null;
}

turbo.getTable = function(inNode) {
	var n = inNode;
	while ((n)&&(turbo.getTagName(n) != 'table'))
		n = n.parentNode;
	return n;
	/*	
	var n = inNode;
	while (n) {
		if (n.rows) 
			return n;
		n = n.parentNode;
	}
	return false;
	*/
}

// sizing stuff

/*
    +-------------------------+
    |  margin                 |
    | +---------------------+ |
    | |  border             | |
    | | +-----------------+ | |
    | | |  padding        | | |
    | | | +-------------+ | | |
    | | | |   content   | | | |
    | | | +-------------+ | | |
    | | +-|-------------|-+ | |
    | +---|-------------|---+ |
    +-|---|-------------|---|-+
    |<--->|contentmargin|<--->|
    | |<->| innermargin |<->| |
    | |<----- offset ------>| |
    | |   |<- content ->|   | |
    | |<------ inner ------>| |
    |<-------- outer -------->|
*/

turbo.getWindowSize = function() {
	if (window.innerWidth)
		return { w: window.innerWidth, h: window.innerHeight };		
	else		
		return { w: document.documentElement.clientWidth, h: document.documentElement.clientHeight };		
}

turbo.getContentSize = function(inElement) {
	if ((inElement)&&(inElement != document.body))
		with (dojo.html.getContentBox(inElement))
			return { w: width, h: height };
	else
		return turbo.getWindowSize();
}

turbo.getInnerSize = function(inElement) {
	if (inElement && inElement != document.body) {
		var s = dojo.html.getBorderBox(inElement);
		return { w: s.width, h: s.height };
	} else
		return turbo.getWindowSize();
}

turbo.getOuterSize = function(inElement) {
	if (inElement && inElement != document.body) {
		var s = dojo.html.getMarginBox(inElement);
		return { w: s.width, h: s.height };
	} else
		return turbo.getWindowSize();
}

turbo.setContentSize = function(inElement, inW, inH) {
	var siz = turbo.getContentSize(inElement);
	if (inW > 0 && inW != siz.w)
		inElement.style.width = inW + 'px';
	if (inH > 0 && inH != siz.h)
		inElement.style.height = inH + 'px';
}

turbo.getBounds = function(inNode) {
	var bounds = turbo.getContentSize(inNode);
	with (dojo.html.getPadding(inNode)) {
		bounds.l = width;
		bounds.t = height;
	}
	return bounds;
}

turbo.setBounds = function(inElement, inL, inT, inW, inH) {
	if (!inElement)
		return;
	with (inElement.style) {
		if (inL >= 0) 
			left = inL + 'px';
		if (inT >= 0)
			top = inT + 'px';
	}
	turbo.setMarginBoxSize(inElement, inW, inH);
}

turbo.fitToParent = function(inNode) {
	with (turbo.getBounds((inNode.parentNode)||(document.body)))
		turbo.setBounds(inNode, l, t, w, h);
}

turbo.setStyle = function(inElement, inStyle, inValue) {
	if (inElement && inElement.style[inStyle] != inValue)
	  inElement.style[inStyle] = inValue;
}

turbo.setStyleLeftPx = function(inElement, inLeft) {
	turbo.setStyle(inElement, 'left', inLeft + 'px'); 
}

turbo.setStyleTopPx = function(inElement, inTop) {
	turbo.setStyle(inElement, 'top', inTop + 'px'); 
}

turbo.setStyleWidthPx = function(inElement, inWidth) {
	if (inWidth >= 0)
		turbo.setStyle(inElement, 'width', inWidth + 'px');
}

turbo.setStyleHeightPx = function(inElement, inHeight) {
	if (inHeight >= 0)
		turbo.setStyle(inElement, 'height', inHeight + 'px');
}

turbo.setStyleSizePx = function(inElement, inWidth, inHeight) {
	turbo.setStyleWidthPx(inElement, inWidth);
	turbo.setStyleHeightPx(inElement, inHeight);
}

turbo.setStyleBoundsPx = function(inElement, inLeft, inTop, inWidth, inHeight) {
	turbo.setVisibility(inElement, false);
	turbo.setStyleLeftPx(inElement, inLeft); 
	turbo.setStyleTopPx(inElement, inTop); 
	turbo.setStyleWidthPx(inElement, inWidth);
	turbo.setStyleHeightPx(inElement, inHeight);
	turbo.setVisibility(inElement, true);
}

turbo.setOuterStyleWidthPx = function(inElement, inWidth) {
	if (inElement)
		dojo.html.setMarginBoxWidth(inElement, inWidth);
}

turbo.setOuterStyleHeightPx = function(inElement, inHeight) {
	if (inElement)
		dojo.html.setMarginBoxHeight(inElement, inHeight);
}

// style stuff

turbo.setShowing = function(inElement, inShow) {
	inElement = turbo.$(inElement);
	if (inElement && inElement.style)
		inElement.style.display = (inShow ? '' : 'none');		
}

turbo.showHide = function(/* element0, e1, ... en, [true|false]*/) {
	var l = arguments.length - 1;
	var show = arguments[l];
	if (show !== true && show !== false) 
	{
		show = true;
		l++;
	}
	for (var i=0; i<l; i++)
		turbo.setShowing(arguments[i], show);
}

turbo.show = turbo.showHide;

turbo.hide = function(/* element0, e1, ... en */) {
	var l = arguments.length;
	for (var i=0; i<l; i++)
		turbo.setShowing(arguments[i], false);
}

turbo.isShowing = function(inElement) {
	inElement = turbo.$(inElement);
	if (!inElement || (inElement["style"] && dojo.html.getComputedStyle(inElement, 'display') == 'none'))
		return false;
	else if (inElement["parentNode"] && inElement.parentNode && inElement.parentNode != document.body)
		return turbo.isShowing(inElement.parentNode);
	else 
		return true;
}
turbo.showing = turbo.isShowing;

turbo.setVisibility = function(inElement, inVisible) {
	inElement = turbo.$(inElement);
	if (inElement && inElement.style)
		inElement.style.visibility = (inVisible ? '' : 'hidden');		
}

turbo.setStyleProperties = function(inNode, inProperties) {
	if (!inNode || !inNode.style)
		return;
	for (var i in inProperties)
		if (i == 'opacity')
			dojo.html.setOpacity(inNode, inProperties[i]);
		else if (i in inNode.style)
			inNode.style[i] = inProperties[i];
}

// Dojo stuff

turbo.clean = function(inNode) {
	if (!inNode)
		return;
	var filter = function(inW) { 
		return inW.domNode && dojo.dom.isDescendantOf(inW.domNode, inNode, true);
	}
	var ws = dojo.widget.byFilter(filter);
	for (var i=0; i<ws.length; i++) {
		var w = ws[i];
		if (dojo.widget.widgetIds[w.widgetId] == w)
			w.destroy();
	}
	delete ws;
	dojo.event.browser.clean(inNode);
}

turbo.connect = function(inNode, inEvent, inContext, inMethod) {
	dojo.event.browser.addListener(inNode, inEvent, turbo.bind(inContext, inMethod));	
}

turbo.mouseEvents = [ 'mouseover', 'mouseout', /*'mousemove',*/ 'mousedown', 'mouseup', 'click', 'dblclick', 'contextmenu' ];
turbo.keyEvents = [ 'keyup', 'keydown', 'keypress' ];

turbo.funnelEvents = function(inNode, inObject, inMethod, inEvents) {
	var evts = (inEvents ? inEvents : turbo.mouseEvents.concat(turbo.keyEvents));
	for (var i=0, l=evts.length; i<l; i++)
		turbo.connect(inNode, 'on' + evts[i], inObject, inMethod);	
//		dojo.event.connect(inNode, 'on' + evts[i], inObject, inMethod);	
}

turbo.killEvent = function(e) { 
	if (e)
		dojo.event.browser.stopEvent(e);
}

// widget stuff

turbo.parseWidgets = function(inNode, inSearchIds) {
	try {
		var n, parser = new dojo.xml.Parse(), sids = (inSearchIds ? inSearchIds : djConfig.searchIds);
		if (sids && sids.length > 0) {
			for (var i=0,l=sids.length; i<l; i++)	{
				n = document.getElementById(sids[i]);
				if (n)
					dojo.widget.getParser().createComponents(parser.parseElement(n, null, true));
			}
		} 
		else
			dojo.widget.getParser().createSubComponents(parser.parseElement(inNode, null, true));
	} catch(e) {
		turbo.debug("turbo.parseWidgets: an exception was thrown", e);
	}
}

turbo.widgetClick = function(id, onclick) {
	var w = dojo.widget.byId('id');
	var f = turbo.evalObjPath(onClick);
	if (w && f) 
		f.call(w);
}

// script stuff

turbo.scripts = [ ];
turbo.loadScript = function(inScript) {
	if (turbo.scripts[inScript])
		return;
	turbo.scripts[inScript] = true;
	dojo.io.bind({
		url: inScript,
		mimetype: "text/javascript",
		sync: true
	});
}
				
turbo.loadScriptByTag = function(inScript) {
	if (turbo.scripts[inScript])
		return;
	turbo.scripts[inScript] = true;
	var script = document.createElement('script');
	script.type = 'text/javascript';
	script.language = 'JavaScript';
	turbo.addHeadNode(script);
	script.src = inScript;
}

if (!this.dojo)
	throw new Exception("Requires Dojo toolkit");

dojo.provide("turbo.turbo");
dojo.require("turbo.plumbing");
dojo.require("turbo.lib");

turbo.fine_css = true;
dojo.provide("turbo.lib.more");

turbo.addRemoveClass = function(inNode, inClass, inAdd) {
	if (inNode)
		dojo.html[(inAdd ? "addClass" : "removeClass")].call(dojo.html, inNode, inClass);
}
dojo.provide("turbo.lib.scrim");

dojo.require("turbo.turbo");

dojo.declare('turbo.pool', null, {
	pool: [ ],
	findUnused: function() {
		for (var i= this.pool.length-1; i >= 0; i--)
			if (!pool[i].__inuse)
				return this.pool[i];
		return false;
	},
	create: function() {
	},
	_create: function() {
		var resource = this.create();
		this.pool.push(resource);
	},
	get: function() {
		var resource = this.findUnused() || this._create();
		if (resource)
			resource.__inuse = true;
		return resource;
	},
	release: function(inResource) {
		if (inResource)
			inResource.__inuse = false;
	}
});

turbo.scrim = { 
	pool: new turbo.pool(),
	createIFrame: function() {
		var node = document.createElement('iframe');
		node.setAttribute('frameBorder', 0);
		node.setAttribute('scrolling', 'no');
		return node;
	},
	createDiv: function() {
		return document.createElement('div');
	},
	createScrim: function() {
		var node = (dojo.render.html.ie ? turbo.scrim.createIFrame() : turbo.scrim.createDiv());
		node.isScrim = true;	
		return node;
	},
	get: function(inParent, inId, inProperties, inBounds) {
		var node = turbo.scrim.pool.get();
		node.scrimActive = true;
		node.style.border = 0;
		node.style.position = 'absolute';
		node.id = inId;
		node.name = inId;
		/*if (inId) {
			node.setAttribute('id', inId);
			node.setAttribute('name', inId);
		}*/
		turbo.setStyleProperties(node, inProperties);
		inParent.appendChild(node);	
		turbo.scrim.size(node, inBounds);
		return node;	
	},
	release: function(inScrim) {
		if (inScrim.isScrim) {
			turbo.scrim.pool.release(inScrim);
			dojo.dom.removeNode(inScrim);
		}
	},
	getBounds: function(inScrim) {
		var box = (inScrim.parentNode == document.body ? turbo.getWindowSize() : dojo.html.getPaddingBox(inScrim.parentNode));
		return {t: 0, l: 0, w: box.w || box.width, h: box.h || box.height};
	},
	size: function(inScrim, inBounds) {
		if (!inBounds)
			inBounds = this.getBounds(inScrim);
		turbo.setBounds(inScrim, inBounds.l, inBounds.t, inBounds.w, inBounds.h);
	}
}

turbo.scrim.pool.get = turbo.scrim.createScrim;

dojo.provide('turbo.lib.align');
dojo.require('turbo.turbo');

turbo.aligner = new function() {
	this.enabled = false;
	this.targets = [ ];
	this.getAlignment = function(inElement) {
		return inElement.getAttribute('turboAlign') || inElement.getAttribute('turboalign');
	}
	this.visible = function(inElement) {
		return (dojo.html.getComputedStyle(inElement, 'display') != 'none'); 
	}
	this.listChildrenByAlignment = function(inRoot, inAlign) {
		var result = [ ];
		var node = inRoot.firstChild;
		while (node) {
			if (node.nodeType == 1 && (this.getAlignment(node)==inAlign) && this.visible(node))
				result.push(node);
			node = node.nextSibling;
		}
		return result;
	}
	this.listAlignedChildren = function(inRoot)	{
		var result = { none:[], top:[], left:[], client:[], right:[], bottom:[] };
		var node = inRoot.firstChild;
		while (node) {
			if (node.nodeType == 1 && this.visible(node)) 	{
				var align = this.getAlignment(node);
				if (align)
					if (result[align])
						result[align].push(node);
					else
						result[align] = [ node ];
			}
			node = node.nextSibling;
		}
		return result;
	}
	this.setStylePosition = function(inElement, inPosition)	{
		if (inElement.style.position != inPosition)
			inElement.style.position = inPosition;
	}
	this.normalizeAlignedElement = function(inElement) {
		this.setStylePosition(inElement, 'absolute');
	}
	this.alignElement = function(inElement, inL, inT, inW, inH) {
		this.normalizeAlignedElement(inElement);
		turbo.setBounds(inElement, inL, inT, inW, inH);
		this.alignChildren(inElement);
	}
	this.alignChildren = function(inRoot)	{
		var all_aligns = this.listAlignedChildren(inRoot);
		var siz = turbo.getContentSize(inRoot);
		var top = dojo.html.getPixelValue(inRoot, 'padding-top', true);
		var left = dojo.html.getPixelValue(inRoot, 'padding-left', true);
		//
		var l, r, t, b, w, h, c, aligns;
		//		
		aligns = all_aligns.none;
		for (var i=0; i<aligns.length; i++) {
			var p = dojo.html.getComputedStyle(aligns[i], 'position');
			//turbo.debugf('node [%s, %s, %s] has computed position [%s]', aligns[i].id, aligns[i].dojoAttachPoint, aligns[i].className, p);
			//if (p != 'relative' && p !='absolute')
				//this.setStylePosition(aligns[i], 'relative');
		}
		//
		aligns = all_aligns.top;
		//t = 0;
		t = top;
		for (var i=0; i<aligns.length; i++)	{
			this.alignElement(aligns[i], left, t, siz.w);
			t += aligns[i].offsetHeight;
		}
		//
		aligns = all_aligns.bottom;
		b = siz.h + top;
		c = aligns.length;
		for (var i=c-1; i >= 0; i--)	{
			b -= aligns[i].offsetHeight;
			this.alignElement(aligns[i], left, b, siz.w);
		}
		//
		h = b - t;
		//
		aligns = all_aligns.left;
		l = left;
		for (var i=0; i<aligns.length; i++)	{
			this.alignElement(aligns[i], l, t, -1, h);
			l += aligns[i].offsetWidth;
		}
		//
		aligns = all_aligns.right;
		r = siz.w + left;
		c = aligns.length;
		for (var i=c-1; i >= 0; i--)
		{
			r -= aligns[i].offsetWidth;
			this.alignElement(aligns[i], r, t, -1, h);
		}
		//
		w = r - l;
		//
		aligns = all_aligns.client;
		for (var i=0; i<aligns.length; i++)
		{
			this.alignElement(aligns[i], l, t, w, h);
			break;
		}
		//
		aligns = all_aligns.none;
		for (var i=0; i<aligns.length; i++)
			this.alignChildren(aligns[i]);
	}
	this.alignTargets = function() {
		for (var i=0,l=turbo.aligner.targets.length; i<l; i++)
			turbo.aligner.alignChildren(turbo.aligner.targets[i]);
	}
	this.alignFrom = function(inElement) {
		if (turbo.aligner.enabled)
			turbo.aligner.alignChildren(inElement);
	}
	this.alignNow = function() {
		//dojo.debug('alignNow');
		turbo.aligner.alignFrom(document.body);
		turbo.aligner.alignTargets();
	}
	this.lastAlign = 0;
	this.align = function()	{
		if (!turbo.aligner.enabled)
			return;
		// only align at most every 100 ms
		//	if (turbo.aligner.lastAlign - turbo.time() <= -100)
		//{
			turbo.aligner.alignNow();
			//turbo.aligner.lastAlign = turbo.time();
		//}
	}
	this.alignLater = function(inDelay)	{
		//dojo.debug('alignLater');
		turbo.defer(turbo.aligner.align, (inDelay ? inDelay : 200));
	}
	this.enable = function(inDisableIfFalse) {
		turbo.aligner.enabled = (inDisableIfFalse !== false);
	}
	this.start = function()	{
		turbo.aligner.enable();
		turbo.aligner.alignLater(500);
	}
	this.addTarget = function(inElement) {
		var e = (dojo.lang.isString(inElement) ? turbo.$(inElement) : inElement);
		e.style.position = 'relative';
		turbo.aligner.targets.push(e);
	}
}


dojo.provide('turbo.lib.theme');
dojo.require('dojo.io.*');
dojo.require('dojo.html');
dojo.require('turbo.turbo');

turbo.stylesheet = { 
	cssFiles: [ ],
	links: [ ],
	rules: 0,
	loadCost: 0,
	processCost: 0,
	dummy: 0
}

turbo.stylesheet.create = function(inBase) {
	//turbo.debug('linking to theme base file ' + inBase + '/base.css');
	var l = document.createElement('link');
	l.setAttribute('rel', 'stylesheet');
	l.setAttribute('type', 'text/css');
	l.setAttribute('href', inBase + '/base.css');
	return l;
}

turbo.stylesheet.getLink = function(inBase) {
	var l = turbo.stylesheet.links[inBase];
	if (!l) {
		l = turbo.stylesheet.create(inBase);
		// have to add it right away or it will have to styleSheet property on IE
		turbo.addHeadNode(l);
		turbo.stylesheet.links[inBase] = l;
	}
	return l;
}

turbo.stylesheet.getLinkSheet = function(inLink) {
	var s = null;
	if (!inLink.addRule) 
	{
		turbo.debug('stylesheet has no addRule method');
		if (inLink.styleSheet)
			s = inLink.styleSheet;
		else 
			turbo.debug('turbo.stylesheet.getLinkSheet: link has no .styleSheet property');
	}
	return s;
}

turbo.stylesheet.loaded = function(inPath) {
	if (!inPath || turbo.stylesheet.cssFiles[inPath]) 
		return true;
	turbo.stylesheet.cssFiles[inPath] = true; 
	return false;
}

/*
turbo.stylesheet.append = function(inPath) {
	if (turbo.stylesheet.loaded(inPath))
		return;
	//turbo.debug('appending [CSS]: ' + inPath);		
	var t = turbo.time();
	dojo.io.bind({
		url: inPath,
		sync: true,
		load: function(inEvt, inData) { 
			//var lc = turbo.time() - t;
			//turbo.stylesheet.loadCost += lc;
			//
			//t = turbo.time();
			turbo.stylesheet._append(inPath, inData); 
			//t = turbo.time() - t;
			//turbo.stylesheet.processCost += t;
			//
			//var m = '...' + inPath.substr(25) + ' loaded/processed in ' + lc + '/' + t + 'ms [total time loading/processing/total sheets: ' + turbo.stylesheet.loadCost + '/' + turbo.stylesheet.processCost + '/' + (turbo.stylesheet.loadCost + turbo.stylesheet.processCost) + 'ms]';
			//turbo.debug(m);
		},
		error: function(e, m) { turbo.debug(m.message + ": " + inPath); }
	});
}

turbo.stylesheet._append = function(inPath, inData) {
	var s = turbo.stylesheet.getLink(turbo.pathpop(inPath)).styleSheet;
	if (!s || !s.addRule)
		return;
	//
	// space = [\s]*
	// comment start = \/\* 
	// ungreedy anything = [\s\S]*?
	// comment end = \*\/
	//
	// Remove /asterisk asterisk/ comments
	var pattern = /(\/\*[\s\S]*?\*\/)/g;
	inData = inData.replace(pattern, '');
	var pattern = /[\s]*([^{]*)({[^}]*})/g;
	while ((result = pattern.exec(inData)) != null) {
		var rule = result[2];
		var selectors = result[1].split(',');
		for (var i in selectors) 
			s.addRule(selectors[i], rule);
		for (var i in selectors) 
			turbo.debug('turbo.stylesheet._append: adding rule ' + selectors[i] + rule);
	}
}
*/

turbo.stylesheet.append = function(inPath) {
	turbo.stylesheet.importStyleSheet(inPath);
}

turbo.stylesheet.importStyleSheet = function(inPath) {
	if (turbo.stylesheet.loaded(inPath))
		return;
	//turbo.debug('importing [CSS]: ' + inPath);		
	dojo.html.insertCssFile(inPath, null, false, true); 
}

turbo.stylesheet.importThemeFile = function(inPath) {
	if (turbo.by_hand_css || djConfig["turbo_hand_css"]) // hand loaded CSS
		return;
	if (turbo.fine_css || djConfig["turbo_fine_css"]) { // fine-grained CSS
		if (dojo.render.html.ie)
			turbo.stylesheet.append(inPath); // use stylesheet accumulation for IE (slow)
		else
			turbo.stylesheet.importStyleSheet(inPath);
	}
	else {
		turbo.stylesheet.importStyleSheet(turbo.pathpop(inPath) + '/theme.css');		
	}
}

turbo.themes = new function() {
	this.theme = '';
	this.themeable = [ ];
	this.addThemeable = function(inObject) {
		this.themeable.push(inObject);
	}
	this.removeThemeable = function(inObject) {
		for (var i in this.themeable)
			if (this.themeable[i] == inObject) {
					this.themeable.splice(i, 1);
					return;
			}
	}
	this.setTheme = function(inTheme)	{
		this.theme = (inTheme && inTheme.toLowerCase() != 'xp' ? inTheme : 'default');
		for (var i=0, l=this.themeable.length; i<l; i++)
			this.themeable[i].setTheme(this.theme);
		if (turbo['aligner']) {
			turbo.defer(turbo.aligner.align, 250);
			turbo.defer(turbo.aligner.align, 1000);
		}
	}
	this.set = this.setTheme;
}

dojo.addOnLoad(function() {
  dojo.html.addClass(document.body, (dojo.render.html.moz ? 'turbo-gecko' : (dojo.render.html.ie ? 'turbo-ie' : '')));
});

dojo.provide("turbo.lib.drag");

dojo.require("turbo.turbo");

turbo.drag = { };

turbo.drag.dragging = false;
turbo.drag.hysteresis = 2;

turbo.drag.capture = function(inElement) {
	//turbo.debug('turbo.drag.capture');
	if (inElement.setCapture)
		inElement.setCapture();
	else {
		document.addEventListener("mousemove", inElement.onmousemove, true);
		document.addEventListener("mouseup", inElement.onmouseup, true);
		document.addEventListener("click", inElement.onclick, true);
	}
}

turbo.drag.release = function(inElement) {
	//turbo.debug('turbo.drag.release');
	if (inElement.releaseCapture)
		inElement.releaseCapture();
	else {
		document.removeEventListener("click", inElement.onclick, true);
		document.removeEventListener("mouseup", inElement.onmouseup, true);
		document.removeEventListener("mousemove", inElement.onmousemove, true);
	}
}

turbo.drag.start = function(inElement, inOnDrag, inOnEnd, inEvent, inOnStart) {
	if (/*turbo.drag.elt ||*/ !inElement || turbo.drag.dragging) {
		turbo.debug('failed to start drag: bad input node or already dragging');
		return;
	}
	turbo.drag.dragging = true;
	turbo.drag.elt = inElement;
	turbo.drag.events = { 
		drag: inOnDrag || turbo.nop, 
		end: inOnEnd || turbo.nop, 
		start: inOnStart || turbo.nop, 
		oldmove: inElement.onmousemove, 
		oldup: inElement.onmouseup, 
		oldclick: inElement.onclick 
	};
	turbo.drag.positionX = (inEvent && ('screenX' in inEvent) ? inEvent.screenX : false);
	turbo.drag.positionY = (inEvent && ('screenY' in inEvent) ? inEvent.screenY : false);
	turbo.drag.started = (turbo.drag.position === false);
	inElement.onmousemove = turbo.drag.mousemove;
	inElement.onmouseup = turbo.drag.mouseup;
	inElement.onclick = turbo.drag.click;
	turbo.drag.capture(turbo.drag.elt);
}

turbo.drag.end = function() {
	//turbo.debug("turbo.drag.end");
	turbo.drag.release(turbo.drag.elt);
	turbo.drag.elt.onmousemove = turbo.drag.events.oldmove;	
	turbo.drag.elt.onmouseup = turbo.drag.events.oldup;	
	turbo.drag.elt.onclick = turbo.drag.events.oldclick;	
	turbo.drag.elt = null;
	try {
		if (turbo.drag.started)
			turbo.drag.events.end();
	} finally {
		turbo.drag.dragging = false;
	}
}

turbo.drag.calcDelta = function(inEvent) {
	inEvent.deltaX = inEvent.screenX - turbo.drag.positionX;
	inEvent.deltaY = inEvent.screenY - turbo.drag.positionY;
}

turbo.drag.hasMoved = function(inEvent) {
	return Math.abs(inEvent.deltaX) + Math.abs(inEvent.deltaY) > turbo.drag.hysteresis;
}

turbo.drag.mousemove = function(inEvent) {
	inEvent = dojo.event.browser.fixEvent(inEvent);
	turbo.killEvent(inEvent);
	turbo.drag.calcDelta(inEvent);
	if ((!turbo.drag.started)&&(turbo.drag.hasMoved(inEvent))) {
		turbo.drag.events.start(inEvent);
		turbo.drag.started = true;
	}
	if (turbo.drag.started)
		turbo.drag.events.drag(inEvent);
}

turbo.drag.mouseup = function(inEvent) {
	//turbo.debug("turbo.drag.mouseup");
	turbo.killEvent(dojo.event.browser.fixEvent(inEvent));
	turbo.drag.end();
}

turbo.drag.click = function(inEvent) {
	turbo.killEvent(dojo.event.browser.fixEvent(inEvent));
	//turbo.drag.end();
}


dojo.provide('turbo.lib.app');
dojo.require('turbo.lib.align');
dojo.require('turbo.lib.scrim');
dojo.require('dojo.event.*');

turbo.app = {
	//this.marshall = turbo.marshall;
	marshall: function()
	{
		var id = '';
		var scope = dj_global;
		for (var i=0; i < arguments.length; i++) {
			id = arguments[i];
			if (!scope[id])
				scope[id] = turbo.$(id);
		}
		return scope[id];
	},
	onresize: function() {
		// onresize is called when the window size changes, a splitter is moved, or aligner.align is fired
		// connect to me or subscribe to turbo.resize topic
	},
	alignerAlign: function() {
		// connected to aligner.align
		this.onresize();
	},
	resize: function() {
		// provokes onresize
		turbo.aligner.align();
	},
	_windowResize: function() {
		// invoked from window.onresize
		turbo.app.resize();
		turbo.app.resizePending = null;
	},
	windowResize: function() {
		(!this.resizePending)&&(this.resizePending = turbo.defer(turbo.app._windowResize, 250));
	},
	display: function()	{
		turbo.aligner.enable();
		this._windowResize();	
		turbo.defer(turbo.app.windowResize, 500);
		if (turbo.turbo_disable_curtain || djConfig['turbo_disable_curtain'])
			this.showPageBody();
		else {
			turbo.scrim.release(this.scrim);
			this.scrim = null;
		}
	},
	encurtain: function()	{
		this.scrim = turbo.scrim.get(document.body, 'turboCurtain', {zIndex: 9999, backgroundColor: 'white'});
		this.showPageBody();
	},
	showPageBody: function() {
		document.body.style.display = 'block';
	},
	init: function() { },
	initialize: function() {
		if (!turbo.turbo_disable_curtain && !djConfig['turbo_disable_curtain'])
			this.encurtain();
		try {
			this.init();
		}	catch(e) {
			dojo.debug("turbo.app.init failed: " + e);
			turbo.debug(e);
		}
		turbo.defer(turbo.bind(this, this.display), 100);
		dojo.event.connect(turbo.aligner, 'align', this, 'alignerAlign');
		dojo.event.connect(window, 'onresize', this, 'windowResize');
	}
}

turbo.app.curtainHtml = "";

dojo.addOnLoad(turbo.app, 'initialize');

// help grids and aligner play nice
// browsers don't like resizing visible tables
if (false)
dojo.addOnLoad(function() {
	showHideGrids = function(inShow)
	{
		//turbo.debug('tda.app.installAlignFix: setGridsDisplay(' + inDisplay + ')');
		var grids = dojo.widget.getWidgetsByType("TurboGridClassic");
		for (var i=0; i<grids.length; i++)
			turbo.showHide(grids[i].domNode, inShow);
			//turbo.setVisibility(grids[i].domNode, inShow);
	}
	hideGrids = function() { showHideGrids(false); };
	showGrids = function() { showHideGrids(true); };
	dojo.event.connect('before', turbo.aligner, 'align', 'hideGrids');
	dojo.event.connect('after', turbo.aligner, 'align', 'showGrids');
});

// resize publisher
dojo.event.topic.registerPublisher("turboresize", turbo.app, "onresize");

// register resize listener like so:
// dojo.event.topic.subscribe("turboresize", obj, "resize");


dojo.provide("turbo.widgets.TurboWidget");

dojo.require("dojo.widget.*");
dojo.require("turbo.turbo");
dojo.require("turbo.lib.theme");
dojo.require("turbo.lib.align");

turbo.widgetRoot = "../turbo/widgets/";
turbo.templateRoot = turbo.widgetRoot + "templates/";
turbo.themeRoot = turbo.widgetRoot + "themes/";
turbo.themePath = dojo.hostenv.getBaseScriptUri() + turbo.themeRoot;

turbo.loadJs = function(inPath) {
	//turbo.debug('loading [JS]: ' + inPath);
	turbo.loadScript(dojo.hostenv.getBaseScriptUri() + inPath);
}

turbo.loadCss = function(inPath) {
	turbo.stylesheet.importThemeFile(dojo.hostenv.getBaseScriptUri() + inPath);
}

turbo.setWidgetType = function(inWidget, inType) {
	if (inWidget.widgetType == 'HtmlWidget')
		inWidget.widgetType = inType;
}

dojo.widget.defineWidget("dojo.widget.html.TurboWidget", dojo.widget.HtmlWidget,
	function() {
		this.styleRoot = (this.styleRoot || this.widgetType);
		this.setClassName = this.setStyledClass; 
		// patch inherited
		if (this._inherited)
			this.inherited = this._inherited;
	},{
	// paths
	themeRoot: turbo.themeRoot,
	templateRoot: turbo.templateRoot,
	// override some defaults
  isContainer: false,
  // useful properties
	align: '',
	debuggable: true,
	flex: '',
	layout: '',
	style: '',
	theme: '',
	themeable: true,
	themeJs: false,
	turboalign: '',
	// methods
	turboCreate: function() { },
	turboDestroy: function() { },
	setTemplatePath: function() {
		this.templatePath = dojo.uri.dojoUri(turbo.templateRoot + this.widgetType + ".html");
	},
	initialize: function(inArgs, inFrag, inParent) {
		// force into global namespace
		if (this.widgetId.substr(-2, 1) != '_' && this.widgetId.substr(-3, 1) != '_') // i.e. not auto-generated, like Widget_0
			dj_global[this.widgetId] = this;
		//	
		if (this.extraArgs["turboAlign"])
			this.turboalign = this.extraArgs.turboAlign;
		if (this.turboalign)
			this.domNode.setAttribute('turboalign', this.turboalign);
		if (this.layout)
			this.domNode.setAttribute('layout', this.layout);
		if (this.flex)
			this.domNode.setAttribute('flex', this.flex);
		//	
		this.className = this.extraArgs["class"];
		if (this.className)
			dojo.html.addClass(this.domNode, this.className);
		//
		var node = this.getFragNodeRef(inFrag);
		if (node!=this.domNode)
			this.domNode.style.cssText += node.style.cssText;
		//	
		this.domNode.id = this.widgetId;
		//
		if (this.themeable) {
			turbo.themes.addThemeable(this);
			if (!this.theme)
				this.theme = turbo.themes.theme;
			this.setTheme(this.theme);
		}
		//
		this.turboCreate();
	},
	uninitialize: function() {
		if (this.themeable)
			turbo.themes.removeThemeable(this);
		this.turboDestroy();
		window[this.widgetId] = null;
	},
	"default": function(inMethodName, inArguments) {
		this.constructor.prototype[inMethodName].apply(this, inArguments);
	},
	tie: function(inMethod /*, ...*/) {
		return turbo.tie.apply(turbo, [this].concat(turbo.cloneArguments(arguments)));
	},
	bindArgEvent: function(inEvent, inArgs)	{
		if (inArgs[inEvent])
			this[inEvent] = turbo.getFunction(inArgs[inEvent]);
	},
	bindArgEvents: function(inArgs) {
		// don't we need this for old Dojo?
		return;
		//var bound = [ ];
		for (var arg in inArgs)	{		
			if (/*!bound[arg] &&*/ dojo.lang.isFunction(this[arg])) {
				//bound[arg] = true;
				var value = inArgs[arg];
				if (value.search(/[^\w\.]+/i) == -1) {
					//turbo.debug('binding  "' + value + '" to "' + arg + '"');
					var func = turbo.getFunction(value);
					//if (func)
					//	this.debug('bindArgEvents', 'binding "' + arg + '" to "' + value + '"');
					if (func)
						this[arg] = func;
					else
						this.debug('bindArgEvents', 'could not bind "' + arg + '" to "' + value + '"');
				}
			}
		}
	},
	connectEvents: function(inNode /*,inEvents*/)	{
		for (var i=1, l=arguments.length; i<l; i++)
			turbo.connect(inNode, 'on' + arguments[i].toLowerCase(), this, 'do' + arguments[i]);	
	},
	getWidgetFragment: function(inFrag)	{
		return inFrag["dojo:"+this.widgetType.toLowerCase()];
	},
	createChildWidgets: function(inFrag) {
		/*var comps = */dojo.widget.getParser().createSubComponents(inFrag);
		/* could scan comps and do some kind of parenting operations here */
	},
	installChildren: function(inFrag, inParent)	{
		var parent = (inParent ? inParent : this.domNode);
		// children: move v. create:
		// if create first then children can have problems if parent changes
		// if move first then children can have problems if can't be inserted until parent changes
		// problematic if the node expects it's parent node to be something that it's not
		dojo.dom.moveChildren(this.getFragNodeRef(inFrag), parent);
		// problematic if child widgets expect their parent node to be something that it's not
		this.createChildWidgets(inFrag);
	},
	getStylePath: function(inStyle)	{
		return this.themeRoot + (inStyle ? inStyle : 'default') + '/' + this.styleRoot + inStyle;
	},
	loadStyle: function(inStyle) {
		var p = this.getStylePath(inStyle);
		//this.debug('loadStyle', 'turbo.loadCss(%s)', p);
		turbo.loadCss(p + ".css");
		if (this.themeJs)
			turbo.loadJs(p + "Theme.js");
	},
	setStyle: function(inStyle)	{
		this.style = (inStyle == 'default' || inStyle == 'XP' ? '' : inStyle);
		this.loadStyle(this.style);
		this.styleChanged();
	},
	setTheme: function(inTheme)	{
		this.loadStyle(''); // ensure we have defaults
		if (!this.themeable)
			return;
		if (inTheme && (inTheme.charAt(0) == '+')) {
			this.themeable = false;
			inTheme = inTheme.substring(1); 
		}
		this.setStyle(inTheme);
	},
	styleChanged: function() {
		// stub
	},
	setStyledClass: function(inObj, inSuffix) {
		if (!inSuffix)
			inSuffix = '';
		inObj.className = this.classTag + inSuffix + (this.style ? ' ' + this.classTag + this.style + inSuffix : '');
	},
	debug: function(inFunc, inMsg) {
		if (this.debuggable)
			turbo.debug(this.widgetId + ' [' + this.widgetType + ']: ' + inFunc + ': ' + turbo.printf.apply(turbo, turbo.cloneArguments(arguments, 1)));
		//if (this.debuggable)
		//	turbo.debug(this.widgetId + ' [' + this.widgetType + ']: ' + inFunc + ': ' + inMsg);
	},
	showHide: function(inShow) {	
		if (inShow)
			this.show();
		else
			this.hide();
	}
});
dojo.widget.TurboWidget = dojo.widget.html.TurboWidget;

dojo.getObject = function(/*String*/name, /*Boolean*/create, /*Object*/obj, /*Boolean*/returnWrapper){
	// summary: 
	//		gets an object from a dot-separated string, such as "A.B.C"
	//	description: 
	//		useful for longer api chains where you have to test each object in
	//		the chain
	//	name: 	
	//		Path to an object, in the form "A.B.C".
	//	obj:
	//		Optional. Object to use as root of path. Defaults to
	//		'dojo.global()'. Null may be passed.
	//	create: 
	//		Optional. If true, Objects will be created at any point along the
	//		'path' that is undefined.
	//	returnWrapper:
	//		Optional. Returns an object with two properties, 'obj' and 'prop'.
	//		'obj[prop]' is the reference indicated by 'name'.
	var tobj, tprop;
	if(typeof name != "string"){
		// clearly an error
		// FIXME: why is this here?
		return undefined; // Undefined
	}
	tobj = obj;
	if(!tobj){ tobj = dojo.global(); }
	var parts=name.split("."), i=0, lobj, tmp, tname;
	do{
		lobj = tobj;
		tname = parts[i];
		tmp = tobj[parts[i]];
		if((create)&&(!tmp)){
			tmp = tobj[parts[i]] = {};
		}
		tobj = tmp;
		i++;
	}while(i<parts.length && tobj);
	tprop = tobj;
	tobj = lobj;
	return (returnWrapper) ? { obj: tobj, prop: tname } : tprop; // Object
}