require("io")
require("lfs")
require("uuid")
local constants = require("shared.streaming.constants")
local os = require "os"
local string = require "string"
require "BackupService"

module(..., package.seeall)

--////////////////////////////////////////////////////////////////////////////////////////////////////////
-- FUNCIONES PRIVADAS
--////////////////////////////////////////////////////////////////////////////////////////////////////////
local function GetTemporalDirectory()
	return string.gsub(os.getenv("TEMP"), "\\", "\\\\") .. "\\inconcert\\web\\mediaServer\\"
end
--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
local function CreateNewFileName()
	if uuid then
		return uuid.new() .. ".bkp"
	end
	return "default.bkp"
end
--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
local function CreateDirectory(directoryName)
	assert(type(directoryName) == "string")
	os.execute("md " .. directoryName)
end
--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
local function GetTemporalFileName()
	local directory = GetTemporalDirectory()
	CreateDirectory(directory)
	return GetTemporalDirectory() .. "\\" .. CreateNewFileName()
end
--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
local function BuildStream(fileName)
	local file = io.open(fileName, "rb")
	if not file then
		return constants.StreamCreationResult.SOURCE_UNAVAILABLE
	end
	
	local stream = {
		__file = file
		,size = lfs.attributes(fileName).size
		,bitrate = 192
		,fileName = fileName
		,displayName = fileName:match("([^\\]*)$")
		,GetChunk = function(self, size)
			local buffer = self.__file:read(size)
			local status = (buffer and constants.StreamIOResult.MORE_DATA) or constants.StreamIOResult.LAST_CHUNK
			return buffer, status
		end
		,Release = function(self)
			self.__file:close()
	        LogInfo("Removing temporal backup file: %s ", fileName)
			os.remove(self.fileName)
		end
	}
	return stream
end

--////////////////////////////////////////////////////////////////////////////////////////////////////////
-- FUNCIONES PUBLICAS  
--////////////////////////////////////////////////////////////////////////////////////////////////////////
function iterator(stream)
	if not (stream) then
		return
	end
	repeat
		local data, status = stream:GetChunk(8192)
		if status == constants.StreamIOResult.LAST_CHUNK then
			LogInfo("streanBackup.interator: Se ha completado de enviar el archivo.")
			stream:Release()
		end
		local ok = coroutine.yield(data)
		if not ok then
			LogInfo("streanBackup.interator: El cliente ha coratdo la conexin.")
			stream:Release()
			return nil, "The client dropped the connection"
		end
	until status ~= constants.StreamIOResult.MORE_DATA
end
--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
function Init(partInformation, backupInformation, odbc)
	local headers = {}
	local stream, status
	local fileName = GetTemporalFileName()
	LogInfo("Temporal backup file: %s ", fileName)
	local backup = BackupService.Manager(odbc.mmprodat)
	local result = backup:StoreInteraction(partInformation, backupInformation, fileName)
	if result == 0 then
		stream = BuildStream(fileName)
		if stream == 0 then
			LogError("Backup file note exist: %s", fileName)
			status = "500 "
			headers = {
				["Content-Type"] = "text/plain"
			}
		else
			headers["Content-Length"] = stream.size
			status = "200 ok"
		end
	else
		LogError("Backup error cause number: %s", tostring(result))
		status = "500 "
		headers = {
			["Content-Type"] = "text/plain"
		}
	end
	return status, headers, coroutine.wrap(function() iterator(stream) end)
end
