require "io"
local constants = require "shared.streaming.constants"
require "lfs"
require "LibMmio"
require "LibLame"

module(..., package.seeall)

function Init(filename)
	local file = LibMmio.open(filename, "r")
	if not file then
		return constants.StreamCreationResult.SOURCE_UNAVAILABLE
	end
	
	file:seek(0, "set")
	-- busco el header RIFF o WAVE
	mmckinfoParent = LibMmio.MMCKINFO()
	mmckinfoParent.fccType = "WAVE"
	file:descend(mmckinfoParent, nil, "findriff")
	
	-- Busco y leo el chunk FMT
	mmckinfoSubchunk = LibMmio.MMCKINFO()
	mmckinfoSubchunk.ckid = "fmt"
	file:descend(mmckinfoSubchunk, mmckinfoParent, "findchunk")
	
	-- leo el formato del wav
	local data = file:read(LibMmio.WAVEFORMATEX.size)
	waveformat = LibMmio.WAVEFORMATEX(data)
	
	-- salgo del chunk FMT
	file:ascend(mmckinfoSubchunk)
	
	-- Busco y leo el chunk data	
	mmckinfoSubchunk.ckid = "data"
	file:descend(mmckinfoSubchunk, mmckinfoParent, "findchunk")
	
	local dataLength = mmckinfoSubchunk.cksize
	local isStereo
	if waveformat.nChannels == 1 then
		isStereo = false
	elseif waveformat.nChannels == 2 then
		isStereo = true
	else
		error("Invalid number of channels: " .. waveformat.nChannels)
	end
	
	local encoder = LibLame.Encoder()
	local status, err = encoder:Initialize(isStereo, waveformat.nSamplesPerSec, 192)
	if not status then
		file:close()
		error(err)
	end
	
	local stream = {
		size = dataLength,
		bitrate = 192,
		displayName = filename:match("([^\\]*)$"),
		numSamplesToEncode = encoder:SamplesToEncode(),
		maxBufferSize = encoder:MaxBufferSize(),
		
		__remainder = nil, -- ac me guardo temporalmente datos entre invocaciones a GetChunk
		GetChunk = function(self, size)
			local length = 0
			local read
			local collector = {}
			-- si me quedaron datos de la tanda anterior, los agrego ahora
			if self.__remainder then
				table.insert(collector, self.__remainder)
				length = length + #self.__remainder
				self.__remainder = nil
			end
			while(length < size) do
				read = file:read(self.numSamplesToEncode * 2)
				if read then
					read = encoder:Encode(read, #read)
					if read then
						table.insert(collector, read)
						length = length + #read
					end
				else
					read = encoder:Flush()
					if read then table.insert(collector, read) end
					return self:__SendOutput(table.concat(collector), size, constants.StreamIOResult.LAST_CHUNK)
				end
			end
			return self:__SendOutput(table.concat(collector), size, constants.StreamIOResult.MORE_DATA)
		end,
		
		-- usualmente encodeo ms de lo que tengo que mandar, entonces mando solo lo pedido y el resto lo guardo para el prximo envo
		__SendOutput = function(self, data, size, result)
			if data then
				self.__remainder = string.sub(data, size + 1)
				return string.sub(data, 1, size), result
			else
				return "", result
			end
		end,
		
		Release = function(self)
			file:close()
		end,
	}
	
	return stream
end