local oo = require("loop.base")
require("LuaOrm.Mapper")
require("LuaOrm.Entity")
require("LuaOrm.Model")
require("LuaOrm.DBMS.DataSource")
require("LuaOrm.Associations.Associations")
require("LuaOrm.Associations.AssociationArray")
local jsonEncoder = require("LuaOrm.Serializers.Json")
require("json")
require("LuaOrm.Date")
require("inconcert.translators")
require "luasql.mysql"
local rpairs = LuaOrm.Associations.AssociationArray.Iterator
local m_filterInstances = nil

module(..., package.seeall)
LogInfo("Inicializo Modulo 'filter'")

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function new(moduleName, p_applicationId)
	local applicationId = p_applicationId or moduleName
	if not (_M.m_filterInstances) then
		_M.m_filterInstances = {}
	end
	_M.m_filterInstances[moduleName] = _M.m_filterInstances[moduleName] or {
		moduleName = moduleName,
		applicationId = applicationId,
		listOfTables = {},
		DynamicData = {
			data = nil,
			Populate = function(self, entity)
				local fieldId = string.upper(entity.fieldId)
				local values = {}
				if (self.data ~= nil and self.data[fieldId] ~= nil) then
					if not entity.suggestedURI then
						values = self.data.GetRows(self.data[fieldId], entity)
					end
				end
				return values
			end
		},
		InitializeTables = _M.InitializeTables,
		GetConfig = _M.GetConfig,
		GetFilterModel = _M.GetFilterModel,
		GetList = _M.GetList,
		Save = _M.Save,
		Delete = _M.Delete,
		SetDynamicData = _M.SetDynamicData,
		SetPopulateFunction = _M.SetPopulateFunction,
		Serialize = _M.Serialize,
		GetDefaultFilter = _M.GetDefaultFilter,
		GetTranslator = _M.GetTranslator,
		GetSuggestedList = _M.GetSuggestedList
	}
	return _M.m_filterInstances[moduleName]
end

-- lista de tablas utilizadas, cada uno de estos elementos de la tabla contiene el objeto 'Model' de LuaORM
listOfTables = {
	['filterHeader'] = nil
	,['filterDetail'] = nil
	,['filterComboData'] = nil
	,['filterCombo'] = nil
}

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
local function CreateConnection(database)
	if not database then
		local err = "Connection error using suggested list in filter, not defined the data source."
		return nil, err
	end
	local env = luasql["mysql"]()
	local connectionString = ""
	for property, value in pairs(database) do
		connectionString = connectionString .. property .. "=" .. value .. ";"
	end
	local conn, err = env:connect(connectionString, "", "")
	if not conn then
		return nil, err
	end
	return conn
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function SetPopulateFunction(self, populateFunction)
	if (populateFunction) then
		self.DynamicData = populateFunction
	end
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function SetDynamicData(self, data)
 	self.DynamicData.data = data
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetTranslator(self, applicationId)
	return inconcert.translators[applicationId]
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Defino las tablas a utilizar con sus realciones
function InitializeTables(self, dataSource)
	LogDebug("filter.InitializeTables")	
	LogDebug('LuaOrm.Model-> [filterHeader] - os:clock(): ' .. os:clock())
	self.listOfTables['filterHeader'] = LuaOrm.Model {
		tableName = "filterHeader"
		,dataSource = dataSource
		,Associations = {
			hasMany("filterDetail", nil, true)
		}
		,Attributes = {
			DisplayFilterName = function(entity)
				--solo traduzco el valor predeterminado "Default" ya que sale de los datos iniciales.
				-- los demas valores no se traducen ya que son datos del usuario.
				-- (filtros guardados con algun nombre que porpociono el usuario)
				if entity.filterId:lower() == "default" then
					return _T("Default")
				end
				return entity.filterId
			end
		}
	}
	LogDebug('LuaOrm.Model-> [filterDetail] - os:clock(): ' .. os:clock())
	self.listOfTables['filterDetail'] = LuaOrm.Model {
		tableName = "filterDetail"
		,dataSource = dataSource
		,Associations = {
			hasMany("filterCombo" , nil, true),
			belongsToOne("filterHeader", nil, false)
		}
		,Attributes = {
			caption = function(entity)
				-- el translator de la aplicacion que instancio al filtro
				local translator = inconcert.translators[self.applicationId] 
				return translator:Text(entity.captionText)
			end
			,dynamicValues = function(entity)
				local dynamicValues = nil
				--Cargo datos dinmicos de los combos
				-- la tabla 'DynamicData' es una tabla lua que se carga dinamicamente segun la aplicaciones que utilice el filtro, debe implementar la funcion 'Populate', esta
				-- debe retornar una tabla con los valores a llenar en el combo dinamico.
				--LogDebug("")
				--LogDebug("(filtro) (InitializeTables) -> _M.DynamicData= " .. tostring(_M.DynamicData))
				--LogDebug("")
				local dynamicDataObject = self.DynamicData
				dynamicValues = dynamicDataObject:Populate(entity)
				return dynamicValues
			end
		}
	}
	LogDebug('LuaOrm.Model-> [filterComboData] - os:clock(): ' .. os:clock())
	self.listOfTables['filterComboData'] = LuaOrm.Model {
		tableName = "filterComboData"
		,dataSource = dataSource
		,Associations = {}
		,Attributes = {
			translatedValue = function(entity)
				-- el translator de la aplicacion que instancio al filtro
				local translator = inconcert.translators[self.applicationId] 
				return translator:Text(entity.Value)
			end
		}
	}
	LogDebug('LuaOrm.Model-> [filterCombo] - os:clock(): ' .. os:clock())
	self.listOfTables['filterCombo'] = LuaOrm.Model {
		tableName = "filterCombo"
		,dataSource = dataSource
		,Associations = {
			belongsToOne("filterDetail", nil, true),
			belongsToOne("filterComboData", nil, true)
		}
	}
	-- retorna la lista de tablas inicializadas
	return self.listOfTables
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Fucion encargada de actualizar los registros en la BD, en las tablas (filterHeader y filterDetail)
function Update(self, myFilter, filterFromClient)
	LogDebug("voy a realizar Update")
	-- Recorro la lista de registros de FilterDetail y los actualizo con los valores recibidos del cliente
	for rowIndex, row in rpairs(myFilter.filterDetails) do
		local valueToAssign = filterFromClient.filterDetails[rowIndex].value
		row.value = valueToAssign
	end
	LogDebug('myFilter:Save()')
	myFilter:Save()
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Funcion  que se encarga de llamar a la funcion (InsertInHeaderTable)  para que esta  agregue los nuevo registros a los modelos de las tablas en LuaORM.
--	Esta funcion  (InsertInHeaderTable)  va llamando a otras funciones que van agreando registros a las tablas LUAS correspondientes (Realcion entre tablas)
--        Luego de obtener estas tablas LUAS, se  manda guardar a LuaORM, para que este se encargue de actualizar la BD.
function Inserts(self, filterFromClient)
	LogDebug("voy a procesar INSERT")
	-- obtengo los registros nuevos  del filtro a guardar:
	--	 myFilterHeader = 1 registro de la tabla (filterHeader)
	local myFilterHeader = _M.InsertInHeaderTable(self, filterFromClient)
	-- Mando a que LuaORM guarde los nuevos registros
	LogDebug('-myFilterHeader:Save() Antes')
	if (myFilterHeader:Save() == true) then
		LogInfo("Nuevo filtro agregado con xito.")
	else
		LogError("Error al gurardar el nuevo filtro.")
	end
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Esta funcion es la encargada de agregar los nuevos registros de la tabla (filterHeader) en LuaORM
--	a su vez esta llama a la tabla realcionada (filterDetail) para que haga lo mismo
function InsertInHeaderTable(self, filterFromClient)
	LogInfo(' Inserto Registro en filterHeader')
	local myFilterHeader = self.listOfTables['filterHeader']:New()
	-- recorro los campos del filtro (filterHeader) obtenido desde el cliente y actulizo los valores del registro a impcatar en las Tablas de la BD
	for fieldName, valueOfField in pairs(filterFromClient) do
		if (type(valueOfField) ~= "table") then
			myFilterHeader[fieldName] = valueOfField
		end
	end
	-- Si filterDetail es una tabla mando a realizar los insert a esta tabla
	if (type(filterFromClient.filterDetails) == "table") then
		_M.InsertInDetailTable(self, filterFromClient.filterDetails, myFilterHeader)
	end
	return myFilterHeader
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Esta funcion es la encargada de agregar los nuevos registros de la tabla (filterDetail) en LuaORM
--	a su vez esta llama a la tabla realcionada (filterCombo) para que haga lo mismo
--	(esto es para los campos de tipo SELECT (Combo))
function InsertInDetailTable(self, filterHeaderFromClient, myFilterHeader)
	-- Recorro cada uno de los registros de la tabla recibida desde el cliente correspondiente a (FilterDetail)
	for keyReg, row in pairs(filterHeaderFromClient) do
		local myFilterDetail = self.listOfTables['filterDetail']:New()
		-- Recorro cada uno de los campos del registro de la tabla (FilterDetail)
		for fieldName, valueOfField in pairs(row) do
			if (type(valueOfField) ~= "table") then
				myFilterDetail[fieldName] = valueOfField
			end
		end
		-- Si es un elememtno HTML tipo Select (Combo), no es dinmica la carga de este y tiene valores asociados, mado insertar en la tabla de Combos los valores de este.
		if (row.tagId == "select") then
			if (row.isDynamic == 0 and type(row.filterCombos) == "table") then
				_M.InsertInDetailsValueTable(self, row.filterCombos, myFilterDetail)
			end
		end
		myFilterHeader.filterDetails:Add(myFilterDetail)
	end
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Esta funcion es la encargada de agregar los nuevos registros de la tabla (filterCombo) en LuaORM
--	Estos son los valors de los campos  de tipo SELECT (Combo))
function InsertInDetailsValueTable(self, filterDetailFromClient, myFilterDetail)
	-- Recorro cada uno de los registros de la tabla recibida desde el cliente correspondiente a (FilterCombo)
	for keyReg, row in pairs(filterDetailFromClient) do
		local myFilterDetailValue = self.listOfTables['filterCombo']:New()
		-- Recorro cada uno de los campos del registro de la tabla (FilterCombo)
		for fieldName, valueOfField in pairs(row) do
			if (type(valueOfField) ~= "table") then
				myFilterDetailValue[fieldName] = valueOfField
			end
		end
		myFilterDetail.filterCombos:Add(myFilterDetailValue)
	end
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- funciones publicas
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Funcion que retorna una tabla lua con las tablas utilizadas por el filtro
function GetFilterModel(self, dataSource)
	LogDebug('filter\init.lua - GetFilterModel (Entra) - dataSource = ' .. tostring(dataSource))
	self:InitializeTables(dataSource)
	return self.listOfTables
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Esta funcion retorna una tabla con la configuracion del filtro 
--    El parmetro que se recibe "buttonsExcluded" es array que contiene los botones a excluir (los que no se desean incluir en la botonera del filtro)
--          ejemplo de paramtro recibido     -- >   {[1] = "save",[2] = "reset"}
--              en este caso no incluiriamos los botones "save" y "reset" y solo se mostraria el boton 
--              "apply"
function GetConfig(self, buttonsExcluded)
	local filterTranslator = inconcert.translators.Filter
	local _T = function(...)
		return filterTranslator:Text(...)
	end
	local buttons = {
		apply = _T("Search"),
		reset = _T("Clean"),
		save = _T("Save")
	}
	if (buttonsExcluded) then
		for i, button in ipairs(buttonsExcluded) do
			buttons[button] = nil
		end
	end

	return { 
		applyButtonCaption = buttons.apply
		,resetButtonCaption = buttons.reset
		,saveButtonCaption  = buttons.save
		,refreshButtonCaption = _T("Refresh")
		,popUpTitle  = _T("Save Filter")
		,popUpLabelText  = _T("Filter Name")
		,popUpLabelCheck  = _T("Apply Filter")
		,title  = _T("Filter")
		,message
		,messages = {
			alreadyExist = _T("The filter already existes in the Filter list."),
			notOverwriteFilterSystem = _T("You can not overwrite filter system."),
			toolTipSuggestedCombo = _T("Start Search")
		}
	}
end

	--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	--
	function GetDefaultFilter(self)
	if not(self.filterList) then
		return nil
	end
	for _, filterHeader in pairs(self.filterList) do
		-- si el grupoid es vacio es el filtro por defecto
		if (filterHeader.groupId == "") then
			local defaultFilter = {__filterType = filterHeader.filterType}
			for _,filter in rpairs(filterHeader.filterDetails) do               
				if (filter.tagId ~= "select") then
					defaultFilter[filter.fieldId] = filter.value
				else
					defaultFilter[filter.fieldId] = {
						selectedValue = "__UNSELECTED",
						values = {}
					}
					local comboValues = filter.filterCombos
					local iterator = rpairs
					if (filter.isDynamic == 1) then               
						comboValues = filter.dynamicValues
						iterator = pairs          
					end
					if (comboValues) then
						local i = 0
						for _,filterValue in iterator(comboValues) do
							i = i + 1 
							defaultFilter[filter.fieldId].values[i] = filterValue.ItemId
						end
					end
					
				end
			end
			return defaultFilter
		end
	end
	return nil
	end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- funcion encargada de buscar en la BD la lista de filtros existentes para la aplicacion Web
--	una vez encontrados estos, los agregar en el formato de filtro que espera InconcerFilter.js
function GetList(self, applicationId, userId, vcc, pageName, groupId)
	--LogDebug('GetList - applicationId = "' ..applicationId .. '", userId = "' .. userId .. '", vcc = "' .. vcc .. '", pageName = "' .. pageName .. '", groupId = "' .. groupId .. '"')
	--LogDebug('antes del FindAll')	
	local filterList = self.listOfTables['filterHeader']:FindAll("(applicationId = ? and pageName = ?) and ((userId = ? and vcc = ?) or (groupId = ?))", 
																	{applicationId, pageName, userId, vcc, groupId, order = "filterId"})
	if (#filterList == 0) then
		filterList = self.listOfTables['filterHeader']:FindAll("(applicationId = ? and pageName = ?) and ((userId = ? and vcc = ?) or (groupId = ''))", 
																	{applicationId, pageName, userId, vcc, order = "filterId"})
	end
	--LogDebug('despues del FindAll  - filterList = ' .. tostring(filterList))
	self.filterList = filterList
	local jsonFilterList = ''
	local containsDetail = false
	if (filterList) then
		local encoder = oo.new(jsonEncoder)   
		local i = 0	
		jsonFilterList = '{'
		-- recorro la lista de registros y los agrego a la lista de filtros que utiliza InconcertFilter.js
		for k, v in pairs(filterList) do
			i = i + 1
			if (i > 1) then
				jsonFilterList = jsonFilterList .. ', '
			else
				containsDetail = true
			end
			local jsonFilter = encoder:Serialize(v, false, nil, nil, {"filterDetails.filterCombos.filterComboData"}) 
			-- el FilterHeader
			jsonFilterList = jsonFilterList ..
				'"' .. v.filterId .. '": ' .. jsonFilter
		end
		jsonFilterList = jsonFilterList .. '}'
	end
	--LogDebug('filter.Get - Salgo de la funcion y retorno el filtro')
	return jsonFilterList, containsDetail
end	

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function Serialize(self, hasLoadFilterList, applicationId, userId, vcc, pageName, groupId, buttonsExcluded)
	local jsonFilterList = ''
	local containsDetail = false
	if (tonumber(hasLoadFilterList) == 1) then
		jsonFilterList, containsDetail = self:GetList(applicationId, userId, vcc, pageName, groupId)
	    jsonFilterList = ', "filtersList": ' .. jsonFilterList
	end	
	return '{' ..
		'"config": ' .. json.encode(self:GetConfig(buttonsExcluded)) ..
		jsonFilterList ..
	'}',
	containsDetail
end	

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function UpdateEntitySystem(filter)
	if (string.len(filter.groupId) > 0) then
		ConvertToNonEntitySystem(filter)
		if (filter.filterDetails) then
			for detailKey, detailEntity in pairs(filter.filterDetails) do
				ConvertToNonEntitySystem(detailEntity)
				 if (filter.filterCombos) then
					for comboKey, comboEntity in pairs(filter.filterCombos) do
						ConvertToNonEntitySystem(comboEntity)              
					end
				end
			end
		end
	end
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function ConvertToNonEntitySystem(entity)
	entity.groupId = ''
	entity.userId = inconcert.authUser.userName
	entity.vcc = inconcert.authUser.virtualContactCenter
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Funcion encargada de derivar las solicitud de Guardar el filtro, ya sea para Actualizar o para Ingresar un filtro Nuevo.
function Save(self, filter, assignedToUser)
	if assignedToUser ~= false then
		_M.UpdateEntitySystem(filter)
	end
	LogDebug('filter.Save - applicationId = ' .. filter.applicationId)
	LogDebug('filter.Save - pageName = ' .. filter.pageName)
	LogDebug('filter.Save - userId = ' .. filter.userId)
	LogDebug('filter.Save - vcc = ' .. filter.vcc)
	LogDebug('filter.Save - groupId = ' .. filter.groupId)
	LogDebug('filter.Save - filterId = ' .. filter.filterId)
	-- busca si existe el filtro
	local myFilter = self.listOfTables['filterHeader']:Find({
												applicationId = filter.applicationId,   
												pageName = filter.pageName,
												userId = filter.userId, 
												vcc = filter.vcc,
												groupId = filter.groupId,
												filterId = filter.filterId
											})
	if (myFilter) then
		-- si el fitro existe lo actualizo
		_M.Update(self, myFilter, filter)
	else
		-- si el fitro No existe agrego los registros correspondientes
		_M.Inserts(self, filter)
	end
	return ""
end

function Delete(self, filter)
	LogDebug('filter.Delete - applicationId = ' .. filter.applicationId)
	LogDebug('filter.Delete - pageName = ' .. filter.pageName)
	LogDebug('filter.Delete - userId = ' .. filter.userId)
	LogDebug('filter.Delete - vcc = ' .. filter.vcc)
	LogDebug('filter.Delete - groupId = ' .. filter.groupId)
	LogDebug('filter.Delete - filterId = ' .. filter.filterId)
	-- busca si existe el filtro
	local filterHeader = self.listOfTables['filterHeader']:Find({
												applicationId = filter.applicationId,   
												pageName = filter.pageName,
												userId = filter.userId, 
												vcc = filter.vcc,
												groupId = filter.groupId,
												filterId = filter.filterId
											})
	if not filterHeader then
		return false
	end
	local result, err
	for index, fd in rpairs(filterHeader.filterDetails) do
		if fd.filterCombo then
			if fd.filterCombo.filterComboData then
				result, err = fd.filterCombo.filterComboData:TransactionalDelete()
			end
			result, err = fd.filterCombo:TransactionalDelete()
		end
		result, err = fd:TransactionalDelete()
	end
	result, err = filterHeader:TransactionalDelete()
	return true
end

function GetSuggestedList(self, fieldId, inputWord)
	local suggestedList = '<?xml version="1.0" encoding="UTF-8"?>'
	suggestedList = suggestedList .. '<complete>'
	if not inputWord then
		LogError("Error getting suggested list. Has not entered a search. " .. tostring(inputWord))
		suggestedList = suggestedList .. '</complete>'
		return suggestedList
	end
	local dynamicDataObject = self.DynamicData
	if not dynamicDataObject then
		LogError("Error getting suggested list. Has not exist module 'dynamic data' in virutal machine.")
		suggestedList = suggestedList .. '</complete>'
		return suggestedList
	end
	if not dynamicDataObject.Get and (not dynamicDataObject.data or not dynamicDataObject.data.Get) then
		LogError("Error getting suggested list. Has not defined 'Get' function in module 'dynamic data'.")
		suggestedList = suggestedList .. '</complete>'
		return suggestedList
	end
	local dynamicInfo
	if dynamicDataObject.Get then
		dynamicInfo = dynamicDataObject:Get(fieldId)
	else
		dynamicInfo = dynamicDataObject.data:Get(fieldId)
	end
	if not dynamicInfo then
		LogError("Error getting suggested list. Has not entered a search. fieldid  " .. tostring(fieldId))
		suggestedList = suggestedList .. '</complete>'
		return suggestedList
	end	
	local conn, err = CreateConnection(dynamicInfo.database)
	if (conn == nil) then
		LogError("Error getting suggested list. %s", tostring(err))
		suggestedList = suggestedList .. '</complete>'
		return suggestedList
	end
	local query = dynamicInfo:BuildQuery(inputWord)
	if (query == nil) then
		LogError("Error getting suggested list. Query error %s", tostring(query))
		suggestedList = suggestedList .. '</complete>'
		return suggestedList
	end	
	local rows = {}
	local total = 0
	local cursor, err = conn:execute(query)
	if not cursor then 
		LogError("Error getting suggested  list: %s", err.ErrorDescription)
	else
		local row, fetched = cursor:fetch({}, "a"), 1
		while row and (not count or fetched <= count) do
			suggestedList = suggestedList .. '<option value="' .. row[dynamicInfo.itemId] .. '">' .. row[dynamicInfo.itemDescription] .. '</option>'
			row, fetched = cursor:fetch({}, "a"), fetched + 1
		end
		cursor:close()
	end
	suggestedList = suggestedList .. '</complete>'
	conn = nil
	return suggestedList
end
