require("LuaOrm.Mapper")
require("LuaOrm.Entity")
require("LuaOrm.Model")
require("LuaOrm.DBMS.DataSource")
require("LuaOrm.Associations.Associations")
local luaSerializer = require("LuaOrm.Serializers.Lua")
local m_permissioninstances = nil

module(..., package.seeall)--, inconcert.orbit.application.Decorate)

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function new(applicationName)
	if not (_M.m_permissioninstances) then
		_M.m_permissioninstances = {}
	end
	_M.m_permissioninstances[applicationName] = _M.m_permissioninstances[applicationName] or {
		applicationName = applicationName,
		models = {},
		InitializeTables = _M.InitializeTables,
		GetActions = _M.GetActions,
		LoadActions = _M.LoadActions,
		LoadApplications = _M.LoadApplications,
		LoadVccLicences = _M.LoadVccLicences,
		GetPermissionsModel = _M.GetPermissionsModel,
		IsActionEnabled = _M.IsActionEnabled,
		IsLicenseEnabled = _M.IsLicenseEnabled,
		HasPermissionToAction = _M.HasPermissionToAction,
		HasPermissionTo = _M.HasPermissionTo,
		CheckPermission = _M.CheckPermission,
		GetFirstAvailableAction = _M.GetFirstAvailableAction,
		AddAction = _M.AddAction,
		RemoveAction = _M.RemoveAction
	}
	return _M.m_permissioninstances[applicationName]
end

--[=[
obtiene la primer accion disponible
]=]
function GetFirstAvailableAction(self, domain)
	local domain = domain or "navigation"
	local domainActions = inconcert.session.userpermissions[domain]
	if (domainActions == nil) then
		return nil
	end
	local firstAction = next(SharedSessions.ShallowCopy(domainActions))
	return domainActions[firstAction]
end

--[=[
Verifica que se tengan permisos para realizar una accin dada. Si no se indica el domain, se asume 'navigation'
]=]
function HasPermissionToAction(self, action, domain)
	return true
	--[[if not action then
		return nil
	end
	local domain = domain or "navigation"
	local domainActions = inconcert.session.userpermissions[domain]
	if not domainActions then
		return nil
	end
	return domainActions[action]
	]]
end

function HasPermissionTo(self, action, domain)
	if self:HasPermissionToAction(action, domain) then
		return true
	end
	LogInfo("You not have permission to action '%s', domain '%s'.", action, domain)
	return false
end

function CheckPermission(self, action, domain, functionToCall)
	return function(web, ...)
		if self:HasPermissionTo(action, domain) then
			return functionToCall(web, ...)
		end
		LogWarning("403 forbidden - action: '%s', domain: '%s' on application: '%s'", action, domain, self.applicationName)
		web.status = "403"
		return "forbidden"
	end
end

function GetActions(self, domain, category)
	if not inconcert.session.userpermissions then
		return nil
	end
	local domainActions = inconcert.session.userpermissions[domain]
	if not domainActions then
		return nil
	end
	if (category) then
		local categoryActions = {}
		for k, v in pairs(SharedSessions.ShallowCopy(domainActions)) do
			if v.Category == category then
				categoryActions[v.Id] = SharedSessions.DeepCopy(v)
			end
		end
		return categoryActions
	else
		return SharedSessions.DeepCopy(domainActions)
	end
end

function IsLicenseEnabled(self, vcc, licensename) 
	if (licensename == nil or licensename == "") then
		return true
	end
	local list = self.models['PermissionVCCLicense']:FindAll("VCC = ? and LicenseId = ?", {vcc, licensename})
	if list then
		for k, v in pairs(list) do
			if (v.LicenseId == licensename and v.Total > 0) then
				return true
			end
		end
	end
	return false
end

function IsActionEnabled(self, action, domain)
	if (inconcert.session.userpermissions and domain and action) then
		local domainActions = inconcert.session.userpermissions[domain]
		return domainActions[action] ~= nil
	else
		return false
	end
end

--es un join entre lo que tenga el grupo mas lo que tenga el usuario
function LoadActions(self) 
	local vcc = inconcert.authUser.virtualContactCenter
	local user = inconcert.authUser.userName
	local group = inconcert.authUser.userGroup
	
	local actions = {}
	
	local groupActions =  self.models["GroupPermission"]:FindAll(
		"ApplicationId = ? and GroupId = ? and (VCC = ? or VCC = ?) ", {self.applicationName, group, "system", vcc})
	for k, groupAction in pairs(groupActions) do
		actions[groupAction.ActionId] = groupAction.ApplicationAction
	end
	local userActions = self.models["UserPermission"]:FindAll("ApplicationId = ? and UserId = ? and VCC = ?", {self.applicationName, user, vcc})
	for k, userAction in pairs(userActions) do
		local action = userAction.ApplicationAction
		if (userAction.Enabled == 0) then
			actions[userAction.ActionId] = nil
		else
			actions[userAction.ActionId] = action
		end
	end
	--reagrupo las acciones por categoria...
	local domainactions = {}
	for k, v in pairs(actions) do
		if self:IsLicenseEnabled(vcc, v.License) then
			LogDebug("User '%s' has action '%s' licensed by '%s'", tostring(user), tostring(v.Id), tostring(v.License))
			if not domainactions[v.Domain] then
				domainactions[v.Domain] = {}
			end
			domainactions[v.Domain][v.Id] = luaSerializer:Serialize(v)
		else
			LogDebug("User '%s' does not have action '%s' licensed by '%s'", tostring(user), tostring(v.Id), tostring(v.License)) 
		end
	end
	inconcert.session.userpermissions = domainactions
end

function LoadApplications(self, where, values)
	--parametros opcionales
	return self.models["Application"]:FindAll(where, values) or {}
end

function LoadVccLicences(self)
	local vcc = inconcert.authUser.virtualContactCenter
	return self.models["PermissionVCCLicense"]:FindAll("VCC = ?", {vcc}) or {}
end

function InitializeTables(self, dataSource)
	self.models['Application'] = LuaOrm.Model {
		tableName = "Application",
		dataSource = dataSource,
		Associations = {
			hasMany("ApplicationAction", nil, true),
			hasMany("ApplicationLicense", nil, true)
		},
		Attributes = {
			DisplayDescription = function(entity) 
				return _T(entity.Description)
			end
		}
	}

	self.models['MenuApplicationCategory'] = LuaOrm.Model {
		tableName = "MenuApplicationCategory",
		dataSource = dataSource,
		Associations = {
			hasMany("ApplicationAction", {ApplicationId = "ApplicationId", Category = "Id"})
		}
	}

	self.models['ApplicationAction'] = LuaOrm.Model {
		tableName = "ApplicationAction",
		dataSource = dataSource,
		Associations = {
			belongsToOne("MenuApplicationCategory", {ApplicationId = "ApplicationId", Id = "Category"}),
			hasOne("MenuApplicationAction", nil, true)
		},
		Attributes = {
			categoryOrder = function(action)
				if (action.MenuApplicationCategory == nil) then
					return
				end
				return action.MenuApplicationCategory.Order
			end,
			actionOrder = function(action)
				if (action.MenuApplicationAction == nil) then
					return
				end
				return action.MenuApplicationAction.Order
			end,
			displayDomain = function(action)
				if not action.Domain then
					return ""
				end
				return _T(action.Domain)
			end,
			displayCategory = function(action)
				if not action.Category then
					return ""
				end
				local category = action.Category
				local applicationId = action.ApplicationId:lower()
				if applicationId == "reports" or applicationId == "scriptinglauncher" then
					if category:lower() == "system" then
						return _T(category)
					elseif category:lower() == inconcert.authUser.virtualContactCenter:lower() then
						if applicationId == "reports" then
							return _T("custom")
						else
							return _T("launch")
						end
					end
				end
				return _T(category)
			end,
			displayDescription = function(action)
				if not action.Description then
					return ""
				end
				return _T(action.Description)
			end
		}
	}

	self.models['MenuApplicationAction'] = LuaOrm.Model {
		tableName = "MenuApplicationAction",
		dataSource = dataSource,
		Associations = {}
	}

	self.models['GroupApplication'] = LuaOrm.Model {
		tableName = "GroupApplication",
		dataSource = dataSource,
		Associations = {
			belongsToOne("Application")
		}
	}

	self.models['UserApplication'] = LuaOrm.Model {
		tableName = "UserApplication",
		dataSource = dataSource,
		Associations = {}
	}

	self.models['UserPermission'] = LuaOrm.Model {
		tableName = "UserPermission",
		dataSource = dataSource,
		Associations = {
			belongsToOne("ApplicationAction")
		}
	}

	self.models['GroupPermission'] = LuaOrm.Model {
		tableName = "GroupPermission",
		dataSource = dataSource,
		Associations = {
			belongsToOne("Application"),
			belongsToOne("ApplicationAction")
		}
	}

	self.models['PermissionVCCLicense'] = LuaOrm.Model {
		tableName = "VCCLicense",
		name = "PermissionVCCLicense",
		dataSource = dataSource,
		Associations = {}
	}

	self.models['ApplicationLicense'] = LuaOrm.Model {
		tableName = "ApplicationLicense",
		dataSource = dataSource,
		Associations = {}
	}
	-- retorna la lista de tablas inicializadas
	return self.models
end

--[=[
Funcion que retorna una tabla lua con las tablas utilizadas por el modelo de permisos
--]=]
function GetPermissionsModel(self, dataSource)
	self:InitializeTables(dataSource)
	return self.models
end

--[[
Funcion para ingresar una accin a los permisos
]]
function AddAction(self, Id, Domain, Category, Description, License, Data)
	local action = {
		Id = Id,
		ApplicationId = self.applicationName,
		Domain = Domain,
		Category = Category,
		Description = Description,
		License = License,
		Data = Data,
	}
	
	local dbAction = self.models.ApplicationAction:Find (action)
	if dbAction then
	    for k,v in pairs (action) do
	        dbAction[k] = v
	    end
        action = dbAction
	else
	    action = self.models.ApplicationAction:New(action)
	end
	
	if action then
		action:Save()
	end
end

--[[
Funcion para eliminar un permiso (cambia las tablas ApplicationAction, GroupPermission y UserPermission)
]]
function RemoveAction(self, Id)
	--Elimino los permisos de la accion asociados al grupo
	local groupPermissions = self.models.GroupPermission:FindAll ("ApplicationId = ? And ActionId = ?" , {self.applicationName, Id})	
	for k, v in pairs(groupPermissions) do 
	    v:Delete()
	end
	--Elimino los permisos de la accion asociados al grupo	
	local userPermissions = self.models.UserPermission:FindAll ("ApplicationId = ? And ActionId = ?" , {self.applicationName, Id})	
	for k, v in pairs(userPermissions) do 
		v:Delete()
	end
	--Finalmente la accion
	local action = self.models.ApplicationAction:Find {Id = Id, ApplicationId = self.applicationName}
	if action then
		action:TransactionalDelete()
	end
end
