/*jslint  evil:true */
/*global $, jQuery window

*/

/*!
	* Plug-In				:	jQuery Password Strength
	* Version				:	1.0 
	* Autor					:	Ellio de Mello
	* Descripción			:	Valida la fuerza de una password!!
	* Archivo				:	password_strength.js 
	* Dependencias			:	jQuery v1.3.2      (JS) 

	---------------------------------------------------------------------------------------------------------- 
	* Opcion		Tipo						Descripcion 
	---------------------------------------------------------------------------------------------------------- 
	* veredictWords		(object list)			Lista de palabras y class (css) a aplicar según la fuerza la pwd.
					{
						text: "TEXTO",			La palabra a mostrar (fuerza de la contraseña)
						className: "CLASE"		La clase a aplicar
					},
	---------------------------------------------------------------------------------------------------------- 
	*
	* Sintaxis
	* ---------
	* HTML		:	<input id="demo" type="text" />
	* JS		:	$(selector).inconcert_password_strength([{opc1: "val1", .., .., ..}]);

	* Ejemplos
	* ---------
	*	1)
	*	HTML	:	<input id="demo" type="text" />
					<input id="id2" type="text" />
	*	JS		:	$("#id1").inconcert_password_strength(); 
					$("#id2").inconcert_password_strength(); 
	*	2)
	*	HTML	:	<input id="id1" type="text" />
	*	JS		:	$("#id1").inconcert_password_strength({
						veredictWords: {
							 empty: {text: "Empty", className: "emptyClass"},
							 very_weak: {text: "Very Weak", className: "emptyVeryWeak"},
							 weak: {text: "Weak", className: "emptyWeak"},
							 mediocre: {text: "Mediocre", className: "emptyMediocre"},
							 strong: {text: "Strong", className: "emptyStrong"},
							 stronger: {text: "Stronger", className: "emptyStronger"}
						}
					}); 
	*
*/

/* ************************************************************
 * 
 * http://www.geekwisdom.com/dyn/passwdmeter
 * 
 * 
 * 

Password Strength Factors and Weightings

password length:
level 0 (3 point): less than 4 characters
level 1 (6 points): between 5 and 7 characters
level 2 (12 points): between 8 and 15 characters
level 3 (18 points): 16 or more characters

letters:
level 0 (0 points): no letters
level 1 (5 points): all letters are lower case
level 2 (7 points): letters are mixed case

numbers:
level 0 (0 points): no numbers exist
level 1 (5 points): one number exists
level 1 (7 points): 3 or more numbers exists

special characters:
level 0 (0 points): no special characters
level 1 (5 points): one special character exists
level 2 (10 points): more than one special character exists

combinatons:
level 0 (1 points): letters and numbers exist
level 1 (1 points): mixed case letters
level 1 (2 points): letters, numbers and special characters 
                    exist
level 1 (2 points): mixed case letters, numbers and special 
                    characters exist


NOTE: Because I suck at regex the code might need work
      
NOTE: Instead of putting out all the logging information,
      the score, and the verdict it would be nicer to stretch
      a graphic as a method of presenting a visual strength
      guage.

************************************************************ */

(function($) {
	$.fn.inconcert_password_strength = function(user_options) {

		//opciones x default del tooltip
		var default_options = {
			veredictWords: {
				 empty: {
				 	text: "Empty",
					className: "emptyClass"
				 },
				 very_weak: {
				 	text: "Very Weak",
					className: "veryWeakClass"
				 },
				 weak: {
				 	text: "Weak",
					className: "weakClass"
				 },
				 mediocre: {
				 	text: "Mediocre",
					className: "mediocreClass"
				 },
				 strong: {
				 	text: "Strong",
					className: "strongClass"
				 },
				 stronger: {
				 	text: "Stronger",
					className: "strongerClass"
				 }
			}
		};
		var options = $.extend(default_options , user_options);

		var testPassword = function(passwd) {
			var score = 0;
			var veredict = "weak";
			var log = "";

			// PASSWORD LENGTH
			if (passwd.length>0 && passwd.length<5) { // length 4 or less
				score = (score+3);
				log = log + "3 points for length (" + passwd.length + ")\n";
			}
			else if (passwd.length>4 && passwd.length<8) { // length between 5 and 7
				score = (score+6);
				log = log + "6 points for length (" + passwd.length + ")\n";
			}
			else if (passwd.length>7 && passwd.length<16) {// length between 8 and 15
				score = (score+12);
				log = log + "12 points for length (" + passwd.length + ")\n";
			}
			else if (passwd.length>15){  // length 16 or more
				score = (score+18);
				log = log + "18 point for length (" + passwd.length + ")\n";
			}

			// LETTERS (Not exactly implemented as dictacted above because of my limited understanding of Regex)
			if (passwd.match(/[a-z]/)){ // [verified] at least one lower case letter
				score = (score+1);
				log = log + "1 point for at least one lower case char\n";
			}

			if (passwd.match(/[A-Z]/)) {                              // [verified] at least one upper case letter
				score = (score+5);
				log = log + "5 points for at least one upper case char\n";
			}

			// NUMBERS
			if (passwd.match(/\d+/)) {                                // [verified] at least one number
				score = (score+5);
				log = log + "5 points for at least one number\n";
			}

			if (passwd.match(/(.*[0-9].*[0-9].*[0-9])/)) {            // [verified] at least three numbers
				score = (score+5);
				log = log + "5 points for at least three numbers\n";
			}

			// SPECIAL CHAR
			if (passwd.match(/.[!,@,#,$,%,^,&,*,?,_,~]/)) {           // [verified] at least one special character
				score = (score+5);
				log = log + "5 points for at least one special char\n";
			}

										 // [verified] at least two special characters
			if (passwd.match(/(.*[!,@,#,$,%,^,&,*,?,_,~].*[!,@,#,$,%,^,&,*,?,_,~])/)) {
				score = (score+5);
				log = log + "5 points for at least two special chars\n";
			}

			// COMBOS
			if (passwd.match(/([a-z].*[A-Z])|([A-Z].*[a-z])/)) {      // [verified] both upper and lower case
				score = (score+2);
				log = log + "2 combo points for upper and lower letters\n";
			}

			if (passwd.match(/([a-zA-Z])/) && passwd.match(/([0-9])/)) { // [verified] both letters and numbers
				score = (score+2);
				log = log + "2 combo points for letters and numbers\n";
			}

										// [verified] letters, numbers, and special characters
			if (passwd.match(/([a-zA-Z0-9].*[!,@,#,$,%,^,&,*,?,_,~])|([!,@,#,$,%,^,&,*,?,_,~].*[a-zA-Z0-9])/)) {
				score = (score+2);
				log = log + "2 combo points for letters, numbers and special chars\n";
			}

			if (score === 0) {
				 veredict = "empty";
			}
			else if(score < 16) {
				veredict = "very_weak";
			}
			else if (score > 15 && score < 25) {
				veredict = "weak";
			}
			else if (score > 24 && score < 35) {
				veredict = "mediocre";
			}
			else if (score > 34 && score < 45) {
				veredict = "strong";
			}
			else {
				veredict = "stronger";
			}

			return {
				score: score,
				veredict: veredict,
				log: log
			};
		};

		var show = function() {
			var test = testPassword($(this).val());
			var veredict = options.veredictWords[test.veredict] ? options.veredictWords[test.veredict] : test.veredict;
			var strengthPassordId = $(this).attr("strengthPwdId");
			$("#" + strengthPassordId).removeClass().addClass(veredict.className).text(veredict.text).show();
		};

		var initilizeControl = function(self) {
			var strengthPassordId = self.attr('id') + 'StrengthPasswordValidation';
			self.attr("strengthPwdId", strengthPassordId);
			self.parent().append('<span id="' + strengthPassordId + '" class="clase" style="display:none;color:#FF0000;"></span>');
			//--------------------------------------------------------
			//handlers que manipulan el 
			self.keyup(show).bind("paste", null, show);
		};

		if (this.selector) {
			initilizeControl($(this.selector));
		} else {
			this.each(function(i, control) {
				initilizeControl($(control));
			});
		}
	};
})(jQuery); 
