local format = string.format
local assert = assert
local type = type
local ipairs = ipairs

module((...))

local function Field(field)
	return {name = field[1], type = field[2]}
end

local requiredFields = {
	Field{"pageName", "string"},
	Field{"permissionBuiltIn", "boolean"},
	Field{"applicationActionId", "string"}	
}
local requiredGuiFields = {
	Field{"classNameImageTitle", "string"}
}

local function RequiredField(field)
	return format("Field '%s' is required and must be of type '%s'", field.name, field.type)
end

--[=[
Controla que la vista defina los campos requeridos y con el tipo correcto
--]=]
function Define(view)
	assert(type(view) == "table", "Define must be called with a table")
	assert(type(view.GUI) == "table", "View must have a 'GUI' field")
	
	for _, field in ipairs(requiredFields) do
		assert(type(view[field.name]) == field.type, RequiredField(field))
	end
	
	local GUI = view.GUI
	for _, field in ipairs(requiredGuiFields) do
		assert(type(GUI[field.name]) == field.type, RequiredField(field))
	end
	if GUI.GetButtonLabel and GUI.classNameLinkNew then
		view.GUI.displayButtonNew = true
	else
		view.GUI.displayButtonNew = false
	end
	
	return view
end
