require("json")
local oo = require("loop.base")
local jsonEncoder = require("LuaOrm.Serializers.Json")
local m_paginatedManager = require("shared.view.paginatedManager")
local m_instances = nil
local m_moduleLocation = select(2, ...)

module(..., package.seeall)

assert(m_moduleLocation, "should be running under inconcertWebHandler")

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
local function GetViewTitle(view, applicationId)
	local applicationActionId = view.applicationActionId
	if view.GUI.GetViewTitle then
		return view.GUI.GetViewTitle()
	end
	local model = require("apps." .. applicationId .. ".model"):GetModels()
	local applicationAction = model.ApplicationAction:Find{Id = applicationActionId, ApplicationId = applicationId}
	if not applicationAction then
		error("Couldn't find view ".. applicationActionId)
	end
	view.GUI.GetViewTitle = function()
		local description = applicationAction.Description
		return _T(description)
	end
	return view.GUI.GetViewTitle()
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
function new(applicationName, parameters)
	_M.m_instances = _M.m_instances or {}
	
	if _M.m_instances[applicationName] then
		return _M.m_instances[applicationName]
	end

	local newInstance = {
		applicationName = applicationName,
		GetGridStructure = _M.GetGridStructure,
		FilterExecute = _M.FilterExecute
	}
	
	-- validar los parmetros obligatorios que me tienen que pasar
	assert(parameters.viewDefinitions)
	
	-- wrappers para usarse desde el dispatcher Orbit
	newInstance.GetInitialDataGrid = function(...)
		return _M.GetInitialDataGrid(newInstance, ...)
	end
	newInstance.CanOpen = _M.CanOpen
	newInstance.Open = function(...)
		return _M.Open(newInstance, ...)
	end
	newInstance.ApplyFilter = function(...)
		return _M.ApplyFilter(newInstance, ...)
	end
	newInstance.SaveFilter = function(...)
		return _M.SaveFilter(newInstance, ...)
	end
	newInstance.GetQueryCondition = function(...)
		return _M.GetQueryCondition(newInstance, ...)
	end

	for key, object in pairs(parameters) do
		newInstance[key] = object
	end
	
	_M.m_instances[applicationName] = newInstance
	
	return newInstance
end

--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
function GetGridStructure(self, viewName, loadDynamicFilterData)
	local applicationId = self.applicationName
	local userId = inconcert.authUser.userName
	local vcc = inconcert.authUser.virtualContactCenter
	local groupId = inconcert.authUser.userGroup
	local translator = inconcert.translators[applicationId]
	
	--seteo la defincion de datos dinamicos
	if (loadDynamicFilterData == 1) then
		self.filterMng:SetDynamicData(self.dynamicFilterData.GetDefinitionDynamicData())
	end
	-- obtengo el json del filtro
	local jsonFilter = self.filterMng:Serialize(1, applicationId, userId, vcc, viewName, groupId)
	--obtengo el json de palabras traducidas
	local jsonTranslate = json.encode({
		captionPages = _T("Page %1 of %2"),
		gotoPage = _T("Goto Page")
	})
	-- obtengo objeto other de las columnas, las columnas de la grilla princiapal y toda la subgrilla
	local customOtherObject, columnsGrid, subGrid = self.viewDefinitions[viewName].gridStructure(translator)
	-- obtengo el json con otros datos
	local otherObject = customOtherObject or {
		rowsForPage = 10, 
		maxNumberOfPage = 10
	}
	-- json columnas y subgrilla
	local jsonOther = json.encode(otherObject)
	local jsonColumnsGrid = ''
	local jsonSubGrid = ''
	local jsonGenericEvents = ''
	
	if (columnsGrid) then
		jsonColumnsGrid = ', ' .. '"columns": ' .. json.encode(columnsGrid)
	end
 	if (subGrid) then
		jsonSubGrid = ', ' .. '"subGrid": ' .. json.encode(subGrid)
	end
--	jsonGenericEvents = '"events": {onHeaderCellClick: "otherEvents.onHeaderCellClick.orderBy"}'
	jsonGenericEvents = '"events": {onHeaderCellClick: "gridEvents.orderBy"}'	
--	jsonGenericEvents = '"events": ' .. json.encode({
--		{onHeaderCellClick = "otherEvents.onHeaderCellClick.orderBy"}
--	})
	local jsonGrid = '{' ..
		'"grid": {' ..
			'"translate": ' .. jsonTranslate .. ', ' ..
			'"other": ' .. jsonOther .. ', ' ..
			jsonGenericEvents .. ', ' ..			
			'"filter": ' .. jsonFilter ..
			jsonColumnsGrid ..
			jsonSubGrid ..
		'}' ..
	'}'
	
	return jsonGrid, otherObject
end

--//////////////////////////////////////////////////////////////////////////////////
--
function FilterExecute(self, pageId, numberPage, rowsPerPage, maxNumberOfPage,
						forceQuery, filterTable, permissionInCondition, serializeDepth)
	local success = true
	m_paginatedManager:CreateView(pageId)
	if ((m_paginatedManager.ExistsView(pageId) == false) 
			or (forceQuery == 1)
			or (filterTable ~= nil)) then
			
--		LogInfo('  Ejecuto Query porque: o es la primera vez que se cargo la pagina, o se ha realizado F5 en el Browser, o se ha presionado el boton \'Apply\' del filtro')
		local filterId
		if (filterTable) then
			filterId = filterTable.__filterType or filterTable.filterType
			filterId = filterId or pageId .. '_ALL'
		else
			filterId = pageId .. '_ALL'
		end
		LogDebug('filterId = ' .. filterId)
		if (rowsPerPage) then
			m_paginatedManager:SetRowsPerPageInView(pageId, rowsPerPage)
		else
			rowsPerPage = m_paginatedManager:GetRowsPerPageInView(pageId)
		end
		success = m_paginatedManager:ExecuteQuery(pageId, filterId, filterTable, permissionInCondition, self.permission, self.model, self.queries)
	else
		LogInfo('  No ejecuto el query pido la Pagina, ya que se ha presionado un boton del paginado.')
		if not (rowsPerPage) then
			rowsPerPage = m_paginatedManager:GetRowsPerPageInView(pageId)
		end
	end
	local pagedRows = ''
	local numberOfPagesResulting = 0
	local numberOfRowsByLuaORM = 0
	local fitlerOnFilter = false
	if (success) then
		pagedRows = m_paginatedManager:GetPageInView(pageId, numberPage, self.model, 
			m_paginatedManager.GetLazy(pageId, self.queries))
		fitlerOnFilter, pagedRows = m_paginatedManager.FilterOnFilter(pageId, self.queries, pagedRows)
		numberOfPagesResulting = m_paginatedManager:GetNumberOfPagesInView(pageId, self.model)
		numberOfRowsByLuaORM = numberOfPagesResulting * (rowsPerPage or 1) -- calculo la cantidad de registros resultantes en el query
--		LogInfo('  Se va a solicitar la Pagina N ' .. tostring(numberPage))
--		LogInfo('  Nmero de paginas resultantes  = ' .. tostring(numberOfPagesResulting))
--		LogInfo('  Numero de rows resultantes en el query = ' .. numberOfRowsByLuaORM)
	end
	--encodeo
	local jsonPagedRows = ''
	if (pagedRows) then
		if not(fitlerOnFilter) then
			local encoder = oo.new(jsonEncoder)
			serializeDepth = serializeDepth or m_paginatedManager.GetSerializeDepth(pageId, self.queries)
			jsonPagedRows = encoder:SerializeList(pagedRows, false, serializeDepth, 
				m_paginatedManager.GetDefinitionRelationsShip(pageId, self.queries), 
				m_paginatedManager.GetIncludedAttributes(pageId, self.queries))
			else
			jsonPagedRows = json.encode(pagedRows)
		end
	end
	local jsonDataPage = '{' ..
		'"__config": {' ..
			'"numberOfRecords": ' .. numberOfRowsByLuaORM .. ', ' ..
			'"numberOfPages": ' .. numberOfPagesResulting ..
		'},' ..
		'"0": ' .. jsonPagedRows ..
	'}'
	return string.gsub(jsonDataPage, "</script>", "'</s' + 'cript>'")
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetInitialDataGrid(self, viewName, loadDynamicFilterData, permissionInQuery)
	LogDebug("Get Initial Data Grid - Start 1")
	local jsonStructureGrid, definitionGrid = self:GetGridStructure(viewName, loadDynamicFilterData)
	LogDebug("Get Initial Data Grid - after get structure")
	local jsonDataGrid = self:FilterExecute(viewName, 1, definitionGrid.rowsForPage, 
                                            definitionGrid.maxNumberOfPage, 1, 
                                            self.filterMng:GetDefaultFilter(),
                                            permissionInQuery)
	LogDebug("Get Initial Data Grid - after executed filter")
	return jsonStructureGrid, jsonDataGrid
end

--////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- API administration
--////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function CanOpen(self, viewname)
	local view = string.upper(viewname)
	if not self.viewDefinitions[view] then
		return false, "view does not exist"
	end
	if self.permissionManager:HasPermissionToAction(self.viewDefinitions[view].applicationActionId) then
		return true
	end
	return false, "access is forbidden"
end

--////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function Open(self, web, viewName)
	local view = string.upper(viewName)
	if not (self.viewDefinitions[view]) then
		return self.parentModule.not_found(web)
	end
	local hasPermission = self.permissionManager:HasPermissionToAction(self.viewDefinitions[view].applicationActionId)
	if not (hasPermission) then
		return self.parentModule.forbidden(web)
	end
	local definitionView = require("apps." .. self.applicationName .. ".view.definitions").Get()[view]
	local env = {
		content = function()
			local url = self.viewDefinitions[view].pageName
			local captures = {
				applicationName = self.applicationName,
				viewName = view,
				definitionView = definitionView
			}
			return self.htmlRender:IncludeLP(url, captures)
		end,
		Title = GetViewTitle(definitionView, self.applicationName),
		ImageTitleClass = definitionView.GUI.classNameImageTitle or '',
		Navigation_DefaultAction = self.viewDefinitions[view].applicationActionId
	}
	self.parentModule.response.DontCache(web)
	return self.htmlRender:Render(web, env)
end

--////////////////////////////////////////////////////////////////////////////////////////////////////////
--
function ApplyFilter(self, web)
	LogDebug(" ============================================================================")
	LogInfo(" Parmetros posteados desde el cliente")
	LogDebug(" ============================================================================")
	-- Datos recibidos desde el cliente como parametros
	-- Id de la pagina que ha realizado el Request
	local pageId
	if (web.input.pageId) then
		pageId = web.input.pageId
	end
	LogInfo('  Id Pagina = ' .. tostring(pageId))
	-- Nmero de pagina solicitada
	local numberPage = 1
	if (web.input.numberPage) then
		numberPage = tonumber(web.input.numberPage)
	end
	LogInfo('  Numero de pagina = ' .. tostring(numberPage))
	-- Cantidad de filas por paginas que va a mostrar la grilla
	local rowsPerPage
	if (web.input.rowsPerPage) then
		rowsPerPage = tonumber(web.input.rowsPerPage)
	end
	LogInfo('  Filas por pagina = ' .. tostring(rowsPerPage))
	-- cantidad de paginas a mostarar en el paginado
	local maxNumberOfPage
	if (web.input.maxNumberOfPage) then
		maxNumberOfPage = tonumber(web.input.maxNumberOfPage)
	end
	LogInfo('  Cantidad de paginas a mostrar por paginado = ' .. tostring(maxNumberOfPage))
	-- Si fuerza la consulta (Es utilizado para el F5 en el Navegador) se debe ejecutar la consulta siempre
	local forceQuery
	if (web.input.forceQuery) then
		forceQuery = tonumber(web.input.forceQuery)
	end
--	LogInfo('  Fuerza la ejecucion del query = ' .. tostring(forceQuery))
	-- Si ha presionado el boton Apply del filtro tienen que venir los datos
	local filterTable
--	LogInfo('  Filtro a procesar (json) = ' .. tostring(web.input["filter"]))
	if (web.input.filter) then
		filterTable = json.decode(web.input["filter"])
--		if (filterTable) then
--			for k, v in pairs(filterTable) do
--				LogDebug('	-json['.. k ..'] = ' .. tostring(v))
--			end
--		end
	end
	-- Si se necesita agregar a la condiciones de permisos a condicion where
	local permissionInCondition = true
	if (web.input.permissionInQuery) then
		if(tonumber(web.input.permissionInQuery) == 0) then
			permissionInCondition = false
		end
	end
--	LogInfo('  Agrega permiso en where = ' .. tostring(permissionInCondition))
	-- Profundidad al serializar.
	local serializeDepth
	if (web.input.serializedepth) then
		serializeDepth = tonumber(web.input.serializedepth)
	end
--	LogInfo('  Profundiad al seraializar = ' .. tostring(serializeDepth))
--	LogDebug("============================================================================")
	-- ejecuto filtro
	return self:FilterExecute(pageId, numberPage, rowsPerPage, maxNumberOfPage,
								forceQuery, filterTable, permissionInCondition, serializeDepth)
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- Funcion encargada de derivar las solicitud de Guardar el filtro, ya sea para Actualizar o para Ingresar un filtro Nuevo.
function SaveFilter(self, web)
	local filter = json.decode(web.input["filter"])
	self.filterMng:Save(filter.dataFilter)
	
	return ""
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetQueryCondition(self, filter, permissionInCondition)
--m_paginatedManager:ExecuteQuery(pageId, filterId, filterTable, permissionInCondition, self.permission, self.model, self.queries)
	return m_paginatedManager:BuildQueryCondition(filter.__filterType, filter, permissionInCondition, self.permission, self.model, self.queries) 
end
