local oo = require("loop.base")
require("LuaOrm.Mapper")
require("LuaOrm.Entity")
require("LuaOrm.Model")
require("LuaOrm.DBMS.DataSource")
require("LuaOrm.Associations.Associations")
require("LuaOrm.Associations.AssociationArray")
require("LuaOrm.Serializers.Json") 
require("LuaOrm.Date")
local PagedRequest = require "LuaOrm.PagedRequest"
local oo = require("loop.base")
local rpairs = LuaOrm.Associations.AssociationArray.Iterator
local m_permissionQueries = require("shared.view.permissionQueries")
local m_moduleLocation = select(2, ...)

module(..., package.seeall) 

LogInfo("Inicializo Modulo 'paginatedManager para administration'")

local function GetGroupQuery(groupId, moduleQueries)
	if not(moduleQueries) then
		return false
	end
    return moduleQueries.GetGroupQueries(groupId)
end

--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function FilterOnFilter(groupId, moduleQueries, pagedRows)
	local groupQuery = GetGroupQuery(groupId, moduleQueries)
	if not groupQuery or not groupQuery.FilterOnFilter then
		return false, pagedRows
	end
	return groupQuery.FilterOnFilter(pagedRows)
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetDefinitionRelationsShip(groupId, moduleQueries)
	local groupQuery = GetGroupQuery(groupId, moduleQueries)
	if not groupQuery or not groupQuery.GetRelationsShip then
		return
	end
	return groupQuery.GetRelationsShip()
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetSerializeDepth(groupId, moduleQueries)
	local groupQuery = GetGroupQuery(groupId, moduleQueries)
	if not groupQuery or not groupQuery.GetSerializeDepth then
		return
	end
	return groupQuery.GetSerializeDepth()
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetLazy(groupId, moduleQueries)
	local groupQuery = GetGroupQuery(groupId, moduleQueries)
	if not groupQuery or not groupQuery.GetLazy then
		return
	end
	return groupQuery.GetLazy()
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetIncludedAttributes(groupId, moduleQueries)
	local groupQuery = GetGroupQuery(groupId, moduleQueries)
	if not groupQuery or not groupQuery.GetIncludedAttributes then
		return
	end
	return groupQuery.GetIncludedAttributes()
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function CreateView(self, pageId)
    -- tabla que contiene los paginados por vista dentro de la misma session de usuario.
    inconcert.session.listView = inconcert.session.listView or {}    
	if (inconcert.session.listView[pageId] == nil) then
		inconcert.session.listView[pageId] = {
			serializedPagedRows = nil,
			currentRowsPerPage = nil,
			tableName = nil
		}
	end		
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function ExistsView(pageId)
    local valueToReturn = false
    if (inconcert.session.listView[pageId]) then
        valueToReturn = true
    end      
	return valueToReturn
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function SetRowsPerPageInView(self, pageId, rowsPerPage)
	if (inconcert.session.listView[pageId] ~= nil) then
		inconcert.session.listView[pageId].currentRowsPerPage = rowsPerPage
	end		
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetRowsPerPageInView(self, pageId)
	local valueToReturn
	if (inconcert.session.listView[pageId] ~= nil) then
		valueToReturn = inconcert.session.listView[pageId].currentRowsPerPage
	end	
	return valueToReturn
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetPageInView(self, pageId, numberPage, moduleModel, lazy)
	local valueToReturn
	if (inconcert.session.listView[pageId] ~= nil) then
	    local pagedRows = self:DeserializePagedRows(pageId, moduleModel)
		valueToReturn = pagedRows:GetPage(numberPage, lazy)
	end	
	return valueToReturn
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function GetNumberOfPagesInView(self, pageId, moduleModel)
	local valueToReturn
	if (inconcert.session.listView[pageId] ~= nil) then
		local pagedRows = self:DeserializePagedRows(pageId, moduleModel)
		valueToReturn =  pagedRows:GetNumberOfPages()
	end	
	return valueToReturn
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function SerializePagedRows(self, pageId, pagedRows, tableName)
    if (inconcert.session.listView[pageId] ~= nil and pagedRows ~= nil) then
	    inconcert.session.listView[pageId].serializedPagedRows = pagedRows:Serialize()
		inconcert.session.listView[pageId].tableName = tableName
	end
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function DeserializePagedRows(self, pageId, moduleModel) 
    local listOfModelTables = moduleModel:GetModels()	
    local pagedRows = oo.new(PagedRequest)	
    local tableName = inconcert.session.listView[pageId].tableName 
	pagedRows:Deserialize(inconcert.session.listView[pageId].serializedPagedRows, 
                            listOfModelTables[tableName].dataSource, 
                            listOfModelTables[tableName])

    return pagedRows
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function ExecuteQuery(self, pageId, filterId, filterDataTable, permissionInCondition, modulePermission, moduleModel, moduleQueries)
	local rowsPerPage = inconcert.session.listView[pageId].currentRowsPerPage
	local success, tableName, where, whereValues = self:BuildQueryCondition(filterId, filterDataTable, permissionInCondition, modulePermission, moduleModel, moduleQueries)

    if (success) then
        local listOfModelTables = moduleModel:GetModels()
 --       LogDebug('ExecuteQuery - where = ' .. tostring(where))
 --       LogDebug('ExecuteQuery - whereValues = ' .. tostring(whereValues))
        
        local pagedRows = listOfModelTables[tableName]:FindAllPaged(where, whereValues, rowsPerPage)
        self:SerializePagedRows(pageId, pagedRows, tableName)
    end
	
	return success
end
--///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-- 
function BuildQueryCondition(self, filterId, filterDataTable, permissionInCondition, modulePermission, moduleModel, moduleQueries) 
	local query = moduleQueries.GetQuery(filterId)
	if not(query) then
	    error('No existe query asociado al filtro \'' .. tostring(filterId) .. '\'')
	    return false
	end	
	local permissionGroup = modulePermission.GetGroupPermission(inconcert.authUser.userGroup:upper())
	if not(permissionGroup) then
	    error('No existe definicin de permisos asociado para el grupo \'' .. inconcert.authUser.userGroup .. '\'')
	    return false
	end
	
	local whereCondition = m_permissionQueries:GetWhereConditions(permissionGroup, query, filterDataTable, permissionInCondition)
	local whereValues = m_permissionQueries:GetWhereValues(permissionGroup, query, filterDataTable, permissionInCondition)
	
	return true, query.tableName, whereCondition, whereValues
end


