local oo = require("loop.base")
require("LuaOrm.Mapper")
require("LuaOrm.Entity")
require("LuaOrm.Model")
require("LuaOrm.DBMS.DataSource")
require("LuaOrm.Associations.Associations")
require("LuaOrm.Associations.AssociationArray")
require("LuaOrm.Serializers.Json")
require("LuaOrm.Date")
require("inconcert.translators")

local rpairs = LuaOrm.Associations.AssociationArray.Iterator
local LogDebug = LogDebug
local LogInfo = LogInfo
local m_moduleLocation = select(2, ...)
module(..., package.seeall) 

LogInfo("Inicializo Modulo 'permissionQueries'")

--//////////////////////////////////////////////////////////////////////////////////////////////////
--
function GetWhereConditions(self, permissionGroup, queriesObject, filterDataTable, permissionInCondition)
	local whereCondition = ''
	local whereOfPermission = ''
	local andString = ' AND '
	local orString = ' OR '
	local ormSymbol = ' ?'
	local function GetFieldName(field)
		return field.fieldAlias or field.fieldName
	end
	if (queriesObject) then
		whereCondition = queriesObject.where or ''
		if (type(whereCondition) == "table") then
			local condition = ''
			for _,field in ipairs(whereCondition) do
				local temporalCondition = false
				if (filterDataTable) then
				    local fieldValue = filterDataTable[field.fieldName]
					if (fieldValue) then
						if (type(fieldValue) == "table") then
							if (fieldValue.selectedValue == '__UNSELECTED') then
								if (#fieldValue.values > 0) then
									if (field.mustIncludeThoseNotSelected == nil or field.mustIncludeThoseNotSelected == true) then
										temporalCondition = ''
										local whereWithOr = '('
										for index, val in ipairs(fieldValue.values) do
											if (index > 1) then
												whereWithOr = whereWithOr .. orString
											end
											whereWithOr = whereWithOr .. GetFieldName(field) .. ' ' .. field.operator .. ormSymbol
										end
										temporalCondition = whereWithOr .. ')'
									end
								else
									if (field.mustIncludeThoseNotSelected == true) then
										temporalCondition = GetFieldName(field) .. ' ' .. field.operator .. ormSymbol
									end
								end
							else
								temporalCondition = GetFieldName(field) .. ' ' .. field.operator .. ormSymbol
							end
						else
							if (string.len(fieldValue) > 0) then
								if (field.typeof and field.typeof == "date") then
									if (field.operator == "=") then
										temporalCondition = '(' .. GetFieldName(field) .. " >=" .. ormSymbol .. andString .. GetFieldName(field) .. " <=" .. ormSymbol .. ')'
									else
										temporalCondition = GetFieldName(field) .. ' ' .. field.operator .. ormSymbol
									end
								else
									temporalCondition = GetFieldName(field) .. ' ' .. field.operator .. ormSymbol
								end
							end
						end
					else
						temporalCondition = GetFieldName(field) .. ' ' .. field.operator .. ormSymbol
					end
				end
				if (temporalCondition) then
					if (string.len(condition) > 0) then
						condition = condition .. andString
					end
					condition = condition .. temporalCondition
				end
			end
			whereCondition = condition
		end
		if (queriesObject.GetDefaultCondition) then
			local defaultCondition = queriesObject:GetDefaultCondition()
			if (string.len(defaultCondition) > 0) then
				if (string.len(whereCondition) > 0) then
					defaultCondition = defaultCondition .. andString
				end
				whereCondition = defaultCondition .. whereCondition
			end
		end
	end

	if (permissionGroup and permissionInCondition) then
		local tableOfPermission = permissionGroup.where
		if (tableOfPermission == nil) then
			return whereCondition
		end
		for index, condition in ipairs(tableOfPermission) do
			local fieldValue
			if (filterDataTable) then
				fieldValue = filterDataTable[condition.fieldName]
			end
			local value = fieldValue
			if (type(fieldValue) == "table") then
				value = fieldValue.selectedValue
			end
			if (value == '__UNSELECTED') then
				LogDebug("condition.unSelected = " .. tostring(condition.unSelected))
				if (condition.unSelected == nil or condition.unSelected == 'ALL') then
					if (index > 1) then
						whereOfPermission = whereOfPermission .. andString
					end
					local whereWithOr = '('
					for index, val in ipairs(fieldValue.values) do
						if (index > 1) then
							whereWithOr = whereWithOr .. orString
						end
						whereWithOr = whereWithOr .. condition.fieldString
					end
					whereWithOr = whereWithOr .. ')'
					whereOfPermission = whereOfPermission .. whereWithOr
				elseif (condition.unSelected ~= 'NONE') then
					return nil
				end
			else
				if value ~= nil or (condition.isNotAddedIfItIsNull == nil or condition.isNotAddedIfItIsNull == false) then
					if (index > 1) then
						whereOfPermission = whereOfPermission .. andString
					end
					whereOfPermission = whereOfPermission .. condition.fieldString
				end
			end
		end
	end

	if ((string.len(whereOfPermission) > 0) and (string.len(whereCondition) > 0)) then
		whereOfPermission = andString .. whereOfPermission
	end

	return whereCondition .. whereOfPermission
	end
--//////////////////////////////////////////////////////////////////////////////////////////////////
--
function GetWhereValues(self, permissionGroup, queriesObject, filterDataTable, permissionInCondition)
	local whereValues = {}
	local tableOfPermission = nil
	local whereValuesOfDefinition = nil
	if (queriesObject ~= nil and queriesObject.GetDefaultValues ~= nil) then
		whereValuesOfDefinition = queriesObject:GetDefaultValues()
	end
	if (whereValuesOfDefinition ~= nil) then
		whereValues = whereValuesOfDefinition
		if (filterDataTable ~= nil) then 
			if (filterDataTable.__orderBy) then
				local order = 'asc'
				if (filterDataTable.__orderBy.ascendent == false) then
					order = 'desc'
				end
				whereValues.order = filterDataTable.__orderBy.field .. ' ' .. order
			end
			local index = #(whereValues)
			if (type(queriesObject.where) == "table") then
				for _,field in ipairs(queriesObject.where) do
					local hasAddCondition = false
					local fieldValue = filterDataTable[field.fieldName]
					if (fieldValue) then
						if (type(fieldValue) == "table") then
							if (fieldValue.selectedValue == '__UNSELECTED') then 
								if (#fieldValue.values > 0) then 
									if (field.mustIncludeThoseNotSelected == nil or field.mustIncludeThoseNotSelected == true) then
										for _, val in ipairs(fieldValue.values) do
											index = index + 1
											whereValues[index] = val
										end
									end
								else
									if (field.mustIncludeThoseNotSelected == true) then
										hasAddCondition = true
										fieldValue = '__none'
									end
								end
							else
								hasAddCondition = true
								fieldValue = fieldValue.selectedValue
							end
						else
							if (string.len(fieldValue) > 0) then
								if (field.Calculate) then
									fieldValue = field:Calculate(fieldValue)
								elseif (field.typeof and field.typeof == "date" and (field.calculateTimeZone or field.calculateTimeZone == nil)) then
									if not field.uniqueDate then
										local timeZoneDate = LuaOrm.Date()
										fieldValue = timeZoneDate:newFromDifferentTimezone(fieldValue, inconcert.authUser.timezone):toString()
									end
								end
								hasAddCondition = true
							end
						end
					else
						if (field.GetValue) then
							hasAddCondition = true
							fieldValue = field.GetValue()
						else
							error("No existe campo asociado en la BD, ni valor por defecto para el campo '" .. tostring(field.fieldName) .. "' del filtro.")
						end
					end
					if (hasAddCondition) then
						index = index + 1
						whereValues[index] = fieldValue
						-- solo para los campos unique, ya que el valor no viene del lado del cliente para este caso.
						if (field.typeof and field.typeof == "date" and field.uniqueDate == true) then
							whereValues[index] = fieldValue .. ' 00:00:00'
							local timeZoneDate = LuaOrm.Date()
							whereValues[index] = timeZoneDate:newFromDifferentTimezone(whereValues[index], inconcert.authUser.timezone):toString()
							index = index + 1
							whereValues[index] = fieldValue .. ' 23:59:59'
							whereValues[index] = timeZoneDate:newFromDifferentTimezone(whereValues[index], inconcert.authUser.timezone):toString()
						end
					end
				end
			end
		end
	end

	if (permissionGroup and permissionInCondition) then
		tableOfPermission =  permissionGroup.where
	end
	if (tableOfPermission ~= nil) then
		local tableLength = #(whereValues)
		for _, condition in ipairs(tableOfPermission) do
			local fieldValue
			if (filterDataTable) then
				fieldValue = filterDataTable[condition.fieldName]
			end
			local value = fieldValue
			if (type(fieldValue) == "table") then
				value = fieldValue.selectedValue
			end	
			if (value == '__UNSELECTED') then
				if (condition.unSelected == nil or condition.unSelected == 'ALL') then
					for index, val in ipairs(fieldValue.values) do
						tableLength = tableLength + 1
						whereValues[tableLength] = val
					end
				elseif (condition.unSelected ~= 'NONE') then
					return nil
				end
			else
				if  value ~= nil or (condition.isNotAddedIfItIsNull == nil or condition.isNotAddedIfItIsNull == false) then
					if (value == nil) then
					  value =  ((type(condition.defaultValue) == "string") and condition.defaultValue) or condition:defaultValue(queriesObject.tableName)
					end
					tableLength = tableLength + 1
					whereValues[tableLength] = value
				end
			end
		end
	end

	return whereValues
end
